/*---------------------------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Procediments del subsistema Auxiliars
----------------------------------------------------------------------*/


-- Schema BD_POSTVENDA
 ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

 COMMIT;


-------------------------------------------------------------------------------
-- Ubicacions (Pasos)
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_Pais (
/******************************************************************************
	NOM:			Alta_Pais

	REQUERIMENT N:	13
   
	DESCRIPCI:		Dna dalta un pas al nostre sistema

	PARMETRES:

		p_Pais	Nom del Pas

******************************************************************************/
	p_Pais	IN	Paisos.Pais%TYPE)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_Pais: ' || p_Pais;

	INSERT INTO Paisos (Pais)
	VALUES (p_Pais);

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := 'OK';
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El pas ' || p_Pais || ' ja existeix';
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Pais (
/******************************************************************************
	NOM:			Modificar_Pais

	REQUERIMENT N:	14
   
	DESCRIPCI:		Permet modificar les dades associades a un pas

	PARMETRES:

		p_IdPais	Identificador del Pas
		p_Pais	Nom del Pas

******************************************************************************/
	p_IdPais	IN	Paisos.IdPais%TYPE,
	p_Pais	IN	Paisos.Pais%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdPais: ' || p_IdPais || ' p_Pais: ' || p_Pais;

	UPDATE Paisos SET Pais = p_Pais
	WHERE IdPais = p_IdPais;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el pas amb IdPais=' || p_IdPais;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El pas ' || p_Pais || ' ja existeix';
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Eliminar_Pais (
/******************************************************************************
	NOM:			Eliminar_Pais

	REQUERIMENT N:	15
   
	DESCRIPCI:		Elimina un pas del nostre sistema

	PARMETRES:

		p_IdPais	Identificador del pas

******************************************************************************/
	p_IdPais	IN	Paisos.IdPais%TYPE)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdPais: ' || p_IdPais;

	DELETE Paisos
	WHERE IdPais = p_IdPais;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el pas amb IdPais=' || p_IdPais;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			v_Res :=
				'No es pot borrar el pas '
				|| p_IdPais
				|| '. T provncies assignades';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Obtenir_Pais
/******************************************************************************
	NOM:			Obtenir_Pais

	REQUERIMENT N:	16
   
	DESCRIPCI:		Permet obtenir una relaci de pasos, en funci dalguns parmetres.
				Si especifiquem un identificador, ens retornar, si existeix, el pas
				amb aquest identificador. Si no especifiquem lidentificador (null), 
				retornar els pasos el nom dels quals contingui el nom especificat per parmetre.
				Si no sespecifica cap dels dos parmetres, ens retornar la relaci de tots els pasos

	PARMETROS:

		p_RS		Recordset amb el resultat
		p_IdPais	Identificador del pas a cercar
		p_Pais	Pas a cercar

******************************************************************************/

(   
	p_RS		IN OUT	sys_refcursor,
	p_IdPais	IN		Paisos.IdPais%TYPE:=Null,
	p_Pais	IN		Paisos.Pais%TYPE:=Null
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := 'p_RS (IN OUT), p_IdPais: ' || p_IdPais || ' p_Pais:' || p_Pais ;

	IF p_IdPais IS NULL AND p_Pais IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Paisos; 
	ELSIF p_IdPais IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Paisos WHERE IdPais = p_IdPais;
	ELSIF p_Pais IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Paisos WHERE UPPER(Pais) LIKE '%' || UPPER(p_Pais) || '%';
	END IF;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;



------------------------------------------------------------------------------------
-- Ubicacions (Provncies)
-----------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_Provincia (
/******************************************************************************
	NOM:			Alta_Provincia

	REQUERIMENT N:	17
   
	DESCRIPCI:		Dna dalta una provncia al nostre sistema

	PARMETRES:

		p_Provincia	Nom de la provncia
		p_IdPais	Identificador del pas de la provncia

******************************************************************************/
	p_Provincia	IN	Provincies.Provincia%TYPE,
	p_IdPais	IN	Provincies.IdPais%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'  p_Provincia: ' || 
		p_Provincia || 
		'p_IdPais: ' || 
		p_IdPais ;

	INSERT INTO Provincies (IdPais, Provincia)
	VALUES (p_IdPais, p_Provincia);

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := 'OK';
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'La provncia ' || p_Provincia || ' ja existeix';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix el pas amb IdPais=' || p_IdPais;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Provincia (
/******************************************************************************
	NOM:			Modificar_Provincia

	REQUERIMENT N:	18
   
	DESCRIPCI:		Permet modificar les dades associades a una provncia

	PARMETRES:

		p_IdProvincia	Identificador de la provncia
		p_Provincia		Nom de la provncia (Opcional)
					Si s NULL mant el valor anterior
		p_IdPais		Identificador del pas de la provncia (Opcional)
					Si s NULL mant el valor anterior

******************************************************************************/
	p_IdProvincia	IN	Provincies.IdProvincia%TYPE,
	p_Provincia		IN	Provincies.Provincia%TYPE := NULL,
	p_IdPais		IN	Provincies.IdPais%TYPE := NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdProvincia: '
		|| p_IdProvincia
		|| ' p_Provincia: '
		|| p_Provincia
		|| ' p_IdPais: '
		|| p_IdPais;

	IF p_Provincia IS NULL AND p_IdPais IS NULL
	THEN
		v_Res := 'Error: No ha especificat ni provncia ni pas';
	ELSE
		UPDATE Provincies
		SET Provincia = NVL (p_Provincia, Provincia), IdPais = NVL (p_IdPais, IdPais)
		WHERE IdProvincia = p_IdProvincia;

		IF SQL%NOTFOUND
		THEN
			v_Res := 'Error: No existeix la provncia amb IdProvincia=' || p_IdProvincia;
		ELSE
			v_Res := 'OK';
		END IF;
	END IF;

	-- Grabamos en el LOG de SUBRUTINAS
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'La provncia ' || p_Provincia || ' ja existeix';
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix el pas amb IdPais=' || p_IdPais;
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Eliminar_Provincia (
/******************************************************************************
	NOM:			Eliminar_Provincia

	REQUERIMENT N:	19
   
	DESCRIPCI:		Elimina una provncia del nostre sistema

	PARMETRES:

		p_IdProvincia	Identificador de la provncia

******************************************************************************/
	p_IdProvincia	IN	Provincies.IdProvincia%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdProvincia: ' || p_IdProvincia;

	DELETE Provincies WHERE IdProvincia = p_IdProvincia;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix la provncia amb IdProvincia=' || p_IdProvincia;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			v_Res := 'No es pot borrar la provncia '
				|| p_IdProvincia
				|| '. T localitats assignades';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;



CREATE OR REPLACE PROCEDURE Obtenir_Provincia
/******************************************************************************
	NOM:			Obtenir_Provincia

	REQUERIMENT N:	20
   
	DESCRIPCI:		Permet obtenir una relaci de provncies, en funci dalguns parmetres.
				Si especifiquem un identificador, ens retornar, si existeix, la provncia 
				amb aquest identificador. Si no especifiquem lidentificador (null), retornar 
				les provncies el nom de les quals contingui el nom especificat per parmetre. 
				Si no sespecifica cap dels dos parmetres, ens retornar la relaci de totes les provncies

	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdProvincia	Identificador de la provncia a cercar
		p_Provincia		Provncia a cercar

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_IdProvincia	IN		Provincies.IdProvincia%TYPE:=Null,
	p_Provincia		IN		Provincies.Provincia%TYPE:=Null
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := 'p_RS (IN OUT), p_IdProvincia: ' || p_IdProvincia || ' p_Provincia:' || p_Provincia ;

	IF p_IdProvincia IS NULL AND p_Provincia IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM vi_Provincies; 
	ELSIF p_IdProvincia IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM vi_Provincies WHERE IdProvincia = p_IdProvincia;
	ELSIF p_Provincia IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM vi_Provincies WHERE UPPER(Provincia) LIKE '%' || UPPER(p_Provincia) || '%';
	END IF;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;




------------------------------------------------------------------------------------
-- Ubicacions (Localitats)
-----------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_Localitat (
/******************************************************************************
	NOM:			Alta_Localitat

	REQUERIMENT N:	21
   
	DESCRIPCI:		Dna dalta una localitat al nostre sistema

	PARMETRES:

		p_Localitat		Nom de la localitat
		p_IdProvincia	Identificador de la provncia de la localitat

******************************************************************************/
	p_Localitat		IN	Localitats.Localitat%TYPE,
	p_IdProvincia	IN	Localitats.IdProvincia%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_Localitat: ' || 
		p_Localitat || 
		' p_IdProvincia: ' || 
		p_IdProvincia ;

	INSERT INTO Localitats (IdProvincia, Localitat)
	VALUES (p_IdProvincia, p_Localitat);

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := 'OK';
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'La localitat '
				|| p_Localitat
				|| ' ja existeix per a la provncia '
				|| p_IdProvincia;
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix la provncia amb IdProvincia=' || p_IdProvincia;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Localitat (
/******************************************************************************
	NOM:			Modificar_Localitat

	REQUERIMENT N:	22
   
	DESCRIPCI:		Permet modificar les dades associades a una localitat

	PARMETRES:

		p_IdLocalitat	Identificador de la localitat
		p_Localitat		Nom de la localitat (Opcional)
					Si es NULL mant el valor anterior
		p_IdProvincia	Identificador de la provncia de la localitat (Opcional)
					Si es NULL mant el valor anterior

******************************************************************************/
	p_IdLocalitat	IN	Localitats.IdLocalitat%TYPE,
	p_Localitat		IN	Localitats.Localitat%TYPE := NULL,
	p_IdProvincia	IN	Localitats.IdProvincia%TYPE := NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdLocalitat: '
		|| p_IdLocalitat
		|| ' p_Localitat: '
		|| p_Localitat
		|| ' p_IdProvincia: '
		|| p_IdProvincia;

	IF p_Localitat IS NULL AND p_IdProvincia IS NULL
	THEN
		v_Res := 'Error: No ha especificat ni localitat ni provncia';
	ELSE
		UPDATE Localitats
		SET Localitat = NVL (p_Localitat, Localitat), IdProvincia = NVL (p_IdProvincia, IdProvincia)
		WHERE IdLocalitat = p_IdLocalitat;

		IF SQL%NOTFOUND
		THEN
			v_Res := 'Error: No existeix la localitat amb IdLocalitat=' || p_IdLocalitat;
		ELSE
			v_Res := 'OK';
		END IF;
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res :=
				'La localitat '
				|| p_localitat
				|| ' ja existeix per a la provncia '
				|| p_IdProvincia;
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix la provncia amb IdProvincia ' || p_IdProvincia;
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Eliminar_Localitat (
/******************************************************************************
	NOM:			Eliminar_Localitat

	REQUERIMENT N:	23
   
	DESCRIPCI:		Elimina una localitat del nostre sistema

	PARMETRES:

		p_IdLocalitat	Identificador de la localitat

******************************************************************************/
	p_IdLocalitat	IN	Localitats.IdLocalitat%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdLocalitat: ' || p_IdLocalitat;

	DELETE Localitats WHERE IdLocalitat = p_IdLocalitat;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix la localitat amb IdLocalitat=' || p_IdLocalitat;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			v_Res :=
				'No es pot Eliminar la localitat '
				|| p_IdLocalitat
				|| '. T cdis postal assignats';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;



CREATE OR REPLACE PROCEDURE Obtenir_Localitat
/******************************************************************************
	NOM:			Obtenir_Localitat

	REQUERIMENT N:	24
   
	DESCRIPCI:		Permet obtenir una relaci de localitats, en funci dalguns parmetres.
				Si especifiquem un identificador, ens retornar, si existeix, la localitat 
				amb aquest identificador. Si no especifiquem lidentificador (null), retornar 
				les localitats el nom de les quals contingui el nom especificat per parmetre. 
				Si no sespecifica cap dels dos parmetres, ens retornar la relaci de totes les localitats

	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdLocalitat	IdLocalitat a cercar
		p_Localitat		Localitat a cercar

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_IdLocalitat	IN		Localitats.IdLocalitat%TYPE:=Null,
	p_Localitat		IN		Localitats.Localitat%TYPE:=Null
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_IdLocalitat: ' || p_IdLocalitat || ' p_Localitat:' || p_Localitat ;

	IF p_IdLocalitat IS NULL AND p_Localitat IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM vi_Localitats; 
	ELSIF p_IdLocalitat IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM vi_Localitats WHERE IdLocalitat = p_IdLocalitat;
	ELSIF p_Localitat IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM vi_Localitats WHERE UPPER(Localitat) LIKE '%' || UPPER(p_Localitat) || '%';
	END IF;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

-------------------------------------------------------------------------------
-- Ubicacions (Codis Postal)
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_CodiPostal (
/******************************************************************************
	NOM:			Alta_CodiPostal
   
	REQUERIMENT N:	25

	DESCRIPCI:		Dna dalta un codi postal al nostre sistema

	PARMETRES:

		p_CodiPostal	Nom del Codi Postal
		p_IdLocalitat	Identificador de la localitat del Codi Postal

******************************************************************************/
	p_CodiPostal	IN	CodisPostal.CodiPostal%TYPE,
	p_IdLocalitat	IN	CodisPostal.IdLocalitat%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_CodiPostal: ' || p_CodiPostal || ' p_IdLocalitat: ' || p_IdLocalitat;

	INSERT INTO CodisPostal (IdLocalitat, CodiPostal)
	VALUES (p_IdLocalitat, p_CodiPostal);

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := 'OK';
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El Codi Postal ' || p_CodiPostal || ' ja existeix';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix la localitat amb IdLocalitat=' || p_IdLocalitat;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_CodiPostal (
/******************************************************************************
	NOM:			Modificar_CodiPostal

	REQUERIMENT N:	26

	DESCRIPCI:		Permet modificar la localitat associada a un Codi Postal 
   
	PARMETRES:

		p_CodiPostal	Nom del Codi Postal
		p_IdLocalitat	Identificador de la localitat del Codi Postal
******************************************************************************/
	p_CodiPostal	IN	CodisPostal.CodiPostal%TYPE,
	p_IdLocalitat	IN	CodisPostal.IdLocalitat%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_CodiPostal: ' || p_CodiPostal || ' p_IdLocalitat: ' || p_IdLocalitat;

	UPDATE CodisPostal
	SET IdLocalitat = p_IdLocalitat
	WHERE CodiPostal = p_CodiPostal;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el Codi Postal ' || p_CodiPostal;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el Codi Postal ' || p_CodiPostal;
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix la localitat amb IdLocalitat=' || p_IdLocalitat;
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Eliminar_CodiPostal (
/******************************************************************************
	NOM:			Eliminar_CodiPostal

	REQUERIMENT N:	27

	DESCRIPCI:		Elimina un codi postal del nostre sistema

	PARMETRES:

		p_CodiPostal	Identificador del CodiPostal

******************************************************************************/
		p_CodiPostal	IN	CodisPostal.CodiPostal%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_CodiPostal: ' || p_CodiPostal;

	DELETE CodisPostal WHERE CodiPostal = p_CodiPostal;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el Codi Postal ' || p_CodiPostal;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			v_Res :=
				'No es pot borrar el Codi Postal '
				|| p_CodiPostal
				|| '. T clients o centres associats.';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_CodiPostal
/******************************************************************************
	NOM:			Obtenir_CodiPostal

	REQUERIMENT N:	28

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna ubicaci, 
				a partir de lidentificador de codi postal facilitat com a parmetre
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_CodiPostal	Codi Postal a cercar

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_CodiPostal	IN		CodisPostal.CodiPostal%TYPE:=NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := 'p_RS (IN OUT), p_CodiPostal: ' || p_CodiPostal;

	OPEN p_RS for
		SELECT CodiPostal, Localitat , Provincia , Pais  
		From CodisPostal CP, Localitats LO, Provincies PR, Paisos PA
		Where	CP.IdLocalitat = LO.IdLocalitat AND
			LO.IdProvincia = PR.IdProvincia AND             
			PR.IdPais = PA.IdPais AND 
			CASE WHEN p_CodiPostal IS NULL THEN 1 ELSE
			CASE WHEN CP.CodiPostal = p_CodiPostal THEN 1 ELSE 0 END END = 1;
    
	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/


-------------------------------------------------------------------------------
--  Log Execuci
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Obtenir_LogExecucio
/******************************************************************************
	NOM:			Obtenir_LogExecucio

	REQUERIMENT N:	29

	DESCRIPCI:		Permet obtenir tota la informaci complerta emmagatzemada 
				a la taula de logs dexecuci. 
				Podrem facilitar per parmetre la data dinici i la data final, 
				per tal dacotar la informaci retornada 
   
	PARMETRES:

		p_RS		Recordset amb el resultat
		p_Desde	Data d'inici
		p_Fins	Data fi
******************************************************************************/

(   
	p_RS		IN OUT	sys_refcursor,
	p_Desde	IN		LogExecucio.Inici%TYPE:=NULL,
	p_Fins	IN		LogExecucio.Fi%TYPE:=NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_RS (IN OUT),  p_Desde: ' || p_Desde || 'p_Desde: ' || p_Fins;

  
	OPEN p_RS FOR
		SELECT  * FROM LogExecucio  
		WHERE CASE WHEN p_Desde IS NULL THEN 1 ELSE 
			CASE WHEN Inici >= p_Desde THEN 1 ELSE 0 END END = 1 AND 
			CASE WHEN p_Fins Is Null Then 1 ELSE 
			CASE WHEN Inici <= p_Fins Then 1 ELSE 0 END END = 1 
		ORDER BY Inici DESC;
     
	v_Res:= 'OK';

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

      DBMS_OUTPUT.put_line (v_Res);
      v_Fi := SYSDATE;
      v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

COMMIT;

