/*------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Procediments del subsistema Clients
--------------------------------------------------*/


-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

COMMIT;


-------------------------------------------------------------------------------
-- Clients
-------------------------------------------------------------------------------


CREATE OR REPLACE PROCEDURE Alta_Client (
/******************************************************************************
	NOM:			Alta_Client

	REQUERIMENT: 	30

	DESCRIPCI:		Dna d'alta un client al nostre sistema

	PARMETRES:

		p_Nom			Nom del client
		p_Cognoms		Cognoms del client
		p_DNI			DNI del client
		p_CodiPostal	Codi Postal del client
		p_Adreca		Adrea del client
		p_Telefon		Telfon del client
		p_Mobil		Telfon mbil del client
		p_Email		Email del client
		p_DataNaixement	Data de Naixement del client
		p_Sexe		Sexe del client (H = Home, D = Dona)
		p_DataAlta		Data d'alta del client (per defecte, la data del sistema)

******************************************************************************/
	p_Nom			IN	Clients.Nom%TYPE,
	p_Cognoms		IN	Clients.Cognoms%TYPE,
	p_DNI			IN	Clients.DNI%TYPE,
	p_CodiPostal	IN	Clients.CodiPostal%TYPE,
	p_Adreca		IN	Clients.Adreca%TYPE,
	p_Telefon		IN	Clients.Telefon%TYPE,
	p_Mobil		IN	Clients.Mobil%TYPE,
	p_Email		IN	Clients.Email%TYPE,
	p_DataNaixement	IN	Clients.DataNaixement%TYPE,
	p_Sexe		IN	Clients.Sexe%TYPE,
	p_DataAlta		IN	Clients.DataAlta%TYPE := Dia(sysdate)
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_Nom: '
		|| p_Nom
		|| ' p_Cognoms: '
		|| p_Cognoms
		|| ' p_DNI: '
		|| p_DNI
		|| ' p_CodiPostal: '
		|| p_CodiPostal
		|| ' p_Adrea: '
		|| p_Adreca
		|| ' p_Telefon: '
		|| p_Telefon
		|| ' p_Mobil: '
		|| p_Mobil
		|| ' p_Email: '
		|| p_Email
		|| ' p_DataNaixement: '
		|| p_DataNaixement
		|| ' p_Sexe: '
		|| p_Sexe
		|| ' p_DataAlta: '
		|| p_DataAlta;

	-- La data de naixement ha de ser ms petita que la d'alta
	IF p_DataNaixement > p_DataAlta
	THEN
		v_Res := 'La Data alta ha de ser posterior a la de naixement';
		raise_application_error (-20000, '');
	END IF;

	INSERT INTO Clients
		(Nom, Cognoms, DNI, CodiPostal, Adreca, Telefon, Mobil, Email,
		 DataNaixement, Sexe, DataAlta)
	VALUES (p_Nom, p_Cognoms, p_DNI, p_CodiPostal, p_Adreca, p_Telefon, p_Mobil, p_Email,
		 p_DataNaixement, p_Sexe, Dia(p_DataAlta));
 
	-- Log
	v_Fi := SYSDATE;
	v_Res := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el client amb el DNI ' || p_DNI;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2290
		THEN
			v_Res := 'Sexe Incorrecte. Ha de ser H o D';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'El Codi Postal no existeix';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Client (
/******************************************************************************
	NOM:			Modificar_Client

	REQUERIMENT N:	31

	DESCRIPCI:		Permet modificar la informaci associada a un client

	PARMETRES:

		p_IdClient		Identificador del client a modificar
		p_Nom			Nom del client
		p_Cognoms		Cognoms del client
		p_DNI			DNI del client
		p_CodiPostal	Codi Postal del client
		p_Adreca		Adrea del client
		p_Telefon		Telfon del client
		p_Mobil		Telfon mbil del client
		p_Email		Email del client
		p_DataNaixement	Data de Naixement del client
		p_Sexe		Sexe del client (H = Home, D = Dona)
		p_DataAlta		Data d'alta del client (per defecte, la data del sistema)


******************************************************************************/
	p_IdClient		IN	Clients.IdClient%TYPE,
	p_Nom			IN	Clients.Nom%TYPE,
	p_Cognoms		IN	Clients.Cognoms%TYPE,
	p_DNI			IN	Clients.DNI%TYPE,
	p_CodiPostal	IN	Clients.CodiPostal%TYPE,
	p_Adreca		IN	Clients.Adreca%TYPE,
	p_Telefon		IN	Clients.Telefon%TYPE,
	p_Mobil		IN	Clients.Mobil%TYPE,
	p_Email		IN	Clients.Email%TYPE,
	p_DataNaixement	IN	Clients.DataNaixement%TYPE,
	p_Sexe		IN	Clients.Sexe%TYPE,
	p_DataAlta		IN	Clients.DataAlta%TYPE := Dia(sysdate)
)
IS
	v_Res			VARCHAR2 (150);
	v_tmp_DataAlta	Clients.DataAlta%TYPE := NULL;
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdClient: '
		|| p_IdClient
		|| ' p_Nom: '
		|| p_Nom
		|| ' p_Cognoms: '
		|| p_Cognoms
		|| ' p_DNI: '
		|| p_DNI
		|| ' p_CodiPostal: '
		|| p_CodiPostal
		|| ' p_Adrea: '
		|| p_Adreca
		|| ' p_Telefon: '
		|| p_Telefon
		|| ' p_Mobil: '
		|| p_Mobil
		|| ' p_Email: '
		|| p_Email
		|| ' p_DataNaixement: '
		|| p_DataNaixement
		|| ' p_Sexe: '
		|| p_Sexe
		|| ' p_DataAlta: '
		|| p_DataAlta;

	-- Si v_tmp_DataAlta s NULL obtindrem la data d'alta del client
	IF v_tmp_DataAlta IS NULL
	THEN
		SELECT DataAlta
		INTO v_tmp_DataAlta
		FROM Clients
		WHERE IdClient = p_IdClient;
	ELSE
		v_tmp_DataAlta := p_DataAlta;
	END IF;

	-- La data de naixement ha de ser ms petita que la d'alta
	IF p_DataNaixement > v_tmp_DataAlta
	THEN
		v_Res := 'La Data alta ha de ser posterior a la de naixement';
		raise_application_error (-20000, '');
	END IF;

	UPDATE Clients
	SET Nom = p_Nom,
          Cognoms = p_Cognoms,
          DNI = p_DNI,
          CodiPostal = p_CodiPostal,
          Adreca = p_Adreca,
          Telefon = p_Telefon,
          Mobil = p_Mobil,
          Email = p_Email,
          DataNaixement = p_DataNaixement,
          Sexe = p_Sexe,
          DataAlta = Dia(v_tmp_DataAlta)
    WHERE IdClient = p_IdClient  AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix o est donat de baixa el client amb IdClient=' || p_IdClient;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el client amb el DNI ' || p_DNI;
		ELSIF SQLCODE = 100
		THEN
			v_Res := 'No existeix el client amb IdClient=' || p_IdClient;  
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2290
		THEN
			v_Res := 'Sexe Incorrecte. Ha de ser H o D';
        ELSIF SQLCODE = -2291
        THEN
            v_Res := 'El Codi Postal no existeix';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Baixa_Client (
/******************************************************************************
	NOM:			Baixa_Client

	REQUERIMENT N:	32

	DESCRIPCI:		Dna de baixa un client del nostre sistema

	PARMETRES:

		p_IdClient	Identificador del client
		p_DataBaixa	Data de baixa del client. Per defecte ser la del sistema

******************************************************************************/
	p_IdClient		IN	Clients.IdClient%TYPE,
	p_DataBaixa		IN	Clients.DataBaixa%TYPE:=Dia(sysdate)   
   )
IS
	v_DataAlta		Clients.DataAlta%TYPE;   
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdClient: ' || p_IdClient ||  ' p_DataBaixa: ' || p_DataBaixa;

	-- La data de baixa no pot ser anterior a la data d'alta
   
	SELECT DataAlta INTO v_DataAlta 
	FROM Clients 
	WHERE IdClient = p_IdClient AND DataBaixa IS NULL;

	IF v_DataAlta > p_DataBaixa THEN
		v_Res := 'La data de baixa no pot ser anterior a la data alta';
		raise_application_error (-20000, '');   
	END IF;
   
	DELETE Clients
	WHERE IdClient = p_IdClient;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'No existeix o es va donar de baixa el client amb IdClient=' || p_IdClient;  
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			-- No es pot eliminar perqu t registres associats
			-- S'informa la data de baixa

			UPDATE Clients
			SET DataBaixa = p_DataBaixa
			WHERE IdClient = p_IdClient AND DataBaixa IS NULL;

			v_Res := 'OK';
		ELSIF SQLCODE = 100
		THEN
			v_Res := 'No existeix o es va donar de baixa el client amb IdClient=' || p_IdClient;  
		ELSE
			IF v_Res IS NULL
			THEN
				-- Error no controlat per codi
				v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
			ELSE
				-- Error controlat per codi
				v_Res := 'Error:' || v_Res;
			END IF;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Recuperar_Client (
/******************************************************************************
	NOM:			Recuperar_Client

	REQUERIMENT N:	33

	DESCRIPCI:		Permet recuperar un client del nostre sistema, eliminant
				la data de baixa
	PARMETRES:

		p_IdClient	Identificador del Client

******************************************************************************/
	p_IdClient	IN	Clients.IdClient%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdClient: ' || p_IdClient;

	UPDATE Clients
	SET DataBaixa = NULL
	WHERE IdClient = p_IdClient AND DataBaixa IS NOT NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: El client ' || p_IdClient  || ' no existeix o no est donat de baixa';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;



CREATE OR REPLACE PROCEDURE Obtenir_Client_DNI
/******************************************************************************
	NOM:			Obtenir_Client_DNI

	REQUERIMENT N:	34

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun o varis clients, 
				a partir del DNI facilitat com a parmetre. Retornar les dades dels 
				clients el DNI dels quals continguin el facilitat com a parmetre. 
				Tamb per parmetre, especificarem si volem que tamb surtin els clients 
				donats de baixa
                 
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_DNI			DNI a cercar
		p_IncloureBaixes	Indica si es mostraran tamb els que estiguin de baixa
					Per defecte no sortiran (p_IncloureBaixes=0)

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_DNI			IN		Clients.DNI%TYPE,
	p_IncloureBaixes	IN		NUMBER:=0    
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := 'p_RS (IN OUT), p_DNI: ' || p_DNI  || ' p_IncloureBaixes:' || p_IncloureBaixes;

	OPEN p_RS FOR
		SELECT * FROM Clients 
		WHERE UPPER(DNI) LIKE '%' || UPPER(p_DNI) || '%' AND 
		CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
		DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;

	v_Res:= 'OK';

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM Clients WHERE IdClient IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Client_Nom
/******************************************************************************
	NOM:			Obtenir_Client_Nom

	REQUERIMENT N:	35

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun o varis clients, 
				a partir de les dades de nom i cognoms facilitades com a parmetre. 
				Retornar les dades dels clients el nom o cognoms dels quals continguin 
				els facilitats com a parmetre. 
				Tamb per parmetre, especificarem si volem que tamb surtin els clients donats de baixa
                 
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_Nom			Nom del client
		p_Apellidos		Cognoms del client
		p_IncloureBaixes	Indica si es mostraran tamb els que estiguin de baixa
					Por defecte no sortiran (p_IncloureBaixes=0)    
******************************************************************************/

( 
	p_RS			IN OUT	sys_refcursor,
	p_Nom			IN		Clients.Nom%TYPE:=Null,
	p_Cognoms		IN		Clients.Cognoms%TYPE:=Null, 
	p_IncloureBaixes	IN		NUMBER:=0    
  ) 
IS
	v_sql			VARCHAR2(500);  
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_Nom: ' || p_Nom  
		|| ' p_Cognoms: ' || p_Cognoms 
		|| ' p_IncloureBaixes:' || p_IncloureBaixes;

	v_sql:='SELECT * FROM Clients WHERE IdClient IS NOT NULL';

	IF p_IncloureBaixes <> 1 THEN
		v_sql:= v_sql || ' AND DataBaixa IS NULL ';
	END IF;

	IF p_Nom IS NOT NULL THEN
		v_sql:= v_sql || ' AND Nom LIKE ''%'||UPPER(p_Nom) ||'%'''; 
	END IF;

	IF p_Cognoms IS NOT NULL THEN
		v_sql:= v_sql || ' AND Cognoms LIKE ''%'||UPPER(p_Cognoms) ||'%''';
	END IF;


	OPEN p_RS for v_sql;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;



CREATE OR REPLACE PROCEDURE Obtenir_Client_Data
/******************************************************************************
	NOM:			Obtenir_Client_Data

	REQUERIMENT N: 36

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun o varis clients, 
				a partir de les dates facilitades com a parmetre. 
				Retornar les dades dels clients la data dels quals estigui compresa 
				entre les dates facilitades.

				Si noms es facilita la data dinici, retornar els clients a partir daquesta data.
				Si noms es facilita la data final, retornar els clients fins aquesta data.
				Si no es facilita cap data, retornar la totalitat dels clients.

				Podrem especificar per parmetre si el filtre aplica a la data dalta (A) o a la data de baixa (B).

				Tamb per parmetre, especificarem si volem que tamb surtin els clients donats de baixa
                 
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_tipus		'A': Retorna els clients donats d'alta al periode
					'B': Retorna els clients donats de baixa al periode
		p_DataDesde		Data Inici
		p_DataFi		Data Fi (per defecte, la del sistema)
		p_IncloureBaixes	Indica si es mostraran tamb els que estiguin de baixa
					Per defecte no sortiran (p_IncloureBaixes=0)

******************************************************************************/
( 
	p_RS			IN OUT	sys_refcursor, 
	p_Tipus		IN		VARCHAR2,  
	p_DataDesde		IN		Clients.DataAlta%TYPE:=Null,
	p_DataFi		IN		Clients.DataBaixa%TYPE:=Null,
	p_IncloureBaixes	IN		NUMBER:=0      
) 
IS
	v_sql			VARCHAR2(500);  
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT), p_Tipus: ' || p_Tipus ||  
		' p_DataDesde: ' || p_DataDesde  
		|| ' p_DataFi: ' || p_DataFi
		|| ' p_IncloureBaixes:' || p_IncloureBaixes;

	IF p_DataDesde > p_DataFi THEN
		v_Res := 'La data inici no pot ser superior a la data fi';
		raise_application_error (-20000, '');
	END IF;

	CASE UPPER(p_Tipus) 
		WHEN 'A' THEN 
			OPEN p_RS FOR
				SELECT * FROM Clients WHERE
				CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
					DataAlta >= p_DataDesde THEN 1 ELSE 0 END END = 1 AND 
				CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
					DataAlta <= p_DataFi THEN 1 ELSE 0 END END = 1 AND
				CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
					DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
            
		WHEN 'B' THEN 
			OPEN p_RS FOR
				SELECT * FROM Clients WHERE
				CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
					DataBaixa >= p_DataDesde Then 1 Else 0 END END = 1 AND 
				CASE WHEN p_DataFi IS NULL THEN 1 ELSE CASE WHEN 
					DataBaixa <= p_DataFi THEN 1 ELSE 0 END END = 1 ;
	END CASE;

	v_Res:= 'OK';

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;
      
	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/


COMMIT;
