/*------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Procediments del subsistema Productes
--------------------------------------------------*/


-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

COMMIT;


-------------------------------------------------------------------------------
-- Centres
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_Centre (
/******************************************************************************
	NOM:			Alta_Centre
   
	REQUERIMENT N:	37
   
	DESCRIPCI:		Dna dalta un centre al nostre sistema
                 
	PARMETRES:

		p_Centre		Nom del centre
		p_Adreca		Adrea del centre
		p_CodiPostal	Codi Postal del centre
		p_Telefon		Telfon del centre
		p_Contacte		Persona de contacto del centre
		p_DataAlta		Data d'alta del centre (per defecte pren la data del sistema)


******************************************************************************/
	p_Centre		IN	Centres.Centre%TYPE,
	p_Adreca		IN	Centres.Adreca%TYPE,
	p_CodiPostal	IN	Centres.CodiPostal%TYPE,
	p_Telefon		IN	Centres.Telefon%TYPE,
	p_Contacte		IN	Centres.Contacte%TYPE := NULL,
	p_DataAlta		IN	Centres.DataAlta%TYPE := Dia(SYSDATE)
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_Centre: '
		|| p_Centre
		|| ' p_Adreca: '
		|| p_Adreca
		|| ' p_CodiPostal: '
		|| p_CodiPostal
		|| ' p_Telefon: '
		|| p_Telefon
		|| ' p_Contacte: '
		|| p_Contacte
		|| ' p_DataAlta: '
		|| p_DataAlta;


	INSERT INTO Centres (Centre, Adreca, CodiPostal, Telefon, Contacte, DataAlta)
	VALUES (p_Centre, p_Adreca, p_CodiPostal, p_Telefon, p_Contacte, Dia(p_DataAlta));

	-- Log
	v_Fi := SYSDATE;
	v_Res := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El centre ' || p_Centre || ' ja existeix';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix el Codi Postal ' || p_CodiPostal;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Centre (
/******************************************************************************
	NOM:			Modificar_Centre
   
	REQUERIMENT N:	38
   
	DESCRIPCI:		Permet modificar un centre del nostre sistema

	PARMETRES:

	p_IdCentre		Identificador del centre
	p_Centre		Nom del centre
	p_Adreca		Adrea del centre
	p_CodiPostal	Codi Postal del centre
	p_Telefon		Telfon del centre
	p_Contacte		Persona de contacto del centre
	p_DataAlta		Data d'alta del centre (Si no s'introdueix mantindr la data d'alta existent)

******************************************************************************/
	p_IdCentre		IN	Centres.IdCentre%TYPE,
	p_Centre		IN	Centres.Centre%TYPE,
	p_Adreca		IN	Centres.Adreca%TYPE,
	p_CodiPostal	IN	Centres.CodiPostal%TYPE,
	p_Telefon		IN	Centres.Telefon%TYPE,
	p_Contacte		IN	Centres.Contacte%TYPE := NULL,
	p_DataAlta		IN	Centres.DataAlta%TYPE :=  NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdCentre: '
		|| p_IdCentre
		|| ' p_Centre: '
		|| p_Centre
		|| ' p_Adreca: '
		|| p_Adreca
		|| ' p_CodiPostal: '
		|| p_CodiPostal
		|| ' p_Telefon: '
		|| p_Telefon
		|| ' p_Contacte: '
		|| p_Contacte
		|| ' p_DataAlta: '
		|| p_DataAlta;

	UPDATE Centres
	SET Centre = p_Centre,
		Adreca = p_Adreca,
		CodiPostal = p_CodiPostal,
		Telefon = p_Telefon,
		Contacte = p_Contacte,
		DataAlta = NVL (Dia(p_DataAlta), DataAlta)
	WHERE IdCentre = p_IdCentre AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: El centre ' || p_IdCentre || ' no existeix o est donat de baixa' ;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El centre ' || p_Centre || ' ja existeix';
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix el Codi Postal ' || p_CodiPostal;
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Baixa_Centre (
/******************************************************************************
	NOM:			Baixa_Centre
   
	REQUERIMENT N:	39
   
	DESCRIPCI:		Dna de baixa un centre del nostre sistema

	PARMETRES:

		p_IdCentre		Identificador del centre
		p_DataBaixa		Data de baixa del centre

******************************************************************************/
	p_IdCentre		IN	Centres.IdCentre%TYPE, 
	p_DataBaixa		IN	Centres.DataBaixa%TYPE:=Dia(SYSDATE)
   )
IS
	v_DataAlta		Centres.DataAlta%TYPE;
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdCentre: ' || p_IdCentre || ' p_DataBaixa: ' || p_DataBaixa ;

	-- Comprova que el centre existeixi i estigui donat d'alta
	v_Res:= 'El centre amb IdCentre=' || p_IdCentre || ' no existeix o est donat de baixa';
	
	SELECT DataAlta INTO v_DataAlta 
	FROM Centres WHERE IdCentre = p_IdCentre AND DataBaixa IS NULL;
   
	-- Comprova que la data de baixa no sigui anterior a la d'alta
	If v_DataAlta > p_DataBaixa THEN
		v_Res:= 'La data alta del centre no pot ser anterior a la de baixa';
		raise_application_error (-20000, '');
	END IF;   

	DELETE Centres
	WHERE IdCentre = p_IdCentre AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el centre o ja es va donar de baixa ' || p_IdCentre;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			-- No es pot eliminar perqu t productes associats, s'assignar data de baixa
			UPDATE Centres
			SET DataBaixa = Dia(sysdate)
			WHERE IdCentre = p_IdCentre AND DataBaixa IS NULL;

			v_Res := 'OK';
		ELSE
			IF v_Res IS NULL
			THEN
				-- Error no controlat per codi
				v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
			ELSE
				-- Error controlat per codi
				v_Res := 'Error:' || v_Res;
			END IF;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Recuperar_Centre (
/******************************************************************************
	NOM:			Recuperar_Centre
   
	REQUERIMENT N:	40
   
	DESCRIPCI:		Permet recuperar un centre del nostre sistema

	PARMETRES:

		p_IdCentre	Identificador del centre

******************************************************************************/
	p_IdCentre	IN	Centres.IdCentre%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdCentre: ' || p_IdCentre;

	UPDATE Centres
	SET DataBaixa = NULL
	WHERE IdCentre = p_IdCentre AND DataBaixa IS NOT NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No hi ha cap centre donat de baixa amb identificador  ' || p_IdCentre;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Centre
/******************************************************************************
	NOM:			Obtenir_Centre
   
	REQUERIMENT N: 41
   
	DESCRIPCI:		Permet obtenir tota la informaci complerta dun centre, 
				a partir de lidentificador de centre facilitat com a parmetre. 
				Si aquest parmetre s null, retornar els centres el nom dels quals 
				contingui el nom que es passa per parmetre. 
				Si no sespecifica cap dels dos parmetres, retornar una relaci de 
				tots els centres. Tamb tindrem lopci, per parmetre, de decidir si 
				han de sortir o no els centres donats de baixa
                 
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdCentre		IdCentre a cercar
		p_Centre		Centre a cercar
		p_IncloureBaixes	Indica si es mostraran tamb els que estiguin de baixa
					Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_IdCentre		IN		Centres.IdCentre%TYPE:=Null,
	p_Centre		IN		Centres.Centre%TYPE:=Null,
	p_IncloureBaixes	IN		NUMBER:=0    
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_IdCentre: ' || p_IdCentre || ' p_Centre:' || p_Centre
              || ' p_IncloureBaixes:' || p_IncloureBaixes;


 	IF p_IdCentre IS NULL AND p_Centre IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Centres 
			WHERE CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE When 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
        
	ELSIF p_IdCentre IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Centres 
			WHERE IdCentre = p_IdCentre AND 
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
        
	ELSIF p_Centre IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Centres 
			WHERE UPPER(Centre) LIKE '%' || UPPER(p_Centre) || '%' AND 
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
	END IF;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
End;
/

-------------------------------------------------------------------------------
-- Garanties
-------------------------------------------------------------------------------


CREATE OR REPLACE PROCEDURE Alta_Garantia (
/******************************************************************************
	NOM:			Alta_Garantia 

	REQUERIMENT N:	42

	DESCRIPCI:		Dna dalta una garantia al nostre sistema

	PARMETROS:

		p_Garantia		Nom de la garantia
		p_AnysTotal		Anys total de garantia
		p_AnysInsitu	Anys de garantia in situ

******************************************************************************/
	p_Garantia		IN	Garanties.Garantia%TYPE,
	p_AnysTotal 	IN	Garanties.AnysTotal%TYPE,
	p_AnysInsitu	IN	Garanties.AnysInsitu%TYPE)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_Garantia: ' || p_Garantia || ' p_AnysTotal: ' || p_AnysTotal || ' p_AnysInsitu: ' || p_AnysInsitu;

	INSERT INTO Garanties (Garantia, AnysTotal, AnysInsitu)
	VALUES (p_Garantia, p_AnysTotal, p_AnysInsitu);

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := 'OK';
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'La garantia ' || p_Garantia || ' ja existeix';
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Garantia (
/******************************************************************************
	NOM:			Modificar_Garantia
   
	REQUERIMENT N:	43
   
	DESCRIPCI:		Permet modificar una garantia del nostre sistema
                 
	PARMETRES:

		p_IdGarantia	Identificador de la garantia
		p_Garantia		Nom de la garantia
		p_AnysTotal		Anys total de garantia
		p_AnysInsitu	Anys de garantia in situ

******************************************************************************/
	p_IdGarantia	IN	Garanties.IdGarantia%TYPE,
	p_Garantia		IN	Garanties.Garantia%TYPE,
	p_AnysTotal 	IN	Garanties.AnysTotal%TYPE,
	p_AnysInsitu	IN	Garanties.AnysInsitu%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdGarantia: ' ||
		p_IdGarantia ||
		' p_Garantia: ' ||
		p_Garantia ||
		' p_AnysTotal: ' ||
		p_AnysTotal ||
		' p_AnysInsitu: ' ||
		p_AnysInsitu;

	UPDATE Garanties
	SET Garantia = p_Garantia
	WHERE IdGarantia = p_IdGarantia;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix la garantia amb IdGarantia=' || p_IdGarantia;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'La garantia ' || p_Garantia || ' ja existeix';             
        ELSIF SQLCODE = -1407
        THEN
            v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

      DBMS_OUTPUT.put_line (v_Res);
      v_Fi := SYSDATE;
      v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Eliminar_Garantia (
/******************************************************************************
	NOM:			Eliminar_Garantia 
                 
	REQUERIMENT N:	44

	DESCRIPCI:		Elimina un garantia del nostre sistema

	PARMETRES:

		p_IdGarantia	Identificador de la garantia

******************************************************************************/
	p_IdGarantia	IN	Garanties.IdGarantia%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdGarantia: ' || p_IdGarantia;

	DELETE Garanties WHERE IdGarantia = p_IdGarantia;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix la garantia amb IdGarantia=' || p_IdGarantia;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE =  -2292 
		THEN
			-- No es pot eliminar perqu t productes associats
			v_Res := 'No es pot eliminar la garantia ' || p_IdGarantia || ' perqu t productes associats';
		END IF;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Obtenir_Garantia
/******************************************************************************
	NOM:			Obtenir_Garantia

	REQUERIMENT N:	45

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna garantia, 
				a partir de lidentificador de garantia facilitat com a parmetre. 
				Si aquest parmetre s null, retornar les garanties el nom de les 
				quals contingui el nom que es passa per parmetre. 
				Si no sespecifica cap dels dos parmetres, retornar una relaci 
				de totes les garanties 

	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdGarantia	IdGarantia a cercar
		p_Garantia		Garantia a cercar

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_IdGarantia	IN		Garanties.IdGarantia%TYPE:=Null,
	p_Garantia		IN		Garanties.Garantia%TYPE:=Null
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := 'p_RS (IN OUT), p_IdGarantia: ' || p_IdGarantia || ' p_Garantia:' || p_Garantia ;

	IF p_IdGarantia IS NULL AND p_Garantia IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Garanties;
	ELSIF p_IdGarantia IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Garanties Where IdGarantia = p_IdGarantia;
	ELSIF p_Garantia IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Garanties WHERE UPPER(Garantia) LIKE '%' || UPPER(p_Garantia) || '%';
	END IF;

	v_Res:= 'OK';

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

-------------------------------------------------------------------------------
-- Tipus Producte
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_TipusProducte (
/******************************************************************************
	NOM:			Alta_TipusProducte 

	REQUERIMENT N:	46

	DESCRIPCIN:	Dna dalta un tipus de producte al nostre sistema

	PARMETRES:

		p_TipusProducte		Tipus de Producte
		p_IdTipusPare		Tipus pare del Tipus de Producte
		p_DataAlta			Data d'alta

******************************************************************************/
	p_TipusProducte		IN	TipusProducte.TipusProducte%TYPE,
	p_IdTipusPare		IN	TipusProducte.IdTipusPare%TYPE,
	p_DataAlta			IN	TipusProducte.DataAlta%TYPE := Dia(SYSDATE)
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
         ' p_TipusProducte: '
      || p_TipusProducte
      || ' p_IdTipusPare: '
      || p_IdTipusPare
      || ' p_DataAlta: '
      || p_DataAlta;

	INSERT INTO TipusProducte (TipusProducte , IdTipusPare, DataAlta)
	VALUES (p_TipusProducte , p_IdTipusPare, Dia(p_DataAlta));

	-- Log
	v_Fi := SYSDATE;
	v_Res := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el TipusProducte ' || p_TipusProducte;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
        ELSIF SQLCODE = -2291
        THEN
            v_Res := 'El identificador de tipus de producte pare no s correcte';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_TipusProducte (
/******************************************************************************
	NOM:			Modificar_TipusProducte

	REQUERIMENT N:	47

	DESCRIPCI:		Permet modificar un tipus de producte del nostre sistema

	PARMETRES:

		p_IdTipusProducte		Identificador del TipusProducte
		p_TipusProducte		Tipus de producte
		p_IdTipusPare		Tipus pare del Tipus de Producte
		p_DataAlta			Data d'alta (Si no s'introdueix mantindr la data d'alta existent)

******************************************************************************/
	p_IdTipusProducte		IN	TipusProducte.IdTipusProducte%TYPE,
	p_TipusProducte		IN	TipusProducte.TipusProducte%TYPE,
	p_IdTipusPare		IN	TipusProducte.IdTipusPare%TYPE,
	p_DataAlta			IN	TipusProducte.DataAlta%TYPE := NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdTipusProducte: '
		|| p_IdTipusProducte
		|| ' p_TipusProducte: '
		|| p_TipusProducte
		|| ' p_IdTipusPare: '
		|| p_IdTipusPare
		|| ' p_DataAlta: '
		|| p_DataAlta;

	UPDATE TipusProducte
	SET TipusProducte = p_TipusProducte,
		IdTipusPare = p_IdTipusPare,
		DataAlta = NVL (DIA(p_DataAlta), DataAlta)
	WHERE IdTipusProducte = p_IdTipusProducte AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix o donat de baixa el TipusProducte amb IdTipusProducte=' || p_IdTipusProducte;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el TipusProducte ' || p_TipusProducte;
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
        ELSIF SQLCODE = -2291
        THEN
            v_Res := 'El identificador de tipus de producte pare no s correcte';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Baixa_TipusProducte (
/******************************************************************************
	NOM:			Baixa_TipusProducte 

	REQUERIMENT N:	48

	DESCRIPCI:		Elimina o dna de baixa un tipus de producte del nostre sistema, 
				proporcionant el seu identificador. 
				Si el tipus de producte no t cap producte associat seliminar. 
				En cas contrari, es donar de baixa

	PARMETRES:

		p_IdTipusProducte		Identificador del tipus de producte
		p_DataBaixa			Data de baixa del tipus de producte 

******************************************************************************/
	p_IdTipusProducte		IN	TipusProducte.IdTipusProducte%TYPE,
	p_DataBaixa			IN	TipusProducte.DataBaixa%TYPE:=Dia(SYSDATE)
   )
IS
	v_DataAlta		TipusProducte.DataAlta%TYPE;
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdTipusProducte: ' || p_IdTipusProducte || ' p_DataBaixa: ' || p_DataBaixa ;

	-- Comprova que existeixi i estigui donat d'alta
	v_Res:= 'El tipus de producte amb IdTipusProducte=' || p_IdTipusProducte || ' no existeix o est donat de baixa';

	SELECT DataAlta INTO v_DataAlta 
	FROM TipusProducte WHERE IdTipusProducte = p_IdTipusProducte AND DataBaixa IS NULL;
   
	-- Comprova que la data de baixa no sigui anterior a la d'alta
	IF v_DataAlta > p_DataBaixa THEN
		v_Res:= 'La data alta no pot ser anterior a la de baixa';
		raise_application_error (-20000, '');
	END IF;   

	DELETE TipusProducte
	WHERE IdTipusProducte = p_IdTipusProducte AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el TipusProducte ' || p_IdTipusProducte || ', o ja est de baixa ';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			-- No es pot borrar perqu t productes associats, s'assignar la data de baixa
			UPDATE TipusProducte
			SET DataBaixa = Dia(SYSDATE)
			WHERE IdTipusProducte = p_IdTipusProducte AND DataBaixa IS NULL;

			v_Res := 'OK';
		ELSE
			IF v_Res IS NULL
			THEN
				-- Error no controlat per codi
				v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
			ELSE
				-- Error controlat per codi
				v_Res := 'Error:' || v_Res;
			END IF;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Recuperar_TipusProducte (
/******************************************************************************
	NOM:			Recuperar_TipusProducte

	REQUERIMENT N:	49

	DESCRIPCI:		Permet recuperar un tipus de producte del nostre sistema
                
	PARMETRES:                 

		p_IdTipusProducte		Identificador del tipus de producte

******************************************************************************/
	p_IdTipusProducte		IN	TipusProducte.IdTipusProducte%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdTipusProducte: ' || p_IdTipusProducte;

	UPDATE TipusProducte
	SET DataBaixa = NULL
	WHERE IdTipusProducte = p_IdTipusProducte AND DataBaixa IS NOT NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el TipusProducte ' || p_IdTipusProducte;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_TipusProducte
/******************************************************************************
	NOM:			Obtenir_TipusProducte

	REQUERIMENT N:	50

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun tipus de producte, 
				a partir de lidentificador de tipus de producte facilitat com a parmetre. 
				Si aquest parmetre s null, retornar els tipus de producte el nom dels 
				quals contingui el nom que es passa per parmetre. 
				Si no sespecifica cap dels dos parmetres, retornar una relaci de tots 
				els tipus de producte. 
				Tamb tindrem lopci, per parmetre, de decidir si han de sortir o no els 
				tipus de producte donats de baixa 
                  
	PARMETRES:

		p_RS				Recordset amb el resultat
		p_IdTipusProducte		IdTipusProducte a cercar
		p_TipusProducte		TipusProducte a cercar
		p_IncloureBaixes		Indica si es mostraran tamb els que estiguin de baixa
						Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdTipusProducte		IN		TipusProducte.IdTipusProducte%TYPE:=Null,
	p_TipusProducte		IN		TipusProducte.TipusProducte%TYPE:=Null,
	p_IncloureBaixes		IN		NUMBER:=0        
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_IdTipusProducte: ' || p_IdTipusProducte || ' p_TipusProducte:' || p_TipusProducte 
		|| ' p_IncloureBaixes:' || p_IncloureBaixes;

	IF p_IdTipusProducte IS NULL AND p_TipusProducte IS NULL THEN 
		OPEN p_RS FOR
			SELECT * FROM TipusProducte
			WHERE CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;

	ELSIF p_IdTipusProducte IS NOT NULL THEN 
		OPEN p_RS FOR
			SELECT * FROM TipusProducte WHERE IdTipusProducte = p_IdTipusProducte AND 
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
        
	ELSIF p_TipusProducte IS NOT NULL THEN 
		OPEN p_RS FOR
			SELECT * FROM TipusProducte WHERE UPPER(TipusProducte) LIKE '%' || UPPER(p_TipusProducte) || '%' AND
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
	END IF;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

-------------------------------------------------------------------------------
-- Productes
-------------------------------------------------------------------------------


CREATE OR REPLACE PROCEDURE Alta_Producte (
/******************************************************************************
	NOM:			Alta_Producte

	REQUERIMENT N:	51

	DESCRIPCI:		Dna dalta un producte al nostre sistema

	PARMETRES:

		p_IdTipusProducte		Tipus de producte
		p_Descripcio		Descripcio del producte
		p_ISBN			ISBN del producte
		p_NumeroSerie		Nmero de serie del producte
		p_DataVenda			Data de venda del producte
		p_IdCentre			Centre on s'ha fabricat el producte
		p_IdGarantia		Garantia del producte
		p_Alcada			Alada del producte en cm
		p_Ample			Ample del producte en cm
		p_Fons			Fons del producte en cm
		p_DataAlta			Data d'alta del producte (Per defecte pren la data del sistema)

******************************************************************************/
	p_IdTipusProducte		IN	Productes.IdTipusProducte%TYPE,
	p_Descripcio		IN	Productes.Descripcio%TYPE, 
	p_ISBN			IN	Productes.ISBN%TYPE,
	p_NumeroSerie		IN	Productes.NumeroSerie%TYPE,
	p_DataVenda			IN	Productes.DataVenda%TYPE,
	p_IdCentre			IN	Productes.IdCentre%TYPE,
	p_IdGarantia		IN	Productes.IdGarantia%TYPE,
	p_Alcada			IN	Productes.Alcada%TYPE,
	p_Ample			IN	Productes.Ample%TYPE,
	p_Fons			IN	Productes.Fons%TYPE,
	p_DataAlta			IN	Productes.DataAlta%TYPE := Dia(SYSDATE)
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdTipusProducte: '
		|| p_IdTipusProducte
		|| ' p_Descripcio: '
		|| p_Descripcio
		|| ' p_ISBN: '
		|| p_ISBN
		|| ' p_NumeroSerie: '
		|| p_NumeroSerie
		|| ' p_DataVenda: '
		|| p_DataVenda
		|| ' p_IdCentre: '
		|| p_IdCentre
		|| ' p_IdGarantia: '
		|| p_IdGarantia
		|| ' p_Alcada: '
		|| p_Alcada
		|| ' p_Ample: '
		|| p_Ample
		|| ' p_Fons: '
		|| p_Fons
		|| ' p_DataAlta: '
		|| p_DataAlta;

	-- Data de Alta > Data Venda
	IF p_DataAlta < p_DataVenda
	THEN
		v_Res := 'La data de venda ha de ser anterior a la de alta';
		raise_application_error (-20000, '');
	END IF;

	INSERT INTO Productes (IdTipusProducte, Descripcio, ISBN, NumeroSerie, DataVenda, IdCentre, IdGarantia, Alcada, Ample, Fons, DataAlta)
	VALUES (p_IdTipusProducte, p_Descripcio, p_ISBN, p_NumeroSerie, p_DataVenda, p_IdCentre, p_IdGarantia, p_Alcada, p_Ample, p_Fons, Dia(p_DataAlta));
 
	-- Log
	v_Fi := SYSDATE;
	v_REs := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El producte amb ISBN' || p_ISBN || ' ja existeix';
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'El identificador de tipus de producte i/o de centre i/o de garantia no sn correctes';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Producte (
/******************************************************************************
	NOM:			Modificar_Producte

	REQUERIMENT N:	52

	DESCRIPCI:		Permet modificar la informaci associada a un producte

	PARMETRES:

		p_IdProducte		Identificador del producte
		p_IdTipusProducte		Tipus de producte
		p_Descripcio		Descripcio del producte
		p_ISBN			ISBN del producte
		p_NumeroSerie		Nmero de serie del producte
		p_DataVenda			Data de venda del producte
		p_IdCentre			Centre on s'ha fabricat el producte
		p_IdGarantia		Garantia del producte
		p_Alcada			Alada del producte en cm
		p_Ample			Ample del producte en cm
		p_Fons			Fons del producte en cm
		p_DataAlta			Data d'alta del producte (Si no s'introdueix mantindr la data d'alta existent)

******************************************************************************/
	p_IdProducte		IN	Productes.IdProducte%TYPE,
	p_IdTipusProducte		IN	Productes.IdTipusProducte%TYPE,
	p_Descripcio		IN	Productes.Descripcio%TYPE, 
	p_ISBN			IN	Productes.ISBN%TYPE,
	p_NumeroSerie		IN	Productes.NumeroSerie%TYPE,
	p_DataVenda			IN	Productes.DataVenda%TYPE,
	p_IdCentre			IN	Productes.IdCentre%TYPE,
	p_IdGarantia		IN	Productes.IdGarantia%TYPE,
	p_Alcada			IN	Productes.Alcada%TYPE,
	p_Ample			IN	Productes.Ample%TYPE,
	p_Fons			IN	Productes.Fons%TYPE,
	p_DataAlta			IN	Productes.DataAlta%TYPE := NULL
)
IS
	v_tmp_DataAlta	Productes.DataAlta%TYPE := NULL;
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdProducte: '
		|| p_IdProducte
		|| ' p_IdTipusProducte: '
		|| p_IdTipusProducte
		|| ' p_Descripcio: '
		|| p_Descripcio
		|| ' p_ISBN: '
		|| p_ISBN
		|| ' p_NumeroSerie: '
		|| p_NumeroSerie
		|| ' p_DataVenda: '
		|| p_DataVenda
		|| ' p_IdCentre: '
		|| p_IdCentre
		|| ' p_IdGarantia: '
		|| p_IdGarantia
		|| ' p_Alcada: '
		|| p_Alcada
		|| ' p_Ample: '
		|| p_Ample
		|| ' p_Fons: '
		|| p_Fons
		|| ' p_DataAlta: '
		|| p_DataAlta;

	-- Si p_DataAlta s NULL, agafem la data d'alta del producte
	IF p_DataAlta IS NULL
	THEN
		SELECT DataAlta 
		INTO v_tmp_DataAlta
		FROM Productes
		WHERE IdProducte = p_IdProducte;
	ELSE
		v_tmp_DataAlta := p_DataAlta ;
	END IF;

	-- Data de Alta > Data Venda
	IF v_tmp_DataAlta < p_DataVenda
	THEN
		v_Res := 'La data de venda ha de ser anterior a la de alta';
		raise_application_error (-20000, '');
	END IF;

	UPDATE Productes
	SET IdTipusProducte = p_IdTipusProducte,
		Descripcio = p_Descripcio,
		ISBN = p_ISBN,
		NumeroSerie = p_NumeroSerie,
		DataVenda = p_DataVenda,
		IdCentre = p_IdCentre,
		IdGarantia = p_IdGarantia,
		Alcada = p_Alcada,
		Ample = p_Ample,
		Fons = p_Fons,
		DataAlta = DIA(v_tmp_DataAlta)
	WHERE IdProducte = p_IdProducte AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix o est donat de baixa el producte amb IdProducte=' || p_IdProducte;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el producte amb ISBN ' || p_ISBN;
		ELSIF SQLCODE = 100
		THEN
			v_Res := 'No existeix el producte amb IdProducte=' || p_IdProducte;  
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
        ELSIF SQLCODE = -2291
        THEN
            v_Res := 'El identificador de tipus de producte i/o de centre i/o de garantia no sn correctes';
        END IF;      

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Baixa_Producte (
/******************************************************************************
	NOM:			Baixa_Producte

	REQUERIMENT N:	53

	DESCRIPCI:		Dna de baixa un producte del nostre sistema

	PARMETRES:

		p_IdProducte	Identificador del producte
		p_DataBaixa		Data de baixa del producte (Per defecte pren la del sistema)

******************************************************************************/
	p_IdProducte	IN	Productes.IdProducte%TYPE,
	p_DataBaixa		IN	Productes.DataBaixa%TYPE:=Dia(SYSDATE)   
   )
IS
	v_DataAlta		Productes.DataAlta%TYPE;   
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdProducte: ' || p_IdProducte||  ' p_DataBaixa: ' || p_DataBaixa;

	-- La data de baixa no pot ser anterior a la data d'alta
	SELECT DataAlta INTO v_DataAlta FROM Productes 
	WHERE IdProducte = p_IdProducte  AND DataBaixa IS NULL;

	IF v_DataAlta > p_DataBaixa THEN
		v_Res:= 'La data de baixa no pot ser anterior a la de alta';
		raise_application_error (-20000, '');   
	END IF;
   
	DELETE Productes
	WHERE IdProducte = p_IdProducte;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'No existeix o es va donar de baixa el producte amb IdProducte=' || p_IdProducte;  
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			-- No es pot borrar perqu t registres d'altres taules associades , s'assignar la data de baixa         
			UPDATE Productes
			SET DataBaixa = p_DataBaixa 
			WHERE IdProducte = p_IdProducte AND DataBaixa IS NULL;

			v_Res := 'OK';
		ELSIF SQLCODE = 100
		THEN
			v_Res := 'No existeix o es va donar de baixa el producte amb IdProducte=' || p_IdProducte;  
		ELSE
			IF v_Res IS NULL
			THEN
				-- Error no controlat per codi
				v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
			ELSE
				-- Error controlat per codi
				v_Res := 'Error:' || v_Res;
			END IF;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Recuperar_Producte (
/******************************************************************************
	NOM:			Recuperar_Producte

	REQUERIMENT N:	54

	DESCRIPCI:		Permet recuperar un producte del nostre sistema

	PARMETRES:

		p_IdProducte	Identificador del producte

******************************************************************************/
	p_IdProducte	IN	Productes.IdProducte%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdProducte: ' || p_IdProducte;

	UPDATE Productes
	SET DataBaixa = NULL
	WHERE IdProducte = p_IdProducte AND DataBaixa IS NOT NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: El producte ' || p_IdProducte  || ' no existeix o no est donat de baixa';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Producte_TipusProducte
/******************************************************************************
	NOM:			Obtenir_Producte_TipusProducte

	REQUERIMENT N:	55

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun o varis productes, 
				a partir del tipus de producte facilitat com a parmetre. 
				Retornar les dades dels productes el tipus dels quals sigui com el tipus parmetre. 
				Tamb per parmetre, especificarem si volem que tamb surtin els productes donats de baixa 

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_IdTipusProducte		Identificador del tipus de producte
		p_IncloureBaixes		Indica si es mostraran tamb els que estiguin donats de baixa
						Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdTipusProducte		IN		Productes.IdTipusProducte%TYPE,
	p_IncloureBaixes		IN		NUMBER:=0       
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdTipusProducte: ' || p_IdTipusProducte  || 
		' p_IncloureBaixes:' || p_IncloureBaixes;

	OPEN p_RS FOR
		SELECT * FROM Productes 
		WHERE IdTipusProducte = p_IdTipusProducte AND 
		CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
		DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Producte_Desc
/******************************************************************************
	NOM:			Obtenir_Producte_Desc

	REQUERIMENT N:	56

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun o varis productes, 
				a partir de la descripci facilitada com a parmetre. 
				Retornar les dades dels productes la descripci dels quals contingui 
				la facilitada com a parmetre. Tamb per parmetre, especificarem 
				si volem que tamb surtin els productes donats de baixa

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_Descripcio		Descripci a cercar
		p_IncloureBaixes		Indica si es mostraran tamb els que estiguin donats de baixa
						Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_Descripcio	IN		Productes.Descripcio%TYPE,
	p_IncloureBaixes	IN		NUMBER:=0    
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_Descripcio: ' || p_Descripcio  || ' p_IncloureBaixes:' || p_IncloureBaixes;


	OPEN p_RS FOR
		SELECT * FROM Productes 
		WHERE UPPER(Descripcio) LIKE '%' || UPPER(p_Descripcio) || '%' AND 
		CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
		DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Producte_Data
/******************************************************************************
	NOM:			Obtenir_Producte_Data

	REQUERIMENT N:	57

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun o varis productes, 
				a partir de les dates facilitades com a parmetre. 
				Retornar les dades dels productes la data dels quals estigui compresa entre les dates facilitades.

				Si noms es facilita la data dinici, retornar els productes a partir daquesta data.
				Si noms es facilita la data final, retornar els productes fins aquesta data.
				Si no es facilita cap data, retornar la totalitat dels productes.

				Podrem especificar per parmetre si el filtre aplica a la data dalta (A), 
				a la data de baixa (B), o a la data de venda (V).

				Tamb per parmetre, especificarem si volem que tamb surtin els productes donats de baixa
                 
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_Tipus		Tipus de filtre de data
		p_DataDesde		Data inici
		p_DataFi		Data fi (per defecte ser la del sistema)
		p_IncloureBaixes	Indica si es mostraran tamb els que estiguin donats de baixa
					Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/
( 
	p_RS			IN OUT	sys_refcursor, 
	p_Tipus		IN		VARCHAR2,  
	p_DataDesde		IN		Productes.DataAlta%TYPE:=Null,
	p_DataFi		IN		Productes.DataAlta%TYPE:=Null,
	p_IncloureBaixes	IN		NUMBER:=0      
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT), p_Tipus: ' || p_Tipus ||  
		' p_DataDesde: ' || p_DataDesde  
		|| ' p_DataFi: ' || p_DataFi
		|| ' p_IncloureBaixes:' || p_IncloureBaixes;

	IF p_DataDesde > p_DataFi THEN
		v_Res := 'La data de inici no pot ser superior a la data final';
		raise_application_error (-20000, '');
	END IF;

	CASE UPPER(p_Tipus) 
	WHEN 'A' THEN 

		OPEN p_RS FOR
			SELECT * FROM Productes WHERE
			CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
				DataAlta >= p_DataDesde THEN 1 ELSE 0 END END = 1 AND 
			CASE WHEN p_DataFi IS NULL THEN 1 ELSE CASE WHEN 
				DataAlta <= p_DataFi THEN 1 ELSE 0 END END = 1 AND
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
				DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
            
	WHEN 'B' THEN 

		OPEN p_RS FOR
			SELECT * FROM Productes WHERE
			CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
				DataBaixa >= p_DataDesde THEN 1 ELSE 0 END END = 1 AND 
			CASE WHEN p_DataFi IS NULL THEN 1 ELSE CASE WHEN 
				DataBaixa <= p_DataFi THEN 1 ELSE 0 END END = 1 ;

	WHEN 'V' THEN 

		OPEN p_RS FOR
			SELECT * FROM Productes WHERE
			CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
				DataVenda >= p_DataDesde THEN 1 ELSE 0 END END = 1 AND 
			CASE WHEN p_DataFi IS NULL THEN 1 ELSE CASE WHEN 
				DataVenda <= p_DataFi THEN 1 ELSE 0 END END = 1 AND
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
				DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
        
	ELSE
		v_Res := 'El tipus ha de ser algun dels segents: A->Alta, B->Baixa, V->Venda';
		raise_application_error (-20000, '');
	END CASE;


	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;
      
	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Producte_ISBN
/******************************************************************************
	NOM:			Obtenir_Producte_ISBN

	REQUERIMENT N:	58

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun producte, a partir 
				de lISBN facilitat com a parmetre. Retornar les dades dun nic 
				producte lISBN del qual sigui el mateix ISBN del parmetre

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_ISBN			ISBN a cercar
		p_IncloureBaixes		Indica si es mostraran tamb els que estiguin donats de baixa
						Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_ISBN		IN		Productes.ISBN%TYPE,
	p_IncloureBaixes	IN		NUMBER:=0    
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_ISBN: ' || p_ISBN  || ' p_IncloureBaixes:' || p_IncloureBaixes;


	OPEN p_RS FOR
		SELECT * FROM Productes 
		WHERE UPPER(ISBN) LIKE '%' || UPPER(p_ISBN) || '%' AND 
		CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
		DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Producte_Centre
/******************************************************************************
	NOM:			Obtenir_Producte_Centre

	REQUERIMENT N:	59

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun producte, 
				a partir de lidentificador de centre facilitat com a parmetre. 
				Retornar les dades dels productes el centre dels quals sigui el 
				mateix que el facilitat per parmetre 

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_IdCentre			Identificador del centre del producte
		p_IncloureBaixes		Indica si es mostraran tamb els que estiguin donats de baixa
						Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdCentre			IN		Productes.IdCentre%TYPE,
	p_IncloureBaixes		IN		NUMBER:=0       
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdCentre: ' || p_IdCentre  || 
		' p_IncloureBaixes:' || p_IncloureBaixes;

	OPEN p_RS FOR
		SELECT * FROM Productes 
		WHERE IdCentre = p_IdCentre AND 
		CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
		DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Producte_Garantia
/******************************************************************************
	NOM:			Obtenir_Producte_Garantia

	REQUERIMENT N:	60

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun producte, 
				a partir de lidentificador de garantia facilitat com a parmetre. 
				Retornar les dades dels productes la garantia dels quals sigui la 
				mateixa que la facilitada per parmetre 

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_IdGarantia		Identificador de la garantia del producte
		p_IncloureBaixes		Indica si es mostraran tamb els que estiguin donats de baixa
						Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdGarantia		IN		Productes.IdGarantia%TYPE,
	p_IncloureBaixes		IN		NUMBER:=0       
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdGarantia: ' || p_IdGarantia  || 
		' p_IncloureBaixes:' || p_IncloureBaixes;

	OPEN p_RS FOR
		SELECT * FROM Productes 
		WHERE IdGarantia = p_IdGarantia AND 
		CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
		DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/


COMMIT;
