/*------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Procediments del subsistema Agents
--------------------------------------------------*/


-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

COMMIT;


-------------------------------------------------------------------------------
-- Perfils
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_Perfil (
/******************************************************************************
	NOM:			Alta_Perfil
   
	REQUERIMENT N:	61
   
	DESCRIPCI:		Dna dalta un perfil professional al nostre sistema
                 
	PARMETRES:

		p_Perfil	Nom del perfil
		p_DataAlta	Data d'alta del perfil (per defecte pren la data del sistema)


******************************************************************************/
	p_Perfil		IN	Perfils.Perfil%TYPE,
	p_DataAlta		IN	Perfils.DataAlta%TYPE := Dia(SYSDATE)
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_Perfil: '
		|| p_Perfil
		|| ' p_DataAlta: '
		|| p_DataAlta;


	INSERT INTO Perfils (Perfil, DataAlta)
	VALUES (p_Perfil, Dia(p_DataAlta));

	-- Log
	v_Fi := SYSDATE;
	v_Res := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El perfil ' || p_Perfil || ' ja existeix';
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Perfil (
/******************************************************************************
	NOM:			Modificar_Perfil
   
	REQUERIMENT N:	62
   
	DESCRIPCI:		Permet modificar la informaci associada a un perfil

	PARMETRES:

	p_IdPerfil		Identificador del perfil
	p_Nom			Nom del perfil
	p_DataAlta		Data d'alta del perfil (Si no s'introdueix mantindr la data d'alta existent)

******************************************************************************/
	p_IdPerfil		IN	Perfils.IdPerfil%TYPE,
	p_Perfil		IN	Perfils.Perfil%TYPE,
	p_DataAlta		IN	Perfils.DataAlta%TYPE :=  NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdPerfil: '
		|| p_IdPerfil
		|| ' p_Perfil: '
		|| p_Perfil
		|| ' p_DataAlta: '
		|| p_DataAlta;

	UPDATE Perfils
	SET Perfil = p_Perfil,
		DataAlta = NVL (Dia(p_DataAlta), DataAlta)
	WHERE IdPerfil = p_IdPerfil AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: El perfil ' || p_IdPerfil || ' no existeix o est donat de baixa' ;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El perfil ' || p_Perfil || ' ja existeix';
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Baixa_Perfil (
/******************************************************************************
	NOM:			Baixa_Perfil
   
	REQUERIMENT N:	63
   
	DESCRIPCI:		Elimina o dna de baixa un perfil professional del nostre sistema, 
				proporcionant el seu identificador. 
				Si el perfil no t cap agent associat seliminar. En cas contrari, 
				es donar de baixa.

	PARMETRES:

		p_IdPerfil		Identificador del perfil
		p_DataBaixa		Data de baixa del perfil (Per defecte pren la del sistema)

******************************************************************************/
	p_IdPerfil		IN	Perfils.IdPerfil%TYPE, 
	p_DataBaixa		IN	Perfils.DataBaixa%TYPE:=Dia(SYSDATE)
   )
IS
	v_DataAlta		Perfils.DataAlta%TYPE;
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdPerfil: ' || p_IdPerfil || ' p_DataBaixa: ' || p_DataBaixa ;

	-- Comprova que el perfil existeixi i estigui donat d'alta
	v_Res:= 'El perfil amb IdPerfil=' || p_IdPerfil || ' no existeix o est donat de baixa';
	
	SELECT DataAlta INTO v_DataAlta 
	FROM Perfils WHERE IdPerfil = p_IdPerfil AND DataBaixa IS NULL;
   
	-- Comprova que la data de baixa no sigui anterior a la d'alta
	If v_DataAlta > p_DataBaixa THEN
		v_Res:= 'La data alta del perfil no pot ser anterior a la de baixa';
		raise_application_error (-20000, '');
	END IF;   

	DELETE Perfils
	WHERE IdPerfil = p_IdPerfil AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: El perfil ' || p_IdPerfil  || ' no existeix o est donat de baixa';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			-- No es pot eliminar perqu t agents associats, s'assignar data de baixa
			UPDATE Perfils
			SET DataBaixa = Dia(SYSDATE)
			WHERE IdPerfil = p_IdPerfil AND DataBaixa IS NULL;

			v_Res := 'OK';
		ELSE
			IF v_Res IS NULL
			THEN
				-- Error no controlat per codi
				v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
			ELSE
				-- Error controlat per codi
				v_Res := 'Error:' || v_Res;
			END IF;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Recuperar_Perfil (
/******************************************************************************
	NOM:			Recuperar_Perfil
   
	REQUERIMENT N:	64
   
	DESCRIPCI:		Permet recuperar un perfil professional del nostre sistema

	PARMETRES:

		p_IdPerfil	Identificador del perfil

******************************************************************************/
	p_IdPerfil	IN	Perfils.IdPerfil%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdPerfil: ' || p_IdPerfil;

	UPDATE Perfils
	SET DataBaixa = NULL
	WHERE IdPerfil = p_IdPerfil AND DataBaixa IS NOT NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No hi ha cap perfil donat de baixa amb identificador  ' || p_IdPerfil;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Perfil
/******************************************************************************
	NOM:			Obtenir_Perfil
   
	REQUERIMENT N: 65
   
	DESCRIPCI:		Permet obtenir tota la informaci complerta dun perfil professional, 
				a partir de lidentificador de perfil facilitat com a parmetre. 
				Si aquest parmetre s null, retornar els perfils el nom dels quals 
				contingui el nom que es passa per parmetre. 
				Si no sespecifica cap dels dos parmetres, retornar una relaci de 
				tots els perfils. Tamb tindrem lopci, per parmetre, de decidir si 
				han de sortir o no els perfils donats de baixa.
                 
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdPerfil		IdPerfil a cercar
		p_Perfil		Perfil a cercar
		p_IncloureBaixes	Indica si es mostraran tamb els que estiguin de baixa
					Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_IdPerfil		IN		Perfils.IdPerfil%TYPE:=Null,
	p_Perfil		IN		Perfils.Perfil%TYPE:=Null,
	p_IncloureBaixes	IN		NUMBER:=0    
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_IdPerfil: ' || p_IdPerfil || ' p_Perfil:' || p_Perfil
              || ' p_IncloureBaixes:' || p_IncloureBaixes;


 	IF p_IdPerfil IS NULL AND p_Perfil IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Perfils 
			WHERE CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE When 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
        
	ELSIF p_IdPerfil IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Perfils 
			WHERE IdPerfil = p_IdPerfil AND 
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
        
	ELSIF p_Perfil IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Perfils 
			WHERE UPPER(Perfil) LIKE '%' || UPPER(p_Perfil) || '%' AND 
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
	END IF;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

-------------------------------------------------------------------------------
-- Agents
-------------------------------------------------------------------------------


CREATE OR REPLACE PROCEDURE Alta_Agent (
/******************************************************************************
	NOM:			Alta_Agent 

	REQUERIMENT N:	66

	DESCRIPCI:		Dna dalta un agent al nostre sistema

	PARMETRES:

		p_Nom				Nom de l'agent
		p_Cognoms			Cognoms de l'agent
		p_DataAlta			Data d'alta de l'agent (Per defecte pren la data del sistema)

******************************************************************************/
	p_Nom				IN	Agents.Nom%TYPE,
	p_Cognoms			IN	Agents.Cognoms%TYPE,
	p_DataAlta			IN	Agents.DataAlta%TYPE := Dia(SYSDATE)
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_Nom: '
		|| p_Nom
		|| ' p_Cognoms: '
		|| p_Cognoms
		|| ' p_DataAlta: '
		|| p_DataAlta;

	INSERT INTO Agents (Nom, Cognoms, DataAlta)
	VALUES (p_Nom, p_Cognoms, Dia(p_DataAlta));
 
	-- Log
	v_Fi := SYSDATE;
	v_REs := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El agent' || p_Nom || ' ja existeix';
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Agent (
/******************************************************************************
	NOM:			Modificar_Agent

	REQUERIMENT N:	67

	DESCRIPCI:		Permet modificar la informaci associada a un agent

	PARMETRES:

		p_IdAgent			Identificador de l'agent
		p_Nom				Nom de l'agent
		p_Cognoms			Cognoms de l'agent
		p_DataAlta			Data d'alta de l'agent (Per defecte es deixa la data que ja t)

******************************************************************************/
	p_IdAgent		IN	Agents.IdAgent%TYPE,
	p_Nom			IN	Agents.Nom%TYPE,
	p_Cognoms		IN	Agents.Cognoms%TYPE, 
	p_DataAlta		IN	Agents.DataAlta%TYPE := NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdAgent: '
		|| p_IdAgent
		|| ' p_Nom: '
		|| p_Nom
		|| ' p_Cognoms: '
		|| p_Cognoms
		|| ' p_DataAlta: '
		|| p_DataAlta;

	UPDATE Agents
	SET Nom = p_Nom,
		Cognoms = p_Cognoms,
		DataAlta = NVL (Dia(p_DataAlta), DataAlta)
	WHERE IdAgent = p_IdAgent AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: El agent ' || p_IdAgent  || ' no existeix o est donat de baixa';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el agent ' || p_Nom;
		ELSIF SQLCODE = 100
		THEN
			v_Res := 'Error: El agent ' || p_IdAgent  || ' no existeix';  
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
        END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Baixa_Agent (
/******************************************************************************
	NOM:			Baixa_Agent

	REQUERIMENT N:	68

	DESCRIPCI:		Dna de baixa un agent del nostre sistema

	PARMETRES:

		p_IdAgent		Identificador de l'agent
		p_DataBaixa		Data de baixa de l'agent (Per defecte pren la del sistema)

******************************************************************************/
	p_IdAgent		IN	Agents.IdAgent%TYPE,
	p_DataBaixa		IN	Agents.DataBaixa%TYPE:=Dia(SYSDATE)   
   )
IS
	v_DataAlta		Agents.DataAlta%TYPE;   
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdAgent: ' || p_IdAgent||  ' p_DataBaixa: ' || p_DataBaixa;

	-- La data de baixa no pot ser anterior a la data d'alta
	SELECT DataAlta INTO v_DataAlta FROM Agents 
	WHERE IdAgent = p_IdAgent AND DataBaixa IS NULL;

	IF v_DataAlta > p_DataBaixa THEN
		v_Res:= 'La data de baixa no pot ser anterior a la de alta';
		raise_application_error (-20000, '');   
	END IF;
   
	DELETE Agents
	WHERE IdAgent = p_IdAgent;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: El agent ' || p_IdAgent  || ' no existeix o est donat de baixa';  
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			-- No es pot borrar perqu t registres d'altres taules associades , s'assignar la data de baixa         
			UPDATE Agents
			SET DataBaixa = p_DataBaixa 
			WHERE IdAgent = p_IdAgent AND DataBaixa IS NULL;

			v_Res := 'OK';
		ELSIF SQLCODE = 100
		THEN
			v_Res := 'Error: El agent ' || p_IdAgent  || ' no existeix o est donat de baixa';  
		ELSE
			IF v_Res IS NULL
			THEN
				-- Error no controlat per codi
				v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
			ELSE
				-- Error controlat per codi
				v_Res := 'Error:' || v_Res;
			END IF;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Recuperar_Agent (
/******************************************************************************
	NOM:			Recuperar_Agent

	REQUERIMENT N:	69

	DESCRIPCI:		Permet recuperar un agent del nostre sistema

	PARMETRES:

		p_IdAgent	Identificador de l'agent

******************************************************************************/
	p_IdAgent	IN	Agents.IdAgent%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdAgent: ' || p_IdAgent;

	UPDATE Agents
	SET DataBaixa = NULL
	WHERE IdAgent = p_IdAgent AND DataBaixa IS NOT NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: El agent ' || p_IdAgent  || ' no existeix o no est donat de baixa';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Agent
/******************************************************************************
	NOM:			Obtenir_Agent

	REQUERIMENT N:	70

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun agent, 
				a partir de lidentificador facilitat com a parmetre. 
				Si aquest parmetre s null, retornar els agents el nom 
				i/o cognoms dels quals contingui el nom que es passa per parmetre. 
				Si no sespecifica cap dels dos parmetres, retornar una relaci 
				de tots els agents. Tamb tindrem lopci, per parmetre, de decidir 
				si han de sortir o no els agents donats de baixa

	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdAgent		Identificador de l'agent a cercar
		p_Nom			Nom de l'agent a cercar
		p_IncloureBaixes	Indica si es mostraran tamb els que estiguin donats de baixa
					Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_IdAgent		IN		Agents.IdAgent%TYPE:=Null,
	p_Nom			IN		Agents.Nom%TYPE:=Null,
	p_IncloureBaixes	IN		NUMBER:=0    
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_IdAgent: ' || p_IdAgent || ' p_Nom:' || p_Nom
              || ' p_IncloureBaixes:' || p_IncloureBaixes;


 	IF p_IdAgent IS NULL AND p_Nom IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Agents 
			WHERE CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE When 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
        
	ELSIF p_IdAgent IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Agents 
			WHERE IdAgent = p_IdAgent AND 
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
        
	ELSIF p_Nom IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Agents 
			WHERE UPPER(Nom) LIKE '%' || UPPER(p_Nom) || '%' AND 
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
	END IF;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Agent_Data
/******************************************************************************
	NOM:			Obtenir_Agent_Data

	REQUERIMENT N:	71

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun o varis agents, 
				a partir de les dates facilitades com a parmetre. 
				Retornar les dades dels agents la data dels quals estigui compresa 
				entre les dates facilitades.

				Si noms es facilita la data dinici, retornar els agents a partir daquesta data.
				Si noms es facilita la data final, retornar els agents fins aquesta data.
				Si no es facilita cap data, retornar la totalitat dels agents.

				Podrem especificar per parmetre si el filtre aplica a la 
				data dalta (A) o a la data de baixa (B).

				Tamb per parmetre, especificarem si volem que tamb surtin els agents donats de baixa
                 
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_Tipus		Tipus de filtre de data
		p_DataDesde		Data inici
		p_DataFi		Data fi (per defecte ser la del sistema)
		p_IncloureBaixes	Indica si es mostraran tamb els que estiguin donats de baixa
					Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/
( 
	p_RS			IN OUT	sys_refcursor, 
	p_Tipus		IN		VARCHAR2,  
	p_DataDesde		IN		Agents.DataAlta%TYPE:=Null,
	p_DataFi		IN		Agents.DataAlta%TYPE:=Null,
	p_IncloureBaixes	IN		NUMBER:=0      
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT), p_Tipus: ' || p_Tipus ||  
		' p_DataDesde: ' || p_DataDesde  
		|| ' p_DataFi: ' || p_DataFi
		|| ' p_IncloureBaixes:' || p_IncloureBaixes;

	IF p_DataDesde > p_DataFi THEN
		v_Res := 'La data de inici no pot ser superior a la data final';
		raise_application_error (-20000, '');
	END IF;

	CASE UPPER(p_Tipus) 
	WHEN 'A' THEN 

		OPEN p_RS FOR
			SELECT * FROM Agents WHERE
			CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
				DataAlta >= p_DataDesde THEN 1 ELSE 0 END END = 1 AND 
			CASE WHEN p_DataFi IS NULL THEN 1 ELSE CASE WHEN 
				DataAlta <= p_DataFi THEN 1 ELSE 0 END END = 1 AND
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
				DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
            
	WHEN 'B' THEN 

		OPEN p_RS FOR
			SELECT * FROM Agents WHERE
			CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
				DataBaixa >= p_DataDesde THEN 1 ELSE 0 END END = 1 AND 
			CASE WHEN p_DataFi IS NULL THEN 1 ELSE CASE WHEN 
				DataBaixa <= p_DataFi THEN 1 ELSE 0 END END = 1 ;

	ELSE
		v_Res := 'El tipus ha de ser algun dels segents: A->Alta, B->Baixa';
		raise_application_error (-20000, '');
	END CASE;


	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;
      
	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

-------------------------------------------------------------------------------
-- Perfils Agents
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_Perfil_Agent (
/******************************************************************************
	NOM:			Alta_Perfil_Agent 
   
	REQUERIMENT N:	72

	DESCRIPCI:		Permet assignar un determinat perfil a un determinat agent, 
				especificant lidentificador dambds

	PARMETRES:

		p_IdPerfil	Identificador del perfil
		p_IdAgent	Identificador de l'agent

******************************************************************************/
	p_IdPerfil	IN	AgentsPerfils.IdPerfil%TYPE,
	p_IdAgent	IN	AgentsPerfils.IdAgent%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdPerfil: ' || 
		p_IdPerfil || ' 
		p_IdAgent: ' || 
		p_IdAgent;

	INSERT INTO AgentsPerfils (IdPerfil, IdAgent)
	VALUES (p_IdPerfil, p_IdAgent);

	-- Log
	v_Fi := SYSDATE;
	v_Res := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Agent ' || p_IdAgent || ' ja t el perfil ' || p_IdPerfil;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix agent i/o perfil';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Eliminar_Perfil_Agent (
/******************************************************************************
	NOM:			Eliminar_Perfil_Agent
   
	REQUERIMENT N:	73

	DESCRIPCI:		Permet eliminar un determinat perfil a un determinat agent, 
				especificant lidentificador dambds

	PARMETRES:

		p_IdPerfil	Identificador del perfil
		p_IdAgent	Identificador de l'agent

******************************************************************************/
	p_IdPerfil	IN	AgentsPerfils.IdPerfil%TYPE,
	p_IdAgent	IN	AgentsPerfils.IdAgent%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdPerfil: ' || p_IdPerfil || ' p_IdAgent: ' || p_IdAgent;

	DELETE AgentsPerfils
	WHERE IdAgent = p_IdAgent AND IdPerfil = p_IdPerfil;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix assignaci perfil='
			|| p_IdPerfil
			|| ', agent='
			|| p_IdAgent;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Perfil_Agent
/******************************************************************************
	NOM:			Obtenir_Perfil_Agent
   
	REQUERIMENTO N:	74
   
	DESCRIPCI:		Permet obtenir la relaci de perfils que tenen assignats els agents, 
				en funci dels identificadors passats com a parmetres.
				Si especifiquem els dos identificadors (agent i perfil), 
				obtindrem lassignaci corresponent si aquesta existeix.
				Si no especifiquem lidentificador de lagent (null), obtindrem una 
				relaci dels agents que tinguin assignat el perfil que correspon a lidentificador donat.
				Si no especifiquem lidentificador del perfil (null), obtindrem una 
				relaci dels perfils que tingui assignat lagent que correspon a lidentificador donat.
				Si no especifiquem cap dels dos identificadors (null), 
				obtindrem una relaci de totes les assignacions agent  perfil fetes

	PARMETROS:

		p_RS		Recordset amb el resultat
		p_IdPerfil	Identificador del perfil
		p_IdAgent	Identificador de l'agent

******************************************************************************/

(   

	p_RS			IN OUT	sys_refcursor,
	p_IdPerfil		IN		AgentsPerfils.IdPerfil%TYPE:=NULL,
	p_IdAgent		IN		AgentsPerfils.IdAgent%TYPE:=NULL
    ) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := '  p_RS (IN OUT) , p_IdPerfil: ' || p_IdPerfil || 'p_IdAgent: ' || p_IdAgent;

	OPEN p_RS FOR
		SELECT A.*, P.IdPerfil, P.Perfil FROM AgentsPerfils AP, Agents A, Perfils P
		WHERE A.DataBaixa IS NULL AND AP.IdAgent = A.IdAgent AND AP.IdPerfil = P.IdPerfil AND 
		CASE WHEN p_IdPerfil IS NULL THEN 1 ELSE CASE WHEN 
			AP.IdPerfil = p_IdPerfil THEN 1 ELSE 0 END END = 1 AND   
		CASE WHEN p_IdAgent Is Null THEN 1 ELSE CASE WHEN 
			AP.IdAgent = p_IdAgent THEN 1 ELSE 0 END END = 1 ;
    
	v_Res:= 'OK';

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);   
END;
/

-------------------------------------------------------------------------------
-- Provncies Agents
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_Provincia_Agent (
/******************************************************************************
	NOM:			Alta_Provincia_Agent 
   
	REQUERIMENT N:	75

	DESCRIPCI:		Permet assignar una determinada provncia a un determinat agent, 
				especificant lidentificador dambds

	PARMETRES:

		p_IdProvincia	Identificador de la provncia
		p_IdAgent		Identificador de l'agent

******************************************************************************/
	p_IdProvincia	IN	AgentsProvincies.IdProvincia%TYPE,
	p_IdAgent		IN	AgentsProvincies.IdAgent%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdProvincia: ' || 
		p_IdProvincia || ' 
		p_IdAgent: ' || 
		p_IdAgent;

	INSERT INTO AgentsProvincies (IdProvincia, IdAgent)
	VALUES (p_IdProvincia, p_IdAgent);

	-- Log
	v_Fi := SYSDATE;
	v_Res := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Agent ' || p_IdAgent || ' ja t la provncia ' || p_IdProvincia;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix agent i/o provncia';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Eliminar_Provincia_Agent (
/******************************************************************************
	NOM:			Eliminar_Provincia_Agent
   
	REQUERIMENT N:	76

	DESCRIPCI:		Permet eliminar una determinada provncia a un determinat agent, 
				especificant lidentificador dambds

	PARMETRES:

		p_IdProvincia	Identificador de la provncia
		p_IdAgent		Identificador de l'agent

******************************************************************************/
	p_IdProvincia	IN	AgentsProvincies.IdProvincia%TYPE,
	p_IdAgent		IN	AgentsProvincies.IdAgent%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdProvincia: ' || p_IdProvincia || ' p_IdAgent: ' || p_IdAgent;

	DELETE AgentsProvincies
	WHERE IdAgent = p_IdAgent AND IdProvincia = p_IdProvincia;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix assignaci provncia='
			|| p_IdProvincia
			|| ', agent='
			|| p_IdAgent;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Provincia_Agent
/******************************************************************************
	NOM:			Obtenir_Provincia_Agent
   
	REQUERIMENTO N:	77
   
	DESCRIPCI:		Permet obtenir la relaci de provncies que tenen assignats els agents, 
				en funci dels identificadors passats com a parmetres.
				Si especifiquem els dos identificadors (agent i provncia), 
				obtindrem lassignaci corresponent si aquesta existeix.
				Si no especifiquem lidentificador de lagent (null), obtindrem una 
				relaci dels agents que tinguin assignada la provncia que correspon a lidentificador donat.
				Si no especifiquem lidentificador de la provncia (null), obtindrem una 
				relaci de les provncies que tingui assignades lagent que correspon a lidentificador donat.
				Si no especifiquem cap dels dos identificadors (null), 
				obtindrem una relaci de totes les assignacions agent  provncia fetes

	PARMETROS:

		p_RS			Recordset amb el resultat
		p_IdProvincia	Identificador de la provncia
		p_IdAgent		Identificador de l'agent

******************************************************************************/

(   

	p_RS			IN OUT	sys_refcursor,
	p_IdProvincia	IN		AgentsProvincies.IdProvincia%TYPE:=NULL,
	p_IdAgent		IN		AgentsProvincies.IdAgent%TYPE:=NULL
    ) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := '  p_RS (IN OUT) , p_IdProvincia: ' || p_IdProvincia || 'p_IdAgent: ' || p_IdAgent;

	OPEN p_RS FOR
		SELECT A.*, P.IdProvincia, P.Provincia FROM AgentsProvincies AP, Agents A, Provincies P
		WHERE A.DataBaixa IS NULL AND AP.IdAgent = A.IdAgent AND AP.IdProvincia = P.IdProvincia AND 
		CASE WHEN p_IdProvincia IS NULL THEN 1 ELSE CASE WHEN 
			AP.IdProvincia = p_IdProvincia THEN 1 ELSE 0 END END = 1 AND   
		CASE WHEN p_IdAgent Is Null THEN 1 ELSE CASE WHEN 
			AP.IdAgent = p_IdAgent THEN 1 ELSE 0 END END = 1 ;
    
	v_Res:= 'OK';

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);   
END;
/


COMMIT;
