/*------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Procediments del subsistema Incidncies
--------------------------------------------------*/


-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

COMMIT;


-------------------------------------------------------------------------------
-- Estats
-------------------------------------------------------------------------------


CREATE OR REPLACE PROCEDURE Alta_Estat (
/******************************************************************************
	NOM:			Alta_Estat 

	REQUERIMENT N:	78

	DESCRIPCI:		Dna dalta un estat al nostre sistema

	PARMETROS:

		p_Estat 		Nom de l'estat

******************************************************************************/
	p_Estat 		IN	Estats.Estat%TYPE)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_Estat: ' || p_Estat;

	INSERT INTO Estats (Estat)
	VALUES (p_Estat);

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := 'OK';
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El estat ' || p_Estat || ' ja existeix';
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Estat (
/******************************************************************************
	NOM:			Modificar_Estat
   
	REQUERIMENT N:	79
   
	DESCRIPCI:		Permet modificar un estat del nostre sistema
                 
	PARMETRES:

		p_IdEstat	Identificador de l'estat
		p_Estat	Nom de l'estat

******************************************************************************/
	p_IdEstat	IN	Estats.IdEstat%TYPE,
	p_Estat	IN	Estats.Estat%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdEstat: ' ||
		p_IdEstat ||
		' p_Estat: ' ||
		p_Estat;

	UPDATE Estats
	SET Estat = p_Estat
	WHERE IdEstat = p_IdEstat;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el estat amb IdEstat=' || p_IdEstat;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El estat ' || p_Estat || ' ja existeix';             
        ELSIF SQLCODE = -1407
        THEN
            v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

      DBMS_OUTPUT.put_line (v_Res);
      v_Fi := SYSDATE;
      v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Eliminar_Estat (
/******************************************************************************
	NOM:			Eliminar_Estat 
                 
	REQUERIMENT N:	80

	DESCRIPCI:		Elimina un estat del nostre sistema

	PARMETRES:

		p_IdEstat	Identificador de l'estat

******************************************************************************/
	p_IdEstat	IN	Estats.IdEstat%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdEstat: ' || p_IdEstat;

	DELETE Estats WHERE IdEstat = p_IdEstat;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el estat amb IdEstat=' || p_IdEstat;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE =  -2292 
		THEN
			-- No es pot eliminar perqu t incidncies associades
			v_Res := 'No es pot eliminar el estat ' || p_IdEstat || ' perqu t incidncies associades';
		END IF;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Estat
/******************************************************************************
	NOM:			Obtenir_Estat

	REQUERIMENT N:	81

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun estat, 
				a partir de lidentificador destat facilitat com a parmetre. 
				Si aquest parmetre s null, retornar els estats el nom dels 
				quals contingui el nom que es passa per parmetre. 
				Si no sespecifica cap dels dos parmetres, retornar una relaci de tots els estats 

	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdEstat		Identificador de l'estat a cercar
		p_Estat		Estat a cercar

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_IdEstat		IN		Estats.IdEstat%TYPE:=Null,
	p_Estat		IN		Estats.Estat%TYPE:=Null
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := 'p_RS (IN OUT), p_IdEstat: ' || p_IdEstat || ' p_Estat:' || p_Estat ;

	IF p_IdEstat IS NULL AND p_Estat IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Estats;
	ELSIF p_IdEstat IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Estats WHERE IdEstat = p_IdEstat;
	ELSIF p_Estat IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Estats WHERE UPPER(Estat) LIKE '%' || UPPER(p_Estat) || '%';
	END IF;

	v_Res:= 'OK';

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

-------------------------------------------------------------------------------
-- Prioritats
-------------------------------------------------------------------------------


CREATE OR REPLACE PROCEDURE Alta_Prioritat (
/******************************************************************************
	NOM:			Alta_Prioritat 

	REQUERIMENT N:	82

	DESCRIPCI:		Dna dalta una prioritat al nostre sistema

	PARMETROS:

		p_Prioritat		Nom de la prioritat

******************************************************************************/
	p_Prioritat		IN	Prioritats.Prioritat%TYPE)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_Prioritat: ' || p_Prioritat;

	INSERT INTO Prioritats (Prioritat)
	VALUES (p_Prioritat);

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := 'OK';
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'La prioritat ' || p_Prioritat || ' ja existeix';
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Prioritat (
/******************************************************************************
	NOM:			Modificar_Prioritat
   
	REQUERIMENT N:	83
   
	DESCRIPCI:		Permet modificar una prioritat del nostre sistema
                 
	PARMETRES:

		p_IdPrioritat	Identificador de la prioritat
		p_Prioritat		Nom de la prioritat

******************************************************************************/
	p_IdPrioritat	IN	Prioritats.IdPrioritat%TYPE,
	p_Prioritat		IN	Prioritats.Prioritat%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdPrioritat: ' ||
		p_IdPrioritat ||
		' p_Prioritat: ' ||
		p_Prioritat;

	UPDATE Prioritats
	SET Prioritat = p_Prioritat
	WHERE IdPrioritat = p_IdPrioritat;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix la prioritat amb IdPrioritat=' || p_IdPrioritat;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'La prioritat ' || p_Prioritat || ' ja existeix';             
        ELSIF SQLCODE = -1407
        THEN
            v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

      DBMS_OUTPUT.put_line (v_Res);
      v_Fi := SYSDATE;
      v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Eliminar_Prioritat (
/******************************************************************************
	NOM:			Eliminar_Prioritat
                 
	REQUERIMENT N:	84

	DESCRIPCI:		Elimina una prioritat del nostre sistema

	PARMETRES:

		p_IdPrioritat	Identificador de la prioritat

******************************************************************************/
	p_IdPrioritat	IN	Prioritats.IdPrioritat%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdPrioritat: ' || p_IdPrioritat;

	DELETE Prioritats WHERE IdPrioritat = p_IdPrioritat;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix la prioritat amb IdPrioritat=' || p_IdPrioritat;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE =  -2292 
		THEN
			-- No es pot eliminar perqu t incidncies associades
			v_Res := 'No es pot eliminar la prioritat ' || p_IdPrioritat || ' perqu t incidncies associades';
		END IF;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Prioritat
/******************************************************************************
	NOM:			Obtenir_Prioritat

	REQUERIMENT N:	85

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna prioritat, 
				a partir de lidentificador de prioritat facilitat com a parmetre. 
				Si aquest parmetre s null, retornar les prioritats el nom de les 
				quals contingui el nom que es passa per parmetre. 
				Si no sespecifica cap dels dos parmetres, retornar una relaci de totes les prioritats 

	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdPrioritat	Identificador de la prioritat a cercar
		p_Prioritat		Prioritat a cercar

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_IdPrioritat	IN		Prioritats.IdPrioritat%TYPE:=Null,
	p_Prioritat		IN		Prioritats.Prioritat%TYPE:=Null
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := 'p_RS (IN OUT), p_IdPrioritat: ' || p_IdPrioritat || ' p_Prioritat:' || p_Prioritat ;

	IF p_IdPrioritat IS NULL AND p_Prioritat IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Prioritats;
	ELSIF p_IdPrioritat IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Prioritats WHERE IdPrioritat = p_IdPrioritat;
	ELSIF p_Prioritat IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Prioritats WHERE UPPER(Prioritat) LIKE '%' || UPPER(p_Prioritat) || '%';
	END IF;

	v_Res:= 'OK';

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

-------------------------------------------------------------------------------
-- Tipus Incidncia
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_TipusIncidencia (
/******************************************************************************
	NOM:			Alta_TipusIncidencia

	REQUERIMENT N:	86

	DESCRIPCIN:	Dna dalta un tipus d'incidncia al nostre sistema

	PARMETRES:

		p_TipusIncidencia		Tipus d'incidncia
		p_DataAlta			Data d'alta

******************************************************************************/
	p_TipusIncidencia		IN	TipusIncidencia.TipusIncidencia%TYPE,
	p_DataAlta			IN	TipusIncidencia.DataAlta%TYPE := Dia(SYSDATE)
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
         ' p_TipusIncidencia: '
      || p_TipusIncidencia
      || ' p_DataAlta: '
      || p_DataAlta;

	INSERT INTO TipusIncidencia (TipusIncidencia , DataAlta)
	VALUES (p_TipusIncidencia , Dia(p_DataAlta));

	-- Log
	v_Fi := SYSDATE;
	v_Res := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el tipus incidncia ' || p_TipusIncidencia;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_TipusIncidencia (
/******************************************************************************
	NOM:			Modificar_TipusIncidencia

	REQUERIMENT N:	87

	DESCRIPCI:		Permet modificar un tipus d'incidncia del nostre sistema

	PARMETRES:

		p_IdTipusIncidencia	Identificador del tipus d'incidncia
		p_TipusIncidencia		Tipus d'incidncia
		p_DataAlta			Data d'alta (Si no s'introdueix mantindr la data d'alta existent)

******************************************************************************/
	p_IdTipusIncidencia	IN	TipusIncidencia.IdTipusIncidencia%TYPE,
	p_TipusIncidencia		IN	TipusIncidencia.TipusIncidencia%TYPE,
	p_DataAlta			IN	TipusIncidencia.DataAlta%TYPE := NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdTipusIncidencia: '
		|| p_IdTipusIncidencia
		|| ' p_TipusIncidencia: '
		|| p_TipusIncidencia
		|| ' p_DataAlta: '
		|| p_DataAlta;

	UPDATE TipusIncidencia
	SET TipusIncidencia = p_TipusIncidencia,
		DataAlta = NVL (DIA(p_DataAlta), DataAlta)
	WHERE IdTipusIncidencia = p_IdTipusIncidencia AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix o est donat de baixa el tipus incidncia amb IdTipusIncidencia=' || p_IdTipusIncidencia;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el TipusIncidencia ' || p_TipusIncidencia;
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Baixa_TipusIncidencia (
/******************************************************************************
	NOM:			Baixa_TipusIncidencia 

	REQUERIMENT N:	88

	DESCRIPCI:		Elimina o dna de baixa un tipus d'incidncia del nostre sistema, 
				proporcionant el seu identificador. 
				Si el tipus d'incidncia no t cap incidncia associada seliminar. 
				En cas contrari, es donar de baixa

	PARMETRES:

		p_IdTipusIncidencia	Identificador del tipus d'incidncia
		p_DataBaixa			Data de baixa del tipus d'incidncia 

******************************************************************************/
	p_IdTipusIncidencia	IN	TipusIncidencia.IdTipusIncidencia%TYPE,
	p_DataBaixa			IN	TipusIncidencia.DataBaixa%TYPE:=Dia(SYSDATE)
   )
IS
	v_DataAlta		TipusIncidencia.DataAlta%TYPE;
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdTipusIncidencia: ' || p_IdTipusIncidencia || ' p_DataBaixa: ' || p_DataBaixa ;

	-- Comprova que existeixi i estigui donat d'alta
	v_Res:= 'El tipus incidncia amb IdTipusIncidencia=' || p_IdTipusIncidencia || ' no existeix o est donat de baixa';

	SELECT DataAlta INTO v_DataAlta 
	FROM TipusIncidencia WHERE IdTipusIncidencia = p_IdTipusIncidencia AND DataBaixa IS NULL;
   
	-- Comprova que la data de baixa no sigui anterior a la d'alta
	IF v_DataAlta > p_DataBaixa THEN
		v_Res:= 'La data alta no pot ser anterior a la de baixa';
		raise_application_error (-20000, '');
	END IF;   

	DELETE TipusIncidencia
	WHERE IdTipusIncidencia = p_IdTipusIncidencia AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el TipusIncidencia ' || p_IdTipusIncidencia || ', o ja est donat de baixa ';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			-- No es pot borrar perqu t incidncies associades, s'assignar la data de baixa
			UPDATE TipusIncidencia
			SET DataBaixa = Dia(SYSDATE)
			WHERE IdTipusIncidencia = p_IdTipusIncidencia AND DataBaixa IS NULL;

			v_Res := 'OK';
		ELSE
			IF v_Res IS NULL
			THEN
				-- Error no controlat per codi
				v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
			ELSE
				-- Error controlat per codi
				v_Res := 'Error:' || v_Res;
			END IF;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Recuperar_TipusIncidencia (
/******************************************************************************
	NOM:			Recuperar_TipusIncidencia

	REQUERIMENT N:	89

	DESCRIPCI:		Permet recuperar un tipus d'incidncia del nostre sistema
                
	PARMETRES:                 

		p_IdTipusIncidencia	Identificador del tipus d'incidncia

******************************************************************************/
	p_IdTipusIncidencia		IN	TipusIncidencia.IdTipusIncidencia%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdTipusIncidencia: ' || p_IdTipusIncidencia;

	UPDATE TipusIncidencia
	SET DataBaixa = NULL
	WHERE IdTipusIncidencia = p_IdTipusIncidencia AND DataBaixa IS NOT NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el TipusIncidencia ' || p_IdTipusIncidencia;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_TipusIncidencia
/******************************************************************************
	NOM:			Obtenir_TipusIncidencia

	REQUERIMENT N:	90

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun tipus dincidncia, 
				a partir de lidentificador de tipus dincidncia facilitat com a parmetre. 
				Si aquest parmetre s null, retornar els tipus dincidncia el nom dels 
				quals contingui el nom que es passa per parmetre. 
				Si no sespecifica cap dels dos parmetres, retornar una relaci de tots 
				els tipus dincidncia. Tamb tindrem lopci, per parmetre, de decidir si 
				han de sortir o no els tipus dincidncia donats de baixa  
                  
	PARMETRES:

		p_RS				Recordset amb el resultat
		p_IdTipusIncidencia	IdTipusIncidencia a cercar
		p_TipusIncidencia		TipusIncidencia a cercar
		p_IncloureBaixes		Indica si es mostraran tamb els que estiguin de baixa
						Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdTipusIncidencia	IN		TipusIncidencia.IdTipusIncidencia%TYPE:=Null,
	p_TipusIncidencia		IN		TipusIncidencia.TipusIncidencia%TYPE:=Null,
	p_IncloureBaixes		IN		NUMBER:=0        
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_IdTipusIncidencia: ' || p_IdTipusIncidencia || ' p_TipusIncidencia:' || p_TipusIncidencia 
		|| ' p_IncloureBaixes:' || p_IncloureBaixes;

	IF p_IdTipusIncidencia IS NULL AND p_TipusIncidencia IS NULL THEN 
		OPEN p_RS FOR
			SELECT * FROM TipusIncidencia
			WHERE CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;

	ELSIF p_IdTipusIncidencia IS NOT NULL THEN 
		OPEN p_RS FOR
			SELECT * FROM TipusIncidencia WHERE IdTipusIncidencia = p_IdTipusIncidencia AND 
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
        
	ELSIF p_TipusIncidencia IS NOT NULL THEN 
		OPEN p_RS FOR
			SELECT * FROM TipusIncidencia WHERE UPPER(TipusIncidencia) LIKE '%' || UPPER(p_TipusIncidencia) || '%' AND
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
	END IF;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

-------------------------------------------------------------------------------
-- Incidncies
-------------------------------------------------------------------------------


CREATE OR REPLACE PROCEDURE Alta_Incidencia (
/******************************************************************************
	NOM:			Alta_Incidencia

	REQUERIMENT N:	91

	DESCRIPCI:		Dna dalta una incidncia al nostre sistema

	PARMETRES:

		p_IdTipusIncidencia	Tipus d'incidncia
		p_Descripcio		Descripci de la incidncia
		p_IdClient			Client que obre la incidncia
		p_IdProducte		Producte al que afecta la incidncia
		p_IdAgent			Agent que dna d'alta la incidncia
		p_IdPrioritat		Prioritat assignada a la incidncia
		p_IdEstat			Estat actual de la incidncia
		p_DataAlta			Data d'alta de la incidncia (Per defecte pren la data del sistema)
		p_Observacions		Observacions a la incidncia

******************************************************************************/
	p_IdTipusIncidencia	IN	Incidencies.IdTipusIncidencia%TYPE,
	p_Descripcio		IN	Incidencies.Descripcio%TYPE, 
	p_IdClient			IN	Incidencies.IdClient%TYPE,
	p_IdProducte		IN	Incidencies.IdProducte%TYPE,
	p_IdAgent			IN	Incidencies.IdAgent%TYPE,
	p_IdPrioritat		IN	Incidencies.IdPrioritat%TYPE,
	p_IdEstat			IN	Incidencies.IdEstat%TYPE,
	p_DataAlta			IN	Incidencies.DataAlta%TYPE := Dia(SYSDATE),
	p_Observacions		IN	Incidencies.Observacions%TYPE := NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdTipusIncidencia: '
		|| p_IdTipusIncidencia
		|| ' p_Descripcio: '
		|| p_Descripcio
		|| ' p_IdClient: '
		|| p_IdClient
		|| ' p_IdProducte: '
		|| p_IdProducte
		|| ' p_IdAgent: '
		|| p_IdAgent
		|| ' p_IdPrioritat: '
		|| p_IdPrioritat
		|| ' p_IdEstat: '
		|| p_IdEstat
		|| ' p_DataAlta: '
		|| p_DataAlta
		|| ' p_Observacions: '
		|| p_Observacions;

	INSERT INTO Incidencies (IdTipusIncidencia, Descripcio, IdClient, IdProducte, IdAgent, IdPrioritat, IdEstat, DataAlta, Observacions)
	VALUES (p_IdTipusIncidencia, p_Descripcio, p_IdClient, p_IdProducte, p_IdAgent, p_IdPrioritat, p_IdEstat, Dia(p_DataAlta), p_Observacions);
 
	-- Log
	v_Fi := SYSDATE;
	v_REs := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'El identificador de tipus incidncia i/o de client i/o de producte i/o de agent i/o de prioritat i/o de estat no sn correctes';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Incidencia (
/******************************************************************************
	NOM:			Modificar_Incidencia

	REQUERIMENT N:	92

	DESCRIPCI:		Permet modificar la informaci associada a una incidncia, 
				proporcionant el seu identificador. 
				La incidncia es dna per tancada quan el seu estat s Tancada 
				i t informada la data de tancament

	PARMETRES:

		p_IdIncidencia		Identificador de la incidncia
		p_IdTipusIncidencia	Tipus d'incidncia
		p_Descripcio		Descripci de la incidncia
		p_IdClient			Client que obre la incidncia
		p_IdProducte		Producte al que afecta la incidncia
		p_IdAgent			Agent que dna d'alta la incidncia
		p_IdPrioritat		Prioritat assignada a la incidncia
		p_IdEstat			Estat actual de la incidncia
		p_DataAlta			Data d'alta de la incidncia (Si no s'introdueix mantindr la data d'alta existent)
		p_DataTancament		Data de tancament de la incidncia
		p_Observacions		Observacions a la incidncia

******************************************************************************/
	p_IdIncidencia		IN	Incidencies.IdIncidencia%TYPE,
	p_IdTipusIncidencia	IN	Incidencies.IdTipusIncidencia%TYPE,
	p_Descripcio		IN	Incidencies.Descripcio%TYPE, 
	p_IdClient			IN	Incidencies.IdClient%TYPE,
	p_IdProducte		IN	Incidencies.IdProducte%TYPE,
	p_IdAgent			IN	Incidencies.IdAgent%TYPE,
	p_IdPrioritat		IN	Incidencies.IdPrioritat%TYPE,
	p_IdEstat			IN	Incidencies.IdEstat%TYPE,
	p_DataAlta			IN	Incidencies.DataAlta%TYPE := NULL,
	p_DataTancament		IN	Incidencies.DataTancament%TYPE,
	p_Observacions		IN	Incidencies.Observacions%TYPE
)
IS
	v_tmp_DataAlta	Incidencies.DataAlta%TYPE := NULL;
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdIncidencia: '
		|| p_IdIncidencia
		|| ' p_IdTipusIncidencia: '
		|| p_IdTipusIncidencia
		|| ' p_Descripcio: '
		|| p_Descripcio
		|| ' p_IdClient: '
		|| p_IdClient
		|| ' p_IdProducte: '
		|| p_IdProducte
		|| ' p_IdAgent: '
		|| p_IdAgent
		|| ' p_IdPrioritat: '
		|| p_IdPrioritat
		|| ' p_IdEstat: '
		|| p_IdEstat
		|| ' p_DataAlta: '
		|| p_DataAlta
		|| ' p_DataTancament: '
		|| p_DataTancament
		|| ' p_Observacions: '
		|| p_Observacions;

	-- Si p_DataAlta s NULL, agafem la data d'alta de la incidncia
	IF p_DataAlta IS NULL
	THEN
		SELECT DataAlta 
		INTO v_tmp_DataAlta
		FROM Incidencies
		WHERE IdIncidencia = p_IdIncidencia;
	ELSE
		v_tmp_DataAlta := p_DataAlta ;
	END IF;

	-- Data de Alta < Data tancament
	IF v_tmp_DataAlta > p_DataTancament
	THEN
		v_Res := 'La data de tancament ha de ser posterior a la de alta';
		raise_application_error (-20000, '');
	END IF;

	UPDATE Incidencies
	SET IdTipusIncidencia = p_IdTipusIncidencia,
		Descripcio = p_Descripcio,
		IdClient= p_IdClient,
		IdProducte = p_IdProducte,
		IdAgent = p_IdAgent,
		IdPrioritat = p_IdPrioritat,
		IdEstat= p_IdEstat,
		DataAlta = Dia(v_tmp_DataAlta),
		DataTancament = p_DataTancament,
		Observacions = p_Observacions
	WHERE IdIncidencia = p_IdIncidencia AND DataTancament IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix o est tancada la incidncia amb IdIncidencia=' || p_IdIncidencia;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix la incidncia amb IdIncidencia=' || p_IdIncidencia;
		ELSIF SQLCODE = 100
		THEN
			v_Res := 'No existeix la incidncia amb IdIncidencia=' || p_IdIncidencia;  
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
        ELSIF SQLCODE = -2291
        THEN
            v_Res := 'El identificador de tipus incidncia i/o de client i/o de producte i/o de agent i/o de prioritat i/o de estat no sn correctes';
        END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Incidencia_TipusInc
/******************************************************************************
	NOM:			Obtenir_Incidencia_TipusIncidencia

	REQUERIMENT N:	93

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries incidncies, 
				a partir del tipus dincidncia facilitat com a parmetre. 
				Retornar les dades de les incidncies el tipus de les quals sigui com el tipus parmetre 

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_IdTipusIncidencia	Identificador del tipus d'incidncia

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdTipusIncidencia	IN		Incidencies.IdTipusIncidencia%TYPE
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdTipusIncidencia: ' || p_IdTipusIncidencia;

	OPEN p_RS FOR
		SELECT * FROM Incidencies 
		WHERE IdTipusIncidencia = p_IdTipusIncidencia;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Incidencia_Desc
/******************************************************************************
	NOM:			Obtenir_Incidencia_Desc

	REQUERIMENT N:	94

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries incidncies, 
				a partir de la descripci facilitada com a parmetre. 
				Retornar les dades de les incidncies la descripci de les quals contingui 
				la facilitada com a parmetre

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_Descripcio		Descripci a cercar

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_Descripcio	IN		Incidencies.Descripcio%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_Descripcio: ' || p_Descripcio;


	OPEN p_RS FOR
		SELECT * FROM Incidencies 
		WHERE UPPER(Descripcio) LIKE '%' || UPPER(p_Descripcio) || '%';

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Incidencia_Data
/******************************************************************************
	NOM:			Obtenir_Incidencia_Data

	REQUERIMENT N:	95

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries incidncies, 
				a partir de les dates facilitades com a parmetre. 
				Retornar les dades de les incidncies la data de les quals estigui compresa entre les dates facilitades.

				Si noms es facilita la data dinici, retornar les incidncies a partir daquesta data.
				Si noms es facilita la data final, retornar les incidncies fins aquesta data.
				Si no es facilita cap data, retornar la totalitat de les incidncies.

				Podrem especificar per parmetre si el filtre aplica a la data dalta (A), o a la data de tancament (T)
                 
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_Tipus		Tipus de filtre de data
		p_DataDesde		Data inici
		p_DataFi		Data fi (per defecte ser la del sistema)

******************************************************************************/
( 
	p_RS			IN OUT	sys_refcursor, 
	p_Tipus		IN		VARCHAR2,  
	p_DataDesde		IN		Incidencies.DataAlta%TYPE:=Null,
	p_DataFi		IN		Incidencies.DataAlta%TYPE:=Null
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT), p_Tipus: ' || p_Tipus ||  
		' p_DataDesde: ' || p_DataDesde  
		|| ' p_DataFi: ' || p_DataFi;

	IF p_DataDesde > p_DataFi THEN
		v_Res := 'La data de inici no pot ser superior a la data final';
		raise_application_error (-20000, '');
	END IF;

	CASE UPPER(p_Tipus) 
	WHEN 'A' THEN 

		OPEN p_RS FOR
			SELECT * FROM Incidencies WHERE
			CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
				DataAlta >= p_DataDesde THEN 1 ELSE 0 END END = 1 AND 
			CASE WHEN p_DataFi IS NULL THEN 1 ELSE CASE WHEN 
				DataAlta <= p_DataFi THEN 1 ELSE 0 END END = 1;
            
	WHEN 'T' THEN 

		OPEN p_RS FOR
			SELECT * FROM Incidencies WHERE
			CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
				DataTancament >= p_DataDesde THEN 1 ELSE 0 END END = 1 AND 
			CASE WHEN p_DataFi IS NULL THEN 1 ELSE CASE WHEN 
				DataTancament <= p_DataFi THEN 1 ELSE 0 END END = 1 ;

	ELSE
		v_Res := 'El tipus ha de ser algun dels segents: A->Alta, T->Tancament';
		raise_application_error (-20000, '');
	END CASE;


	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;
      
	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Incidencia_Prioritat
/******************************************************************************
	NOM:			Obtenir_Incidencia_Prioritat

	REQUERIMENT N:	96

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries incidncies, 
				a partir de la prioritat facilitada com a parmetre. 
				Retornar les dades de les incidncies la prioritat de les qual sigui 
				la mateixa prioritat del parmetre

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_IdPrioritat		Identificador de la prioritat de la incidncia

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdPrioritat		IN		Incidencies.IdPrioritat%TYPE
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdPrioritat: ' || p_IdPrioritat;

	OPEN p_RS FOR
		SELECT * FROM Incidencies 
		WHERE IdPrioritat = p_IdPrioritat;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Incidencia_Estat
/******************************************************************************
	NOM:			Obtenir_Incidencia_Estat

	REQUERIMENT N:	97

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries incidncies, 
				a partir de lestat facilitat com a parmetre. Retornar les dades de 
				les incidncies lestat de les qual sigui el mateix estat del parmetre 

	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdEstat		Identificador de l'estat de la incidncia

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_IdEstat		IN		Incidencies.IdEstat%TYPE
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdEstat: ' || p_IdEstat;

	OPEN p_RS FOR
		SELECT * FROM Incidencies
		WHERE IdEstat = p_IdEstat;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Incidencia_Client
/******************************************************************************
	NOM:			Obtenir_Incidencia_Client

	REQUERIMENT N:	98

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries incidncies, 
				a partir del client facilitat com a parmetre. Retornar les dades de 
				les incidncies el client de les qual sigui el mateix client del parmetre 

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_IdClient			Identificador del client de la incidncia

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdClient			IN		Incidencies.IdClient%TYPE
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdClient: ' || p_IdClient;

	OPEN p_RS FOR
		SELECT * FROM Incidencies
		WHERE IdClient = p_IdClient;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Incidencia_Agent
/******************************************************************************
	NOM:			Obtenir_Incidencia_Agent

	REQUERIMENT N:	99

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries incidncies, 
				a partir de lagent facilitat com a parmetre. Retornar les dades de les 
				incidncies lagent de les qual sigui el mateix agent del parmetre

	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdAgent		Identificador de l'agent que obre les incidncies

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdAgent			IN		Incidencies.IdAgent%TYPE
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdAgent: ' || p_IdAgent;

	OPEN p_RS FOR
		SELECT * FROM Incidencies
		WHERE IdAgent = p_IdAgent;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

-------------------------------------------------------------------------------
-- Tipus Actuaci
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_TipusActuacio (
/******************************************************************************
	NOM:			Alta_TipusActuacio

	REQUERIMENT N:	100

	DESCRIPCIN:	Dna dalta un tipus d'actuaci al nostre sistema

	PARMETRES:

		p_TipusActuacio		Tipus d'actuaci
		p_DataAlta			Data d'alta

******************************************************************************/
	p_TipusActuacio		IN	TipusActuacio.TipusActuacio%TYPE,
	p_DataAlta			IN	TipusActuacio.DataAlta%TYPE := Dia(SYSDATE)
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
         ' p_TipusActuacio: '
      || p_TipusActuacio
      || ' p_DataAlta: '
      || p_DataAlta;

	INSERT INTO TipusActuacio (TipusActuacio, DataAlta)
	VALUES (p_TipusActuacio, Dia(p_DataAlta));

	-- Log
	v_Fi := SYSDATE;
	v_Res := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el tipus actuaci ' || p_TipusActuacio;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_TipusActuacio (
/******************************************************************************
	NOM:			Modificar_TipusActuacio

	REQUERIMENT N:	101

	DESCRIPCI:		Permet modificar un tipus d'actuaci del nostre sistema

	PARMETRES:

		p_IdTipusActuacio		Identificador del tipus d'actuaci
		p_TipusActuacio		Tipus d'actuaci
		p_DataAlta			Data d'alta (Si no s'introdueix mantindr la data d'alta existent)

******************************************************************************/
	p_IdTipusActuacio		IN	TipusActuacio.IdTipusActuacio%TYPE,
	p_TipusActuacio		IN	TipusActuacio.TipusActuacio%TYPE,
	p_DataAlta			IN	TipusActuacio.DataAlta%TYPE := NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdTipusActuacio: '
		|| p_IdTipusActuacio
		|| ' p_TipusActuacio: '
		|| p_TipusActuacio
		|| ' p_DataAlta: '
		|| p_DataAlta;

	UPDATE TipusActuacio
	SET TipusActuacio = p_TipusActuacio,
		DataAlta = NVL (DIA(p_DataAlta), DataAlta)
	WHERE IdTipusActuacio = p_IdTipusActuacio AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix o est donat de baixa el tipus actuaci amb IdTipusActuacio=' || p_IdTipusActuacio;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix el tipus actuaci ' || p_TipusActuacio;
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Baixa_TipusActuacio (
/******************************************************************************
	NOM:			Baixa_TipusActuacio

	REQUERIMENT N:	102

	DESCRIPCI:		Elimina o dna de baixa un tipus dactuaci del nostre sistema, 
				proporcionant el seu identificador. 
				Si el tipus dactuaci no t cap actuaci associada seliminar. 
				En cas contrari, es donar de baixa

	PARMETRES:

		p_IdTipusActuacio		Identificador del tipus d'actuaci
		p_DataBaixa			Data de baixa del tipus d'actuaci

******************************************************************************/
	p_IdTipusActuacio		IN	TipusActuacio.IdTipusActuacio%TYPE,
	p_DataBaixa			IN	TipusActuacio.DataBaixa%TYPE:=Dia(SYSDATE)
   )
IS
	v_DataAlta		TipusActuacio.DataAlta%TYPE;
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdTipusActuacio: ' || p_IdTipusActuacio || ' p_DataBaixa: ' || p_DataBaixa ;

	-- Comprova que existeixi i estigui donat d'alta
	v_Res:= 'El tipus actuaci amb IdTipusActuacio=' || p_IdTipusActuacio || ' no existeix o est donat de baixa';

	SELECT DataAlta INTO v_DataAlta 
	FROM TipusActuacio WHERE IdTipusActuacio = p_IdTipusActuacio AND DataBaixa IS NULL;
   
	-- Comprova que la data de baixa no sigui anterior a la d'alta
	IF v_DataAlta > p_DataBaixa THEN
		v_Res:= 'La data alta no pot ser anterior a la de baixa';
		raise_application_error (-20000, '');
	END IF;   

	DELETE TipusActuacio
	WHERE IdTipusActuacio = p_IdTipusActuacio AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el TipusActuacio ' || p_IdTipusActuacio || ', o ja est donat de baixa ';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			-- No es pot borrar perqu t actuacions associades, s'assignar la data de baixa
			UPDATE TipusActuacio
			SET DataBaixa = Dia(SYSDATE)
			WHERE IdTipusActuacio = p_IdTipusActuacio AND DataBaixa IS NULL;

			v_Res := 'OK';
		ELSE
			IF v_Res IS NULL
			THEN
				-- Error no controlat per codi
				v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
			ELSE
				-- Error controlat per codi
				v_Res := 'Error:' || v_Res;
			END IF;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Recuperar_TipusActuacio (
/******************************************************************************
	NOM:			Recuperar_TipusActuacio

	REQUERIMENT N:	103

	DESCRIPCI:		Permet recuperar un tipus d'actuaci del nostre sistema
                
	PARMETRES:                 

		p_IdTipusActuacio		Identificador del tipus d'actuaci

******************************************************************************/
	p_IdTipusActuacio		IN	TipusActuacio.IdTipusActuacio%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdTipusActuacio: ' || p_IdTipusActuacio;

	UPDATE TipusActuacio
	SET DataBaixa = NULL
	WHERE IdTipusActuacio = p_IdTipusActuacio AND DataBaixa IS NOT NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el tipus actuaci ' || p_IdTipusActuacio;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_TipusActuacio
/******************************************************************************
	NOM:			Obtenir_TipusActuacio

	REQUERIMENT N:	104

	DESCRIPCI:		Permet obtenir tota la informaci complerta dun tipus dactuaci, 
				a partir de lidentificador de tipus dactuaci facilitat com a parmetre. 
				Si aquest parmetre s null, retornar els tipus dactuaci el nom dels 
				quals contingui el nom que es passa per parmetre. 
				Si no sespecifica cap dels dos parmetres, retornar una relaci de tots 
				els tipus dactuaci. Tamb tindrem lopci, per parmetre, de decidir si 
				han de sortir o no els tipus dactuaci donats de baixa  
                  
	PARMETRES:

		p_RS				Recordset amb el resultat
		p_IdTipusActuacio		Identificador del tipus d'actuaci a cercar
		p_TipusActuacio		Tipus d'actuaci a cercar
		p_IncloureBaixes		Indica si es mostraran tamb els que estiguin donats de baixa
						Per defecte no sortiran (p_IncloureBaixes = 0)

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdTipusActuacio		IN		TipusActuacio.IdTipusActuacio%TYPE:=Null,
	p_TipusActuacio		IN		TipusActuacio.TipusActuacio%TYPE:=Null,
	p_IncloureBaixes		IN		NUMBER:=0        
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_IdTipusActuacio: ' || p_IdTipusActuacio || ' p_TipusActuacio:' || p_TipusActuacio
		|| ' p_IncloureBaixes:' || p_IncloureBaixes;

	IF p_IdTipusActuacio IS NULL AND p_TipusActuacio IS NULL THEN 
		OPEN p_RS FOR
			SELECT * FROM TipusActuacio
			WHERE CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;

	ELSIF p_IdTipusActuacio IS NOT NULL THEN 
		OPEN p_RS FOR
			SELECT * FROM TipusActuacio WHERE IdTipusActuacio = p_IdTipusActuacio AND 
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
        
	ELSIF p_TipusActuacio IS NOT NULL THEN 
		OPEN p_RS FOR
			SELECT * FROM TipusActuacio WHERE UPPER(TipusActuacio) LIKE '%' || UPPER(p_TipusActuacio) || '%' AND
			CASE WHEN p_IncloureBaixes = 1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
	END IF;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

-------------------------------------------------------------------------------
-- Actuacions
-------------------------------------------------------------------------------


CREATE OR REPLACE PROCEDURE Alta_Actuacio (
/******************************************************************************
	NOM:			Alta_Actuacio

	REQUERIMENT N:	105

	DESCRIPCI:		Dna dalta una actuaci al nostre sistema

	PARMETRES:

		p_IdTipusActuacio		Tipus d'actuaci
		p_IdIncidencia		Incidncia associada a l'actuaci
		p_IdAgent			Agent que porta a terme l'actuaci
		p_Descripcio		Descripci de l'actuaci
		p_DataInici			Data d'inici de l'actuaci (Per defecte pren la data del sistema)
		p_Cost			Cost de l'actuaci (Per defecte ser 0)
		p_Observacions		Observacions a l'actuaci

******************************************************************************/
	p_IdTipusActuacio		IN	Actuacions.IdTipusActuacio%TYPE,
	p_IdIncidencia		IN	Actuacions.IdIncidencia%TYPE, 
	p_IdAgent			IN	Actuacions.IdAgent%TYPE,
	p_Descripcio		IN	Actuacions.Descripcio%TYPE,
	p_DataInici			IN	Actuacions.DataInici%TYPE := Dia(SYSDATE),
	p_Cost			IN	Actuacions.Cost%TYPE := 0,
	p_Observacions		IN	Actuacions.Observacions%TYPE := NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdTipusActuacio: '
		|| p_IdTipusActuacio
		|| ' p_IdIncidencia: '
		|| p_IdIncidencia
		|| ' p_IdAgent: '
		|| p_IdAgent
		|| ' p_Descripcio: '
		|| p_Descripcio
		|| ' p_DataInici: '
		|| p_DataInici
		|| ' p_Cost: '
		|| p_Cost
		|| ' p_Observacions: '
		|| p_Observacions;

	INSERT INTO Actuacions (IdTipusActuacio, IdIncidencia, IdAgent, Descripcio, DataInici, Cost, Observacions)
	VALUES (p_IdTipusActuacio, p_IdIncidencia, p_IdAgent, p_Descripcio, Dia(p_DataInici), p_Cost, p_Observacions);
 
	-- Log
	v_Fi := SYSDATE;
	v_REs := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'El identificador de tipus actuaci i/o de incidncia i/o de agent no sn correctes';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Actuacio (
/******************************************************************************
	NOM:			Modificar_Actuacio

	REQUERIMENT N:	106

	DESCRIPCI:		Permet modificar la informaci associada a una actuaci

	PARMETRES:

		p_IdActuacio		Identificador de l'actuaci
		p_IdTipusActuacio		Tipus d'actuaci
		p_IdIncidencia		Incidncia associada a l'actuaci
		p_IdAgent			Agent que porta a terme l'actuaci
		p_Descripcio		Descripci de l'actuaci
		p_DataInici			Data d'inici de l'actuaci (Si no s'introdueix mantindr la data d'alta existent)
		p_DataFi			Data fi de l'actuaci
		p_Cost			Cost de l'actuaci
		p_Observacions		Observacions a l'actuaci

******************************************************************************/
	p_IdActuacio		IN	Actuacions.IdActuacio%TYPE,
	p_IdTipusActuacio		IN	Actuacions.IdTipusActuacio%TYPE,
	p_IdIncidencia		IN	Actuacions.IdIncidencia%TYPE, 
	p_IdAgent			IN	Actuacions.IdAgent%TYPE,
	p_Descripcio		IN	Actuacions.Descripcio%TYPE,
	p_DataInici			IN	Actuacions.DataInici%TYPE := NULL,
	p_DataFi			IN	Actuacions.DataFi%TYPE,
	p_Cost			IN	Actuacions.Cost%TYPE,
	p_Observacions		IN	Actuacions.Observacions%TYPE
)
IS
	v_tmp_DataInici	Actuacions.DataInici%TYPE := NULL;
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdActuacio: '
		|| p_IdActuacio
		|| ' p_IdTipusActuacio: '
		|| p_IdTipusActuacio
		|| ' p_IdIncidencia: '
		|| p_IdIncidencia
		|| ' p_IdAgent: '
		|| p_IdAgent
		|| ' p_Descripcio: '
		|| p_Descripcio
		|| ' p_DataInici: '
		|| p_DataInici
		|| ' p_DataFi: '
		|| p_DataFi
		|| ' p_Cost: '
		|| p_Cost
		|| ' p_Observacions: '
		|| p_Observacions;

	-- Si p_DataInici s NULL, agafem la data d'inici de l'actuaci
	IF p_DataInici IS NULL
	THEN
		SELECT DataInici 
		INTO v_tmp_DataInici
		FROM Actuacions
		WHERE IdActuacio = p_IdActuacio;
	ELSE
		v_tmp_DataInici := p_DataInici;
	END IF;

	-- Data Inici < Data Fi
	IF v_tmp_DataInici > p_DataFi
	THEN
		v_Res := 'La data fi ha de ser posterior a la de inici';
		raise_application_error (-20000, '');
	END IF;

	UPDATE Actuacions
	SET IdTipusActuacio = p_IdTipusActuacio,
		IdIncidencia = p_IdIncidencia,
		IdAgent = p_IdAgent,
		Descripcio = p_Descripcio,
		DataInici = Dia(v_tmp_DataInici),
		DataFi = Dia(p_DataFi),
		Cost = p_Cost,
		Observacions = p_Observacions
	WHERE IdActuacio = p_IdActuacio;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix la actuaci amb IdActuacio=' || p_IdActuacio;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix la actuaci amb IdActuacio=' || p_IdActuacio;
		ELSIF SQLCODE = 100
		THEN
			v_Res := 'No existeix la actuaci amb IdActuacio=' || p_IdActuacio;  
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
        ELSIF SQLCODE = -2291
        THEN
            v_Res := 'El identificador de tipus actuaci i/o de incidncia i/o de agent no sn correctes';
END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Actuacio_TipusActuacio
/******************************************************************************
	NOM:			Obtenir_Actuacio_TipusActuacio

	REQUERIMENT N:	107

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries actuacions, 
				a partir del tipus dactuaci facilitat com a parmetre. Retornar les 
				dades de les actuacions el tipus de les quals sigui com el tipus parmetre 

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_IdTipusActuacio		Identificador del tipus d'actuaci

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdTipusActuacio		IN		Actuacions.IdTipusActuacio%TYPE
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdTipusActuacio: ' || p_IdTipusActuacio;

	OPEN p_RS FOR
		SELECT * FROM Actuacions 
		WHERE IdTipusActuacio = p_IdTipusActuacio;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Actuacio_Desc
/******************************************************************************
	NOM:			Obtenir_Actuacio_Desc

	REQUERIMENT N:	108

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries actuacions, 
				a partir de la descripci facilitada com a parmetre. Retornar les dades 
				de les actuacions la descripci de les quals contingui la facilitada com a parmetre

	PARMETRES:

		p_RS				Recordset amb el resultat
		p_Descripcio		Descripci a cercar

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_Descripcio	IN		Actuacions.Descripcio%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_Descripcio: ' || p_Descripcio;


	OPEN p_RS FOR
		SELECT * FROM Actuacions
		WHERE UPPER(Descripcio) LIKE '%' || UPPER(p_Descripcio) || '%';

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Actuacio_Data
/******************************************************************************
	NOM:			Obtenir_Actuacio_Data

	REQUERIMENT N:	109

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries actuacions, 
				a partir de les dates facilitades com a parmetre. Retornar les dades 
				de les actuacions la data de les quals estigui compresa entre les dates facilitades.

				Si noms es facilita la data dinici, retornar les actuacions a partir daquesta data.
				Si noms es facilita la data final, retornar les actuacions fins aquesta data.
				Si no es facilita cap data, retornar la totalitat de les actuacions.

				Podrem especificar per parmetre si el filtre aplica a la data dinici de 
				lactuaci (I), o a la data final de lactuaci (F)
                 
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_Tipus		Tipus de filtre de data
		p_DataDesde		Data inici
		p_DataFi		Data fi (per defecte ser la del sistema)

******************************************************************************/
( 
	p_RS			IN OUT	sys_refcursor, 
	p_Tipus		IN		VARCHAR2,  
	p_DataDesde		IN		Actuacions.DataInici%TYPE:=Null,
	p_DataFi		IN		Actuacions.DataFi%TYPE:=Null
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT), p_Tipus: ' || p_Tipus ||  
		' p_DataDesde: ' || p_DataDesde  
		|| ' p_DataFi: ' || p_DataFi;

	IF p_DataDesde > p_DataFi THEN
		v_Res := 'La data de inici no pot ser superior a la data final';
		raise_application_error (-20000, '');
	END IF;

	CASE UPPER(p_Tipus) 
	WHEN 'I' THEN 

		OPEN p_RS FOR
			SELECT * FROM Actuacions WHERE
			CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
				DataInici >= p_DataDesde THEN 1 ELSE 0 END END = 1 AND 
			CASE WHEN p_DataFi IS NULL THEN 1 ELSE CASE WHEN 
				DataInici <= p_DataFi THEN 1 ELSE 0 END END = 1;
            
	WHEN 'F' THEN 

		OPEN p_RS FOR
			SELECT * FROM Actuacions WHERE
			CASE WHEN p_DataDesde IS NULL THEN 1 ELSE CASE WHEN 
				DataFi >= p_DataDesde THEN 1 ELSE 0 END END = 1 AND 
			CASE WHEN p_DataFi IS NULL THEN 1 ELSE CASE WHEN 
				DataFi <= p_DataFi THEN 1 ELSE 0 END END = 1 ;

	ELSE
		v_Res := 'El tipus ha de ser algun dels segents: I->Inici, F->Fi';
		raise_application_error (-20000, '');
	END CASE;


	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;
      
	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/

CREATE OR REPLACE PROCEDURE Obtenir_Actuacio_Incidencia
/******************************************************************************
	NOM:			Obtenir_Actuacio_Incidencia

	REQUERIMENT N:	110

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries actuacions, 
				a partir dun identificador dincidncia facilitat com a parmetre. 
				Retornar les dades de les actuacions on lidentificador dincidncia 
				sigui igual al del parmetre 

	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdIncidencia	Identificador de la incidncia associada a l'actuaci

******************************************************************************/

(   
	p_RS			IN OUT	sys_refcursor,
	p_IdIncidencia	IN		Actuacions.IdIncidencia%TYPE
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdIncidencia: ' || p_IdIncidencia;

	OPEN p_RS FOR
		SELECT * FROM Actuacions
		WHERE IdIncidencia = p_IdIncidencia;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Obtenir_Actuacio_Agent
/******************************************************************************
	NOM:			Obtenir_Actuacio_Agent

	REQUERIMENT N:	111

	DESCRIPCI:		Permet obtenir tota la informaci complerta duna o vries actuacions, 
				a partir de lagent facilitat com a parmetre. Retornar les dades de les 
				actuacions lagent de les qual sigui el mateix agent del parmetre

	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdAgent		Identificador de l'agent que porta a terme les actuacions

******************************************************************************/

(   
	p_RS				IN OUT	sys_refcursor,
	p_IdAgent			IN		Actuacions.IdAgent%TYPE
) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_RS (IN OUT) , p_IdAgent: ' || p_IdAgent;

	OPEN p_RS FOR
		SELECT * FROM Actuacions
		WHERE IdAgent = p_IdAgent;

	v_Res:= 'OK';    

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;
/


COMMIT;
