/*------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Crrega inicial de la Base de Dades
------------------------------------------------*/


-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

COMMIT;


/*------------------------------------------------
	Procediment que carregar les segents dades:

	20 Usuaris
	5 Permisos
	5 Paisos
	40 Provincies (8 per a cada pais)
	200 Localitats (5 per a cada provincia)
	1000 CodisPostal (5 per a cada localitat)
	50 Clients
	2 Centres
	3 Garanties
	20 TipusProducte
	100 Productes
	5 Perfils
	10 Agents
	3 Estats
	3 Prioritats
	10 TipusIncidencia
	200 Incidncies
	10 TipusActuacio
	600 Actuacions (3 per a cada incidncia)
-------------------------------------------------*/


CREATE OR REPLACE PROCEDURE Carrega_Inicial_Dades
IS
	n				NUMBER;
	v_IdPermis			Permisos.IdPermis%TYPE;
	v_IdUsuari			Usuaris.IdUsuari%TYPE;
	v_IdPais			Paisos.IdPais%TYPE;
	v_IdProvincia		Provincies.IdProvincia%TYPE;
	v_IdLocalitat		Localitats.IdLocalitat%TYPE;
	v_CodiPostal		CodisPostal.CodiPostal%TYPE;
	v_Sexe			Clients.Sexe%TYPE;
	v_IdCentre			Centres.IdCentre%TYPE;
	v_IdGarantia		Garanties.IdGarantia%TYPE;
	v_IdTipusProducte		TipusProducte.IdTipusProducte%TYPE;
	v_IdPerfil			Perfils.IdPerfil%TYPE;
	v_IdAgent			Agents.IdAgent%TYPE;
	v_IdTipusIncidencia	TipusIncidencia.IdTipusIncidencia%TYPE;
	v_IdClient			Clients.IdClient%TYPE;
	v_IdProducte		Productes.IdProducte%TYPE;
	v_IdPrioritat		Prioritats.IdPrioritat%TYPE;
	v_IdEstat			Estats.IdEstat%TYPE;
	v_IdTipusActuacio		TipusActuacio.IdTipusActuacio%TYPE;
	v_IdIncidencia		Incidencies.IdIncidencia%TYPE;
   
BEGIN
	-- Usuaris i permisos
	n:=0;

	-- 5 permisos
	FOR i IN 1 .. 5
	LOOP
		Alta_Permis('Perms_' || i);
		SELECT MAX(IdPermis) INTO v_IdPermis FROM Permisos ; 

		-- 4 Usuarios (20 en total)
		FOR j IN 1 .. 4      
		LOOP
			n:=n + 1;
			Alta_Usuari('Nom_' || n, 'Cognom_' || n, 'Usuari_' || n, 'Password_' || n);
			SELECT MAX(IdUsuari) INTO v_IdUsuari FROM Usuaris ; 

			-- Donem un perms a cada usuari
			Donar_Permis_Usuari(v_IdPermis, v_IdUsuari); 
		END LOOP;
	END LOOP;
   
	-- Ubicacions
	n:=0;

	-- 5 paisos      
	FOR i IN 1 .. 5
	LOOP
		Alta_Pais('Pais_' || i);
		SELECT MAX(IdPais) INTO v_IdPais FROM Paisos;

		-- 8 provincies per pais (40 en total)
		FOR j IN 1 .. 8
		LOOP
			Alta_Provincia('Provincia_' || j || ':' || 'Pais_' || i, v_IdPais);  
			SELECT MAX(IdProvincia) INTO v_IdProvincia FROM Provincies;

			-- 5 localitats per provincia (200 en total)
			FOR k IN 1 .. 5
			LOOP
				Alta_Localitat('Localitat_' || k || ':' || 'Provincia_' || j || ':' || 'Pais_' || i, v_IdProvincia);  
				SELECT MAX(IdLocalitat) INTO v_IdLocalitat FROM Localitats;

				-- 5 codis postal (1000 en total)
				FOR l IN 1 .. 5
				LOOP
					n:=n + 1;
					Alta_CodiPostal(43200 + n, v_IdLocalitat);
				END LOOP;

			END LOOP;

		END LOOP;           

	END LOOP;

	-- 50 clients
	FOR i IN 1 .. 50
	LOOP
		SELECT CEIL(DBMS_RANDOM.VALUE * 1000) INTO v_CodiPostal FROM dual;
		SELECT DECODE(mod(i,2), 0, 'H', 1, 'D') INTO v_Sexe FROM Dual;

		Alta_Client('NomCli_' || i, 'CognomsCli_' || i, '3989000' || i, 43200 + v_CodiPostal, 'Adrea_' || i, '900000' || i,
				'600000' || i, 'Email_' || i || '@gmail.com', TO_DATE('01/01/1980') + (i * 100), v_Sexe);
	END LOOP;

	-- 2 centres       
	Alta_Centre('Centre_1', 'Rambla Nova, 10', 43201, 977771522, 'Joan Casas');                                         
	Alta_Centre('Centre_2', 'Riera Mir, 25', 43204, 977755637, 'Carles Torres');                                         

	-- 3 garanties
	Alta_Garantia('Garantia_1', 2, 1);                                         
	Alta_Garantia('Garantia_2', 3, 1);                                         
	Alta_Garantia('Garantia_3', 5, 3);                                         

	-- 100 productes
	FOR i IN 1 .. 100
	LOOP
		-- 20 tipus de producte (5 productes de cada tipus)
		IF MOD(i, 5) = 1 THEN        
			n:= CEIL(i / 5);
			Alta_TipusProducte('TipusProducte ' || n, NULL);
			SELECT MAX(IdTipusProducte) INTO v_IdTipusProducte From TipusProducte;
		END IF; 

		SELECT CEIL(DBMS_RANDOM.VALUE * 2) INTO v_IdCentre FROM dual;
		SELECT CEIL(DBMS_RANDOM.VALUE * 3) INTO v_IdGarantia FROM dual;

		Alta_Producte(v_IdTipusProducte, 'Producte_' || i, '000000000' || i, 'SN00000' || i, TO_DATE('01/01/2005') + (i * 10),
					v_IdCentre, v_IdGarantia, 50 + i, 20 + i, 11 + i);
	END LOOP;

	-- Agents i Perfils
	n:=0;

	-- 5 perfils
	FOR i IN 1 .. 5
	LOOP
		Alta_Perfil('Perfil_' || i);
		SELECT MAX(IdPerfil) INTO v_IdPerfil FROM Perfils ; 

		-- 2 agents (10 en total)
		FOR j IN 1 .. 2      
		LOOP
			n:=n + 1;
			Alta_Agent('NomAgent_' || n, 'CognomAgent_' || n);
			SELECT MAX(IdAgent) INTO v_IdAgent FROM Agents ; 

			-- Donem un perfil a cada agent
			Alta_Perfil_agent(v_IdPerfil, v_IdAgent); 

			-- Assignem una provncia a cada agent
			SELECT CEIL(DBMS_RANDOM.VALUE * 40) INTO v_IdProvincia FROM dual;
			Alta_Provincia_Agent(v_IdProvincia, v_IdAgent); 
		END LOOP;
	END LOOP;
   
	-- 3 estats
	Alta_Estat('Estat_1');                                         
	Alta_Estat('Estat_2');                                         
	Alta_Estat('Estat_3');                                         

	-- 3 prioritats
	Alta_Prioritat('Prioritat_1');                                         
	Alta_Prioritat('Prioritat_2');                                         
	Alta_Prioritat('Prioritat_3');                                         

	-- Incidncies
	FOR i IN 1 .. 200
	LOOP
		-- 10 tipus d'incidncia (20 incidncies de cada tipus)
		IF MOD(i, 20) = 1 THEN        
			n:= CEIL(i / 20);
			Alta_TipusIncidencia('TipusIncidencia ' || n);
			SELECT MAX(IdTipusIncidencia) INTO v_IdTipusIncidencia From TipusIncidencia;
		END IF; 

		SELECT CEIL(DBMS_RANDOM.VALUE * 50) INTO v_IdClient FROM dual;
		SELECT CEIL(DBMS_RANDOM.VALUE * 100) INTO v_IdProducte FROM dual;
		SELECT CEIL(DBMS_RANDOM.VALUE * 10) INTO v_IdAgent FROM dual;
		SELECT CEIL(DBMS_RANDOM.VALUE * 3) INTO v_IdPrioritat FROM dual;
		SELECT CEIL(DBMS_RANDOM.VALUE * 3) INTO v_IdEstat FROM dual;

		Alta_Incidencia(v_IdTipusIncidencia, 'Incidncia_' || i, v_IdClient, v_IdProducte, v_IdAgent, v_IdPrioritat, v_IdEstat,
					TO_DATE('01/01/2011') + i, 'Comentaris Incidncia_' || i);
	END LOOP;

	-- Actuacions
	FOR i IN 1 .. 600
	LOOP
		-- 10 tipus d'actuacio (60 actuacions de cada tipus)
		IF MOD(i, 60) = 1 THEN        
			n:= CEIL(i / 60);
			Alta_TipusActuacio('TipusActuacio ' || n);
			SELECT MAX(IdTipusActuacio) INTO v_IdTipusActuacio From TipusActuacio;
		END IF; 

		-- Cada 3 actuacions s'assignaran a una incidncia diferent
		v_IdIncidencia:= 1;
		IF MOD(i, 3) = 1 THEN
			v_IdIncidencia:= v_IdIncidencia + 1;
		END IF;

		SELECT CEIL(DBMS_RANDOM.VALUE * 10) INTO v_IdAgent FROM dual;

		Alta_Actuacio(v_IdTipusActuacio, v_IdIncidencia, v_IdAgent, 'Actuaci_' || i, TO_DATE('01/01/2011') + i, i,
					'Comentaris Actuaci_' || i);
	END LOOP;

EXCEPTION
	WHEN OTHERS   
	THEN
		-- Error no controlat per codi
		DBMS_OUTPUT.put_line ('Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100));
END;
/




-- Executem la crrega inicial de la Base de Dades

EXECUTE Carrega_Inicial_Dades();

COMMIT;
