/*------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Procediments ETL
		Per a les dimensions, actualitza (modificacions) o inserta (dades noves) les dades procedents de la Base de Dades
		Per als fets, inserta la informaci tal com demanda el fet
------------------------------------------------*/


-- Schema ETL_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = ETL_POSTVENDA;

COMMIT;


-----------------------------------------
-- Dimensi Agents
-----------------------------------------
 
CREATE OR REPLACE PROCEDURE Actualitza_D_Agents
/******************************************************************************
	NOM:			Actualitza_D_Agents

	DESCRIPCI:		Actualitza la dimensi dels agents
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci dels agents que han canviat
	UPDATE (
		SELECT DA.Agent AS old_Agent, A.Cognoms AS new_Agent 
		FROM Agents A, D_Agents DA
		WHERE DA.IdAgent = A.IdAgent AND DA.Agent <> A.Cognoms
		)
	SET old_Agent = new_Agent;

	-- Inserci dels agents nous
	INSERT INTO D_Agents 
	SELECT IdAgent, Cognoms FROM Agents 
	WHERE IdAgent NOT IN (SELECT IdAgent FROM D_Agents);
END;
/


-----------------------------------------
-- Dimensi Centres
-----------------------------------------
 
CREATE OR REPLACE PROCEDURE Actualitza_D_Centres
/******************************************************************************
	NOM:			Actualitza_D_Centres

	DESCRIPCI:		Actualitza la dimensi dels centres
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci dels centres que han canviat
	UPDATE (
		SELECT DC.Centre AS old_Centre, C.Centre AS new_Centre
		FROM Centres C, D_Centres DC
		WHERE DC.IdCentre = C.IdCentre AND DC.Centre <> C.Centre
		)
	SET old_Centre = new_Centre;

	-- Inserci dels centres nous
	INSERT INTO D_Centres 
	SELECT IdCentre, Centre FROM Centres 
	WHERE IdCentre NOT IN (SELECT IdCentre FROM D_Centres);
END;
/


-----------------------------------------
-- Dimensi Clients
-----------------------------------------
 
CREATE OR REPLACE PROCEDURE Actualitza_D_Clients
/******************************************************************************
	NOM:			Actualitza_D_Clients

	DESCRIPCI:		Actualitza la dimensi dels clients
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci dels clients que han canviat
	UPDATE (
		SELECT DC.Client AS old_Client, C.Cognoms AS new_Client
		FROM Clients C, D_Clients DC
		WHERE DC.IdClient = C.IdClient AND DC.Client <> C.Cognoms
		)
	SET old_Client = new_Client;

	-- Inserci dels clients nous
	INSERT INTO D_Clients 
	SELECT IdClient, Cognoms FROM Clients 
	WHERE IdClient NOT IN (SELECT IdClient FROM D_Clients);
END;
/


-----------------------------------------
-- Dimensi Estats
-----------------------------------------

CREATE OR REPLACE PROCEDURE Actualitza_D_Estats
/******************************************************************************
	NOM:			Actualitza_D_Estats

	DESCRIPCI:		Actualitza la dimensi dels estats
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci dels estats que han canviat
	UPDATE (
		SELECT DE.Estat AS old_Estat, E.Estat AS new_Estat
		FROM Estats E, D_Estats DE
		WHERE DE.IdEstat = E.IdEstat AND DE.Estat <> E.Estat
		)
	SET old_Estat = new_Estat;

	-- Inserci dels estats nous
	INSERT INTO D_Estats 
	SELECT * FROM Estats 
	WHERE IdEstat NOT IN (SELECT IdEstat FROM D_Estats);
END;
/


-----------------------------------------
-- Dimensi Incidncies
-----------------------------------------

CREATE OR REPLACE PROCEDURE Actualitza_D_Incidencies
/******************************************************************************
	NOM:			Actualitza_D_Incidencies

	DESCRIPCI:		Actualitza la dimensi de les incidncies
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci de les incidncies que han canviat
	UPDATE (
		SELECT DI.Incidencia AS old_Incidencia, I.Descripcio AS new_Incidencia
		FROM Incidencies I, D_Incidencies DI
		WHERE DI.IdIncidencia = I.IdIncidencia AND DI.Incidencia <> I.Descripcio
		)
	SET old_Incidencia = new_Incidencia;

	-- Inserci de les incidncies noves
	INSERT INTO D_Incidencies 
	SELECT IdIncidencia, Descripcio FROM Incidencies
	WHERE IdIncidencia NOT IN (SELECT IdIncidencia FROM D_Incidencies);
END;
/


-----------------------------------------
-- Dimensi Perfils
-----------------------------------------

CREATE OR REPLACE PROCEDURE Actualitza_D_Perfils
/******************************************************************************
	NOM:			Actualitza_D_Perfils

	DESCRIPCI:		Actualitza la dimensi dels perfils
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci dels perfils que han canviat
	UPDATE (
		SELECT DP.Perfil AS old_Perfil, P.Perfil AS new_Perfil
		FROM Perfils P, D_Perfils DP
		WHERE DP.IdPerfil = P.IdPerfil AND DP.Perfil <> P.Perfil
		)
	SET old_Perfil = new_Perfil;

	-- Inserci dels perfils nous
	INSERT INTO D_Perfils 
	SELECT IdPerfil, Perfil FROM Perfils 
	WHERE IdPerfil NOT IN (SELECT IdPerfil FROM D_Perfils);
END;
/


-----------------------------------------
-- Dimensi Prioritats
-----------------------------------------

CREATE OR REPLACE PROCEDURE Actualitza_D_Prioritats
/******************************************************************************
	NOM:			Actualitza_D_Prioritats

	DESCRIPCI:		Actualitza la dimensi de les prioritats
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci de les prioritats que han canviat
	UPDATE (
		SELECT DP.Prioritat AS old_Prioritat, P.Prioritat AS new_Prioritat
		FROM Prioritats P, D_Prioritats DP
		WHERE DP.IdPrioritat = P.IdPrioritat AND DP.Prioritat <> P.Prioritat 
		)
	SET old_Prioritat = new_Prioritat;

	-- Inserci de les prioritats noves
	INSERT INTO D_Prioritats
	SELECT * FROM Prioritats
	WHERE IdPrioritat NOT IN (SELECT IdPrioritat FROM D_Prioritats);
END;
/


-----------------------------------------
-- Dimensi Productes
-----------------------------------------

CREATE OR REPLACE PROCEDURE Actualitza_D_Productes
/******************************************************************************
	NOM:			Actualitza_D_Productes

	DESCRIPCI:		Actualitza la dimensi dels productes
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci dels productes que han canviat
	UPDATE (
		SELECT DP.Producte AS old_Producte, P.Descripcio AS new_Producte
		FROM Productes P, D_Productes DP
		WHERE DP.IdProducte = P.IdProducte AND DP.Producte <> P.Descripcio
		)
	SET old_Producte = new_Producte;

	-- Inserci dels productes nous
	INSERT INTO D_Productes
	SELECT IdProducte, Descripcio FROM Productes
	WHERE IdProducte NOT IN (SELECT IdProducte FROM D_Productes);
END;
/


------------------------------------------------
-- Dimensi Temps
------------------------------------------------
 
CREATE OR REPLACE PROCEDURE Actualitza_D_Temps
/******************************************************************************
	NOM:			Actualitza_D_Temps

	DESCRIPCI:		Actualitza la dimensi del temps
******************************************************************************/ 
IS
BEGIN

	-- Inserim el temps que encara no existeixi al DW
	INSERT INTO D_Temps (Mes, Anyo) 
	SELECT Mes, Anyo FROM vi_D_Temps 
	WHERE Mes NOT IN (SELECT Mes FROM D_Temps);
END;
/


-----------------------------------------
-- Dimensi TipusActuacio
-----------------------------------------

CREATE OR REPLACE PROCEDURE Actualitza_D_TipusActuacio
/******************************************************************************
	NOM:			Actualitza_D_TipusActuacio

	DESCRIPCI:		Actualitza la dimensi dels tipus d'actuaci
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci dels tipus d'actuaci que han canviat
	UPDATE (
		SELECT DT.TipusActuacio AS old_TipusActuacio, T.TipusActuacio AS new_TipusActuacio
		FROM TipusActuacio T, D_TipusActuacio DT
		WHERE DT.IdTipusActuacio = T.IdTipusActuacio AND DT.TipusActuacio <> T.TipusActuacio
		)
	SET old_TipusActuacio = new_TipusActuacio;

	-- Inserci dels tipus d'actuaci nous
	INSERT INTO D_TipusActuacio
	SELECT IdTipusActuacio, TipusActuacio FROM TipusActuacio
	WHERE IdTipusActuacio NOT IN (SELECT IdTipusActuacio FROM D_TipusActuacio);
END;
/


-----------------------------------------
-- Dimensi TipusIncidencia
-----------------------------------------

CREATE OR REPLACE PROCEDURE Actualitza_D_TipusIncidencia
/******************************************************************************
	NOM:			Actualitza_D_TipusIncidencia

	DESCRIPCI:		Actualitza la dimensi dels tipus d'incidncia
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci dels tipus d'incidncia que han canviat
	UPDATE (
		SELECT DT.TipusIncidencia AS old_TipusIncidencia, T.TipusIncidencia AS new_TipusIncidencia
		FROM TipusIncidencia T, D_TipusIncidencia DT
		WHERE DT.IdTipusIncidencia = T.IdTipusIncidencia AND DT.TipusIncidencia <> T.TipusIncidencia
		)
	SET old_TipusIncidencia = new_TipusIncidencia;

	-- Inserci dels tipus d'incidncia nous
	INSERT INTO D_TipusIncidencia
	SELECT IdTipusIncidencia, TipusIncidencia FROM TipusIncidencia
	WHERE IdTipusIncidencia NOT IN (SELECT IdTipusIncidencia FROM D_TipusIncidencia);
END;
/


-----------------------------------------
-- Dimensi TipusProducte
-----------------------------------------

CREATE OR REPLACE PROCEDURE Actualitza_D_TipusProducte
/******************************************************************************
	NOM:			Actualitza_D_TipusProducte

	DESCRIPCI:		Actualitza la dimensi dels tipus de producte
******************************************************************************/ 
IS
BEGIN

	-- Actualitzaci dels tipus de producte que han canviat
	UPDATE (
		SELECT DT.TipusProducte AS old_TipusProducte, T.TipusProducte AS new_TipusProducte
		FROM TipusProducte T, D_TipusProducte DT
		WHERE DT.IdTipusProducte = T.IdTipusProducte AND DT.TipusProducte <> T.TipusProducte
		)
	SET old_TipusProducte = new_TipusProducte;

	-- Inserci dels tipus de producte nous
	INSERT INTO D_TipusProducte
	SELECT IdTipusProducte, TipusProducte FROM TipusProducte
	WHERE IdTipusProducte NOT IN (SELECT IdTipusProducte FROM D_TipusProducte);
END;
/


------------------------------------------------
-- Dimensi Ubicacions
------------------------------------------------
 
CREATE OR REPLACE PROCEDURE Actualitza_D_Ubicacions
/******************************************************************************
	NOM:			Actualitza_D_Ubicacions

	DESCRIPCI:		Actualitza la dimensi de les ubicacions
    				S'agafen les ubicacions de les taules: AgentsProvincies
******************************************************************************/ 
IS
BEGIN

	-- Inserim les ubicacions que encara no existeixin al DW
	INSERT INTO D_Ubicacions 
	SELECT A.* FROM (
		SELECT DISTINCT V.IdProvincia, V.Provincia, V.Pais 
		FROM AgentsProvincies A, vi_Provincies V
		WHERE A.IdProvincia = V.IdProvincia) A
	LEFT JOIN D_Ubicacions D ON A.IdProvincia = D.IdUbicacio
	WHERE D.IdUbicacio IS NULL;
END;
/


CREATE OR REPLACE PROCEDURE Actualitza_Dimensions
/******************************************************************************
	NOM:			Actualitza_Dimensions

	DESCRIPCI:		Actualitza les dimensions del DW
******************************************************************************/ 
IS
BEGIN
  Actualitza_D_Agents;
  Actualitza_D_Centres;
  Actualitza_D_Clients;
  Actualitza_D_Estats;
  Actualitza_D_Incidencies;
  Actualitza_D_Perfils;
  Actualitza_D_Prioritats;
  Actualitza_D_Productes;
  Actualitza_D_Temps;
  Actualitza_D_TipusActuacio;
  Actualitza_D_TipusIncidencia;
  Actualitza_D_TipusProducte;
  Actualitza_D_Ubicacions;
END;
/





------------------------------------------------
-- Fet Agents     
------------------------------------------------
 
CREATE OR REPLACE PROCEDURE Actualitza_F_Agents
/******************************************************************************
	NOM:			Actualitza_F_Agents

	DESCRIPCI:		Actualitza el fet F_Agents
******************************************************************************/ 
IS
BEGIN

	-- Inserim els agents que encara no existeixin al DW
	INSERT INTO F_Agents FA 
		SELECT V.* FROM vi_F_Agents V
		LEFT JOIN F_Agents FA ON 
		V.Mes = FA.IdTemps AND 
		V.IdUbicacio = FA.IdUbicacio AND 
		V.IdPerfil = FA.IdPerfil
	WHERE FA.IdTemps IS NULL;
END;
/


------------------------------------------------
-- Fet Productes      
------------------------------------------------
 
CREATE OR REPLACE PROCEDURE Actualitza_F_Productes
/******************************************************************************
	NOM:			Actualitza_F_Productes

	DESCRIPCI:		Actualitza el fet F_Productes
******************************************************************************/ 
IS
BEGIN

	-- Inserim els productes que encara no existeixin al DW
	INSERT INTO F_Productes FP 
		SELECT V.* FROM vi_F_Productes V
		LEFT JOIN F_Productes FP ON 
		V.Mes = FP.IdTemps AND 
		V.IdCentre = FP.IdCentre AND 
		V.IdTipusProducte = FP.IdTipusProducte
	WHERE FP.IdTemps IS NULL;
END;
/


------------------------------------------------
-- Fet Incidencies      
------------------------------------------------
 
CREATE OR REPLACE PROCEDURE Actualitza_F_Incidencies
/******************************************************************************
	NOM:			Actualitza_F_Incidencies

	DESCRIPCI:		Actualitza el fet F_Incidencies
******************************************************************************/ 
IS
BEGIN

	-- Inserim les incidncies que encara no existeixin al DW
	INSERT INTO F_Incidencies FI 
		SELECT V.* FROM vi_F_Incidencies V
		LEFT JOIN F_Incidencies FI ON 
		V.Mes = FI.IdTemps AND 
		V.IdProducte = FI.IdProducte AND 
		V.IdClient = FI.IdClient AND 
		V.IdTipusIncidencia = FI.IdTipusIncidencia AND  
		V.IdAgent = FI.IdAgent AND 
		V.IdPrioritat = FI.IdPrioritat AND 
		V.IdEstat = FI.IdEstat 
	WHERE FI.IdTemps IS NULL;
END;
/
  

------------------------------------------------
-- Fet Actuacions      
------------------------------------------------
 
CREATE OR REPLACE PROCEDURE Actualitza_F_Actuacions
/******************************************************************************
	NOM:			Actualitza_F_Actuacions

	DESCRIPCI:		Actualitza el fet F_Actuacions
******************************************************************************/ 
IS
BEGIN

	-- Inserim les actuacions que encara no existeixin al DW
	INSERT INTO F_Actuacions FA 
		SELECT V.* FROM vi_F_Actuacions V
		LEFT JOIN F_Actuacions FA ON 
		V.Mes = FA.IdTemps AND 
		V.IdTipusActuacio = FA.IdTipusActuacio AND  
		V.IdIncidencia = FA.IdIncidencia AND 
		V.IdAgent = FA.IdAgent 
	WHERE FA.IdTemps IS NULL;
END;
/


CREATE OR REPLACE PROCEDURE Actualitza_Fets
/******************************************************************************
	NOM:			Actualitza_Fets

	DESCRIPCI:		Actualitza els fets del DW
******************************************************************************/ 
IS
BEGIN
	Actualitza_F_Agents;
	Actualitza_F_Productes;
	Actualitza_F_Incidencies;
	Actualitza_F_Actuacions;
END;
/


 EXECUTE Actualitza_Dimensions();
 
 EXECUTE Actualitza_Fets();


COMMIT;
