/*
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Scripts de les taules de BD_POSTVENDA
*/


-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

COMMIT;


-- Create tables section -------------------------------------------------

-- Table Usuaris

CREATE TABLE Usuaris(
  IdUsuari Number NOT NULL,
  Nom Varchar2(25 ) NOT NULL,
  Cognoms Varchar2(30 ) NOT NULL,
  Login Varchar2(30 ) NOT NULL,
  Password Varchar2(15 ) NOT NULL,
  DataAlta Date NOT NULL,
  DataBaixa Date
)
/

-- Add keys for table Usuaris

ALTER TABLE Usuaris ADD CONSTRAINT Key1 PRIMARY KEY (IdUsuari)
/

ALTER TABLE Usuaris ADD CONSTRAINT Key2 UNIQUE (Login)
/

-- Table Permisos

CREATE TABLE Permisos(
  IdPermis Number NOT NULL,
  Permis Varchar2(30 ) NOT NULL
)
/

-- Add keys for table Permisos

ALTER TABLE Permisos ADD CONSTRAINT Key3 PRIMARY KEY (IdPermis)
/

ALTER TABLE Permisos ADD CONSTRAINT Key4 UNIQUE (Permis)
/

-- Table PermisosUsuaris

CREATE TABLE PermisosUsuaris(
  IdUsuari Number NOT NULL,
  IdPermis Number NOT NULL
)
/

-- Add keys for table PermisosUsuaris

ALTER TABLE PermisosUsuaris ADD CONSTRAINT Key5 PRIMARY KEY (IdUsuari,IdPermis)
/

-- Table LogExecucio

CREATE TABLE LogExecucio(
  IdExecucio Number NOT NULL,
  Inici Timestamp(0) NOT NULL,
  Fi Timestamp(0) NOT NULL,
  Procediment Varchar2(50 ) NOT NULL,
  ParamEntrada Varchar2(1000 ),
  ParamSortida Varchar2(1000 )
)
/

-- Add keys for table LogExecucio

ALTER TABLE LogExecucio ADD CONSTRAINT Key6 PRIMARY KEY (IdExecucio)
/

-- Table CodisPostal

CREATE TABLE CodisPostal(
  CodiPostal Number NOT NULL,
  IdLocalitat Number NOT NULL
)
/

-- Add keys for table CodisPostal

ALTER TABLE CodisPostal ADD CONSTRAINT Key7 PRIMARY KEY (CodiPostal)
/

-- Table Localitats

CREATE TABLE Localitats(
  IdLocalitat Number NOT NULL,
  Localitat Varchar2(30 ) NOT NULL,
  IdProvincia Number NOT NULL
)
/

-- Add keys for table Localitats

ALTER TABLE Localitats ADD CONSTRAINT Key8 PRIMARY KEY (IdLocalitat)
/

ALTER TABLE Localitats ADD CONSTRAINT Key9 UNIQUE (Localitat)
/

-- Table Provincies

CREATE TABLE Provincies(
  IdProvincia Number NOT NULL,
  Provincia Varchar2(30 ) NOT NULL,
  IdPais Number NOT NULL
)
/

-- Add keys for table Provincies

ALTER TABLE Provincies ADD CONSTRAINT Key37 PRIMARY KEY (IdProvincia)
/

ALTER TABLE Provincies ADD CONSTRAINT Key38 UNIQUE (Provincia)
/

-- Table Paisos

CREATE TABLE Paisos(
  IdPais Number NOT NULL,
  Pais Varchar2(30 ) NOT NULL
)
/

-- Add keys for table Paisos

ALTER TABLE Paisos ADD CONSTRAINT Key10 PRIMARY KEY (IdPais)
/

ALTER TABLE Paisos ADD CONSTRAINT Key11 UNIQUE (Pais)
/

-- Table Clients

CREATE TABLE Clients(
  IdClient Number NOT NULL,
  Nom Varchar2(25 ) NOT NULL,
  Cognoms Varchar2(30 ) NOT NULL,
  DNI Varchar2(15 ) NOT NULL,
  CodiPostal Number NOT NULL,
  Adreca Varchar2(50 ) NOT NULL,
  Telefon Varchar2(20 ) NOT NULL,
  Mobil Varchar2(20 ) NOT NULL,
  Email Varchar2(50 ),
  DataNaixement Date NOT NULL,
  Sexe Varchar2(1 ) NOT NULL
    CONSTRAINT  CHK_Clients_Sexe CHECK (Sexe IN ('H','D')),
  DataAlta Date NOT NULL,
  DataBaixa Date
)
/

-- Add keys for table Clients

ALTER TABLE Clients ADD CONSTRAINT Key12 PRIMARY KEY (IdClient)
/

ALTER TABLE Clients ADD CONSTRAINT Key13 UNIQUE (DNI)
/

-- Table Productes

CREATE TABLE Productes(
  Idproducte Number NOT NULL,
  IdTipusProducte Number NOT NULL,
  Descripcio Varchar2(200 ) NOT NULL,
  ISBN Varchar2(13 ) NOT NULL,
  NumeroSerie Varchar2(30 ) NOT NULL,
  DataVenda Date NOT NULL,
  IdCentre Number NOT NULL,
  IdGarantia Number NOT NULL,
  Alcada Number(9,2),
  Ample Number(9,2),
  Fons Number(9,2),
  DataAlta Date NOT NULL,
  DataBaixa Date
)
/

-- Add keys for table Productes

ALTER TABLE Productes ADD CONSTRAINT Key14 PRIMARY KEY (Idproducte)
/

ALTER TABLE Productes ADD CONSTRAINT Key15 UNIQUE (ISBN)
/

-- Table TipusProducte

CREATE TABLE TipusProducte(
  IdTipusProducte Number NOT NULL,
  TipusProducte Varchar2(50 ) NOT NULL,
  IdTipusPare Number,
  DataAlta Date NOT NULL,
  DataBaixa Date
)
/

-- Add keys for table TipusProducte

ALTER TABLE TipusProducte ADD CONSTRAINT Key16 PRIMARY KEY (IdTipusProducte)
/

ALTER TABLE TipusProducte ADD CONSTRAINT Key17 UNIQUE (TipusProducte)
/

-- Table Centres

CREATE TABLE Centres(
  IdCentre Number NOT NULL,
  Centre Varchar2(50 ) NOT NULL,
  CodiPostal Number NOT NULL,
  Adreca Varchar2(50 ) NOT NULL,
  Telefon Varchar2(30 ) NOT NULL,
  Contacte Varchar2(50 ) NOT NULL,
  DataAlta Date NOT NULL,
  DataBaixa Date
)
/

-- Add keys for table Centres

ALTER TABLE Centres ADD CONSTRAINT Key18 PRIMARY KEY (IdCentre)
/

ALTER TABLE Centres ADD CONSTRAINT Key19 UNIQUE (Centre)
/

-- Table Garanties

CREATE TABLE Garanties(
  IdGarantia Number NOT NULL,
  Garantia Varchar2(50 ) NOT NULL,
  AnysTotal Number NOT NULL,
  AnysInsitu Number
)
/

-- Add keys for table Garanties

ALTER TABLE Garanties ADD CONSTRAINT Key20 PRIMARY KEY (IdGarantia)
/

ALTER TABLE Garanties ADD CONSTRAINT Key21 UNIQUE (Garantia)
/

-- Table Agents

CREATE TABLE Agents(
  IdAgent Number NOT NULL,
  Nom Varchar2(25 ) NOT NULL,
  Cognoms Varchar2(30 ) NOT NULL,
  DataAlta Date NOT NULL,
  DataBaixa Date
)
/

-- Add keys for table Agents

ALTER TABLE Agents ADD CONSTRAINT Key22 PRIMARY KEY (IdAgent)
/

-- Table Perfils

CREATE TABLE Perfils(
  IdPerfil Number NOT NULL,
  Perfil Varchar2(30 ) NOT NULL,
  DataAlta Date NOT NULL,
  DataBaixa Date
)
/

-- Add keys for table Perfils

ALTER TABLE Perfils ADD CONSTRAINT Key23 PRIMARY KEY (IdPerfil)
/

ALTER TABLE Perfils ADD CONSTRAINT Key24 UNIQUE (Perfil)
/

-- Table AgentsPerfils

CREATE TABLE AgentsPerfils(
  IdAgent Number NOT NULL,
  IdPerfil Number NOT NULL
)
/

-- Add keys for table AgentsPerfils

ALTER TABLE AgentsPerfils ADD CONSTRAINT Key25 PRIMARY KEY (IdAgent,IdPerfil)
/

-- Table AgentsProvincies

CREATE TABLE AgentsProvincies(
  IdAgent Number NOT NULL,
  IdProvincia Number NOT NULL
)
/

-- Add keys for table AgentsProvincies

ALTER TABLE AgentsProvincies ADD CONSTRAINT Key26 PRIMARY KEY (IdAgent,IdProvincia)
/

-- Table Incidencies

CREATE TABLE Incidencies(
  IdIncidencia Number NOT NULL,
  IdTipusIncidencia Number NOT NULL,
  Descripcio Varchar2(2000 ) NOT NULL,
  IdClient Number NOT NULL,
  Idproducte Number NOT NULL,
  IdAgent Number NOT NULL,
  IdPrioritat Number NOT NULL,
  IdEstat Number NOT NULL,
  DataAlta Date NOT NULL,
  DataTancament Date,
  Observacions Varchar2(2000 )
)
/

-- Add keys for table Incidencies

ALTER TABLE Incidencies ADD CONSTRAINT Key27 PRIMARY KEY (IdIncidencia)
/

-- Table TipusIncidencia

CREATE TABLE TipusIncidencia(
  IdTipusIncidencia Number NOT NULL,
  TipusIncidencia Varchar2(50 ) NOT NULL,
  DataAlta Date NOT NULL,
  DataBaixa Date
)
/

-- Add keys for table TipusIncidencia

ALTER TABLE TipusIncidencia ADD CONSTRAINT Key28 PRIMARY KEY (IdTipusIncidencia)
/

ALTER TABLE TipusIncidencia ADD CONSTRAINT Key29 UNIQUE (TipusIncidencia)
/

-- Table Estats

CREATE TABLE Estats(
  IdEstat Number NOT NULL,
  Estat Varchar2(50 ) NOT NULL
)
/

-- Add keys for table Estats

ALTER TABLE Estats ADD CONSTRAINT Key30 PRIMARY KEY (IdEstat)
/

ALTER TABLE Estats ADD CONSTRAINT Key31 UNIQUE (Estat)
/

-- Table Prioritats

CREATE TABLE Prioritats(
  IdPrioritat Number NOT NULL,
  Prioritat Varchar2(30 ) NOT NULL
)
/

-- Add keys for table Prioritats

ALTER TABLE Prioritats ADD CONSTRAINT Key32 PRIMARY KEY (IdPrioritat)
/

ALTER TABLE Prioritats ADD CONSTRAINT Key33 UNIQUE (Prioritat)
/

-- Table Actuacions

CREATE TABLE Actuacions(
  IdActuacio Number NOT NULL,
  IdTipusActuacio Number NOT NULL,
  IdIncidencia Number NOT NULL,
  IdAgent Number NOT NULL,
  Descripcio Varchar2(2000 ) NOT NULL,
  DataInici Date NOT NULL,
  DataFi Date,
  Cost Number(9,2),
  Observacions Varchar2(2000 )
)
/

-- Add keys for table Actuacions

ALTER TABLE Actuacions ADD CONSTRAINT Key34 PRIMARY KEY (IdActuacio)
/

-- Table TipusActuacio

CREATE TABLE TipusActuacio(
  IdTipusActuacio Number NOT NULL,
  TipusActuacio Varchar2(50 ) NOT NULL,
  DataAlta Date NOT NULL,
  DataBaixa Date
)
/

-- Add keys for table TipusActuacio

ALTER TABLE TipusActuacio ADD CONSTRAINT Key35 PRIMARY KEY (IdTipusActuacio)
/

ALTER TABLE TipusActuacio ADD CONSTRAINT Key36 UNIQUE (TipusActuacio)
/

-- Create relationships section ------------------------------------------------- 

ALTER TABLE PermisosUsuaris ADD CONSTRAINT R1 FOREIGN KEY (IdUsuari) REFERENCES Usuaris (IdUsuari)
/

ALTER TABLE PermisosUsuaris ADD CONSTRAINT R2 FOREIGN KEY (IdPermis) REFERENCES Permisos (IdPermis)
/

ALTER TABLE Provincies ADD CONSTRAINT R3 FOREIGN KEY (IdPais) REFERENCES Paisos (IdPais)
/

ALTER TABLE Localitats ADD CONSTRAINT R4 FOREIGN KEY (IdProvincia) REFERENCES Provincies (IdProvincia)
/

ALTER TABLE CodisPostal ADD CONSTRAINT R5 FOREIGN KEY (IdLocalitat) REFERENCES Localitats (IdLocalitat)
/

ALTER TABLE Clients ADD CONSTRAINT R6 FOREIGN KEY (CodiPostal) REFERENCES CodisPostal (CodiPostal)
/

ALTER TABLE Productes ADD CONSTRAINT R7 FOREIGN KEY (IdTipusProducte) REFERENCES TipusProducte (IdTipusProducte)
/

ALTER TABLE Productes ADD CONSTRAINT R8 FOREIGN KEY (IdCentre) REFERENCES Centres (IdCentre)
/

ALTER TABLE Centres ADD CONSTRAINT R9 FOREIGN KEY (CodiPostal) REFERENCES CodisPostal (CodiPostal)
/

ALTER TABLE Productes ADD CONSTRAINT R10 FOREIGN KEY (IdGarantia) REFERENCES Garanties (IdGarantia)
/

ALTER TABLE TipusProducte ADD CONSTRAINT R11 FOREIGN KEY (IdTipusPare) REFERENCES TipusProducte (IdTipusProducte)
/

ALTER TABLE AgentsPerfils ADD CONSTRAINT R12 FOREIGN KEY (IdAgent) REFERENCES Agents (IdAgent)
/

ALTER TABLE AgentsPerfils ADD CONSTRAINT R13 FOREIGN KEY (IdPerfil) REFERENCES Perfils (IdPerfil)
/

ALTER TABLE AgentsProvincies ADD CONSTRAINT R14 FOREIGN KEY (IdAgent) REFERENCES Agents (IdAgent)
/

ALTER TABLE AgentsProvincies ADD CONSTRAINT R15 FOREIGN KEY (IdProvincia) REFERENCES Provincies (IdProvincia)
/

ALTER TABLE Incidencies ADD CONSTRAINT R16 FOREIGN KEY (IdTipusIncidencia) REFERENCES TipusIncidencia (IdTipusIncidencia)
/

ALTER TABLE Incidencies ADD CONSTRAINT R17 FOREIGN KEY (IdClient) REFERENCES Clients (IdClient)
/

ALTER TABLE Incidencies ADD CONSTRAINT R18 FOREIGN KEY (Idproducte) REFERENCES Productes (Idproducte)
/

ALTER TABLE Incidencies ADD CONSTRAINT R19 FOREIGN KEY (IdAgent) REFERENCES Agents (IdAgent)
/

ALTER TABLE Incidencies ADD CONSTRAINT R20 FOREIGN KEY (IdEstat) REFERENCES Estats (IdEstat)
/

ALTER TABLE Incidencies ADD CONSTRAINT R21 FOREIGN KEY (IdPrioritat) REFERENCES Prioritats (IdPrioritat)
/

ALTER TABLE Actuacions ADD CONSTRAINT R22 FOREIGN KEY (IdTipusActuacio) REFERENCES TipusActuacio (IdTipusActuacio)
/

ALTER TABLE Actuacions ADD CONSTRAINT R23 FOREIGN KEY (IdIncidencia) REFERENCES Incidencies (IdIncidencia)
/

ALTER TABLE Actuacions ADD CONSTRAINT R24 FOREIGN KEY (IdAgent) REFERENCES Agents (IdAgent)
/



-- Grant permissions section -------------------------------------------------


