/*
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Scripts de les taules de DW_POSTVENDA
*/


-- Schema DW_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = DW_POSTVENDA;

COMMIT;


-- Create tables section -------------------------------------------------

-- Table F_Agents

CREATE TABLE F_Agents(
  IdTemps Number NOT NULL,
  IdUbicacio Number NOT NULL,
  IdPerfil Number NOT NULL,
  NumAlta Number NOT NULL,
  NumBaixa Number NOT NULL
)
/

-- Add keys for table F_Agents

ALTER TABLE F_Agents ADD CONSTRAINT Key1 PRIMARY KEY (IdTemps,IdUbicacio,IdPerfil)
/

-- Table F_Productes

CREATE TABLE F_Productes(
  IdTemps Number NOT NULL,
  IdCentre Number NOT NULL,
  IdTipusProducte Number NOT NULL,
  NumAlta Integer NOT NULL,
  NumBaixa Number NOT NULL
)
/

-- Add keys for table F_Productes

ALTER TABLE F_Productes ADD CONSTRAINT Key2 PRIMARY KEY (IdTemps,IdCentre,IdTipusProducte)
/

-- Table F_Incidencies

CREATE TABLE F_Incidencies(
  IdTemps Number NOT NULL,
  IdProducte Number NOT NULL,
  IdClient Number NOT NULL,
  IdTipusIncidencia Number NOT NULL,
  IdAgent Number NOT NULL,
  IdPrioritat Number NOT NULL,
  IdEstat Number NOT NULL,
  NumRegistrades Number NOT NULL,
  NumTancades Number NOT NULL
)
/

-- Add keys for table F_Incidencies

ALTER TABLE F_Incidencies ADD CONSTRAINT Key3 PRIMARY KEY (IdTemps,IdProducte,IdClient,IdTipusIncidencia,IdAgent,IdPrioritat,IdEstat)
/

-- Table F_Actuacions

CREATE TABLE F_Actuacions(
  IdTemps Number NOT NULL,
  IdTipusActuacio Number NOT NULL,
  IdIncidencia Number NOT NULL,
  IdAgent Number NOT NULL,
  NumRegistrades Number NOT NULL,
  NumFinalitzades Number NOT NULL,
  Cost Number(9,2) NOT NULL
)
/

-- Add keys for table F_Actuacions

ALTER TABLE F_Actuacions ADD CONSTRAINT Key4 PRIMARY KEY (IdTemps,IdTipusActuacio,IdIncidencia,IdAgent)
/

-- Table D_Temps

CREATE TABLE D_Temps(
  Mes Number NOT NULL,
  Anyo Number NOT NULL
)
/

-- Add keys for table D_Temps

ALTER TABLE D_Temps ADD CONSTRAINT Key5 PRIMARY KEY (Mes)
/

-- Table D_Ubicacions

CREATE TABLE D_Ubicacions(
  IdUbicacio Number NOT NULL,
  Provincia Varchar2(30 ) NOT NULL,
  Pais Varchar2(30 ) NOT NULL
)
/

-- Add keys for table D_Ubicacions

ALTER TABLE D_Ubicacions ADD CONSTRAINT Key6 PRIMARY KEY (IdUbicacio)
/

-- Table D_Perfils

CREATE TABLE D_Perfils(
  IdPerfil Number NOT NULL,
  Perfil Varchar2(30 ) NOT NULL
)
/

-- Add keys for table D_Perfils

ALTER TABLE D_Perfils ADD CONSTRAINT Key7 PRIMARY KEY (IdPerfil)
/

-- Table D_Centres

CREATE TABLE D_Centres(
  IdCentre Number NOT NULL,
  Centre Varchar2(50 ) NOT NULL
)
/

-- Add keys for table D_Centres

ALTER TABLE D_Centres ADD CONSTRAINT Key8 PRIMARY KEY (IdCentre)
/

-- Table D_TipusProducte

CREATE TABLE D_TipusProducte(
  IdTipusProducte Number NOT NULL,
  TipusProducte Varchar2(50 ) NOT NULL
)
/

-- Add keys for table D_TipusProducte

ALTER TABLE D_TipusProducte ADD CONSTRAINT Key9 PRIMARY KEY (IdTipusProducte)
/

-- Table D_Productes

CREATE TABLE D_Productes(
  IdProducte Number NOT NULL,
  Producte Varchar2(200 ) NOT NULL
)
/

-- Add keys for table D_Productes

ALTER TABLE D_Productes ADD CONSTRAINT Key10 PRIMARY KEY (IdProducte)
/

-- Table D_Clients

CREATE TABLE D_Clients(
  IdClient Number NOT NULL,
  Client Varchar2(60 ) NOT NULL
)
/

-- Add keys for table D_Clients

ALTER TABLE D_Clients ADD CONSTRAINT Key11 PRIMARY KEY (IdClient)
/

-- Table D_TipusIncidencia

CREATE TABLE D_TipusIncidencia(
  IdTipusIncidencia Number NOT NULL,
  TipusIncidencia Varchar2(50 ) NOT NULL
)
/

-- Add keys for table D_TipusIncidencia

ALTER TABLE D_TipusIncidencia ADD CONSTRAINT Key12 PRIMARY KEY (IdTipusIncidencia)
/

-- Table D_Agents

CREATE TABLE D_Agents(
  IdAgent Number NOT NULL,
  Agent Varchar2(60 ) NOT NULL
)
/

-- Add keys for table D_Agents

ALTER TABLE D_Agents ADD CONSTRAINT Key13 PRIMARY KEY (IdAgent)
/

-- Table D_Prioritats

CREATE TABLE D_Prioritats(
  IdPrioritat Number NOT NULL,
  Prioritat Varchar2(30 ) NOT NULL
)
/

-- Add keys for table D_Prioritats

ALTER TABLE D_Prioritats ADD CONSTRAINT Key14 PRIMARY KEY (IdPrioritat)
/

-- Table D_Estats

CREATE TABLE D_Estats(
  IdEstat Number NOT NULL,
  Estat Varchar2(30 ) NOT NULL
)
/

-- Add keys for table D_Estats

ALTER TABLE D_Estats ADD CONSTRAINT Key15 PRIMARY KEY (IdEstat)
/

-- Table D_TipusActuacio

CREATE TABLE D_TipusActuacio(
  IdTipusActuacio Number NOT NULL,
  TipusActuacio Varchar2(50 ) NOT NULL
)
/

-- Add keys for table D_TipusActuacio

ALTER TABLE D_TipusActuacio ADD CONSTRAINT Key16 PRIMARY KEY (IdTipusActuacio)
/

-- Table D_Incidencies

CREATE TABLE D_Incidencies(
  IdIncidencia Number NOT NULL,
  Incidencia Varchar2(2000 ) NOT NULL
)
/

-- Add keys for table D_Incidencies

ALTER TABLE D_Incidencies ADD CONSTRAINT Key17 PRIMARY KEY (IdIncidencia)
/

-- Create relationships section ------------------------------------------------- 

ALTER TABLE F_Agents ADD CONSTRAINT R1 FOREIGN KEY (IdTemps) REFERENCES D_Temps (Mes)
/

ALTER TABLE F_Agents ADD CONSTRAINT R2 FOREIGN KEY (IdUbicacio) REFERENCES D_Ubicacions (IdUbicacio)
/

ALTER TABLE F_Agents ADD CONSTRAINT R3 FOREIGN KEY (IdPerfil) REFERENCES D_Perfils (IdPerfil)
/

ALTER TABLE F_Productes ADD CONSTRAINT R4 FOREIGN KEY (IdTemps) REFERENCES D_Temps (Mes)
/

ALTER TABLE F_Productes ADD CONSTRAINT R5 FOREIGN KEY (IdCentre) REFERENCES D_Centres (IdCentre)
/

ALTER TABLE F_Productes ADD CONSTRAINT R6 FOREIGN KEY (IdTipusProducte) REFERENCES D_TipusProducte (IdTipusProducte)
/

ALTER TABLE F_Incidencies ADD CONSTRAINT R7 FOREIGN KEY (IdTemps) REFERENCES D_Temps (Mes)
/

ALTER TABLE F_Incidencies ADD CONSTRAINT R8 FOREIGN KEY (IdProducte) REFERENCES D_Productes (IdProducte)
/

ALTER TABLE F_Incidencies ADD CONSTRAINT R9 FOREIGN KEY (IdClient) REFERENCES D_Clients (IdClient)
/

ALTER TABLE F_Incidencies ADD CONSTRAINT R10 FOREIGN KEY (IdTipusIncidencia) REFERENCES D_TipusIncidencia (IdTipusIncidencia)
/

ALTER TABLE F_Incidencies ADD CONSTRAINT R11 FOREIGN KEY (IdAgent) REFERENCES D_Agents (IdAgent)
/

ALTER TABLE F_Incidencies ADD CONSTRAINT R12 FOREIGN KEY (IdPrioritat) REFERENCES D_Prioritats (IdPrioritat)
/

ALTER TABLE F_Incidencies ADD CONSTRAINT R13 FOREIGN KEY (IdEstat) REFERENCES D_Estats (IdEstat)
/

ALTER TABLE F_Actuacions ADD CONSTRAINT R14 FOREIGN KEY (IdTemps) REFERENCES D_Temps (Mes)
/

ALTER TABLE F_Actuacions ADD CONSTRAINT R15 FOREIGN KEY (IdTipusActuacio) REFERENCES D_TipusActuacio (IdTipusActuacio)
/

ALTER TABLE F_Actuacions ADD CONSTRAINT R16 FOREIGN KEY (IdIncidencia) REFERENCES D_Incidencies (IdIncidencia)
/

ALTER TABLE F_Actuacions ADD CONSTRAINT R17 FOREIGN KEY (IdAgent) REFERENCES D_Agents (IdAgent)
/



-- Grant permissions section -------------------------------------------------


