/*---------------------------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Procediments comuns
----------------------------------------------------------------------*/


-- Schema BD_POSTVENDA
 ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

 COMMIT;


CREATE OR REPLACE PROCEDURE Grabar_LogExecucio (
/********************************************************************************
	NOM:			Grabar_LogExecucio

	DESCRIPCI: 	Procediment que desa el LOG d'execuci dels diferents
				procediments. Es desa la data en la qual es produeix el procediment
				i els parmetres d'entrada i sortida.
  
				Fa s de PRAGMA AUTONOMOUS_TRANSACTION que ens permet que aquesta
				subrutina es comporti com una transacci diferent a la del
				procs principal i per tant ens permetr portar el control de
				COMMIT o ROLLBACK de forma independent
********************************************************************************/
	p_Inici		LogExecucio.Inici%TYPE,
	p_Fi			LogExecucio.Fi%TYPE,
	p_Procediment	LogExecucio.Procediment%TYPE,
	p_ParamEntrada	LogExecucio.ParamEntrada%TYPE,
	p_ParamSortida	LogExecucio.ParamSortida%TYPE
)
IS
	PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
	INSERT INTO LogExecucio (Inici, Fi, Procediment, ParamEntrada, ParamSortida)
	VALUES (p_Inici, p_Fi, p_Procediment, p_ParamEntrada, p_ParamSortida);

	COMMIT;
END;


CREATE OR REPLACE PUBLIC SYNONYM Grabar_LogExecucio FOR Grabar_LogExecucio;

GRANT EXECUTE ON Grabar_LogExecucio TO PUBLIC;


CREATE OR REPLACE FUNCTION Es_Numero (p_Param VARCHAR2)
	RETURN BOOLEAN
IS
/******************************************************************************
	NOM:			Es_Numero

	DESCRIPCI:		Comprova si el parmetre s numric o no.
******************************************************************************/
	v_n	NUMBER;
BEGIN
	v_n := TO_NUMBER (p_Param);
	RETURN TRUE;
EXCEPTION
	WHEN OTHERS
	THEN
		RETURN FALSE;
END;


CREATE OR REPLACE PUBLIC SYNONYM Es_Numero FOR Es_Numero ;

GRANT EXECUTE ON Es_Numero TO PUBLIC;


CREATE OR REPLACE FUNCTION Es_Data 
/******************************************************************************
	NOM:			Es_Data
	DESCRIPCI:		Comprova si el parmetre p_Data compleix el format de data
				especificat pel parmetre p_format. Per defecte el format
				de la data ser DD/MM/AAAA
******************************************************************************/
(
	p_Data	VARCHAR2,
	p_Format	VARCHAR2 := 'DD/MM/YYYY'
)
	RETURN BOOLEAN
IS
   v_Data_Text	DATE;
BEGIN
	v_Data_Text := TO_DATE (p_Data, p_Format);
	RETURN TRUE;
EXCEPTION
	WHEN OTHERS
	THEN
		RETURN FALSE;
END;


CREATE OR REPLACE PUBLIC SYNONYM Es_Data FOR Es_Data ;

GRANT EXECUTE ON Es_Data TO PUBLIC;


CREATE OR REPLACE FUNCTION Anyo (p_Data DATE)
	RETURN NUMBER
IS
/********************************************************************************
	NOM:			Anyo

	DESCRIPCI:		Retorna l'any de la data passada per parmetre
********************************************************************************/
BEGIN
	RETURN TO_NUMBER (TO_CHAR (p_Data, 'YYYY'));
END;


CREATE OR REPLACE PUBLIC SYNONYM Anyo FOR Anyo;

GRANT EXECUTE ON Anyo TO PUBLIC;


CREATE OR REPLACE FUNCTION Mes (p_Data DATE)
	RETURN NUMBER
IS
/********************************************************************************
	NOM:			Mes

	DESCRIPCI:		Retorna el mes de la data passada per parmetre mab format AAAAMM
********************************************************************************/
BEGIN
	RETURN TO_CHAR (p_Data, 'YYYY') * 100 + TO_CHAR (p_Data, 'MM');
END;


CREATE OR REPLACE PUBLIC SYNONYM Mes FOR Mes ;

GRANT EXECUTE ON Mes TO PUBLIC;


CREATE OR REPLACE FUNCTION Dia (p_Data DATE)
	RETURN DATE
IS
/**********************************************************************************
	NOM:			Dia

	DESCRIPCI:		Retorna el dia de la data passada per parmetre
**********************************************************************************/
BEGIN
	RETURN TO_DATE (TO_CHAR (p_Data, 'DD/MM/YYYY'));
END;


CREATE OR REPLACE PUBLIC SYNONYM Dia FOR Dia ;

GRANT EXECUTE ON Dia TO PUBLIC;


COMMIT ;