/*---------------------------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Vistes emprades a les Bases de Dades
----------------------------------------------------------------------*/


-- Schema BD_POSTVENDA
 ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;
 
 COMMIT;


/******************************************************************************
	VISTA:		vi_Ubicacions

	DESCRIPCI:		Vista que mostra les ubicacions: 
				CodiPostal, localitat, provncia i pas
******************************************************************************/    
CREATE OR REPLACE VIEW vi_Ubicacions AS
    SELECT CP.CodiPostal, CP.IdLocalitat, LO.Localitat, LO.IdProvincia, PR.Provincia, PR.IdPais, PA.Pais
	FROM CodisPostal CP, Localitats LO, Provincies PR, Paisos PA
    WHERE 
		CP.IdLocalitat = LO.IdLocalitat AND  
		LO.IdProvincia = PR.IdProvincia AND 
		PR.IdPais = PA.IdPais;   


CREATE OR REPLACE PUBLIC SYNONYM vi_Ubicacions FOR vi_Ubicacions;


/******************************************************************************
	VISTA:		vi_Localitats

	DESCRIPCI:		Vista que mostra les dades de la localitat, provncia i pas
******************************************************************************/    
CREATE OR REPLACE VIEW vi_Localitats AS
	SELECT LO.*, PR.Provincia, PA.Pais  
	FROM Localitats LO, Provincies PR, Paisos PA
	WHERE LO.IdProvincia = PR.IdProvincia AND PR.IdPais = PA.IdPais; 


CREATE OR REPLACE PUBLIC SYNONYM vi_Localitats FOR vi_Localitats;


/******************************************************************************
	VISTA:		vi_Provincies

	DESCRIPCI:		Vista que mostra les dades de la provncia i pas
******************************************************************************/    
CREATE OR REPLACE VIEW vi_Provincies AS
	SELECT PR.IdProvincia, PR.Provincia, PA.IdPais, PA.Pais
	From Provincies PR, Paisos PA
	WHERE PR.IdPais = PA.IdPais; 
    
CREATE OR REPLACE PUBLIC SYNONYM vi_Provincies FOR vi_Provincies;
    

-- Schema ETL_POSTVENDA
 ALTER SESSION SET CURRENT_SCHEMA = ETL_POSTVENDA;
 
 COMMIT;


/******************************************************************************
	VISTA:		vi_D_Temps

	DESCRIPCI:		Vista que mostra les dades que s'emmagatzemaran a la dimensi temps
				Agafa les dades 'data' de les taules que intervenen als fets
******************************************************************************/    
CREATE OR REPLACE VIEW vi_D_Temps AS
	SELECT DISTINCT Mes(DataAlta) AS Mes, Anyo(DataAlta) AS Anyo 
	FROM Productes
	WHERE DataAlta IS NOT NULL
	UNION
	SELECT DISTINCT Mes(DataBaixa), Anyo(DataBaixa) 
	FROM Productes
	WHERE DataBaixa IS NOT NULL
	UNION
	SELECT DISTINCT Mes(DataAlta), Anyo(DataAlta) 
	FROM Agents
	WHERE DataAlta IS NOT NULL
	UNION
	SELECT DISTINCT Mes(DataBaixa), Anyo(DataBaixa) 
	FROM Agents
	WHERE DataBaixa IS NOT NULL
	UNION
	SELECT DISTINCT Mes(DataAlta), Anyo(DataAlta) 
	FROM Incidencies
	WHERE DataAlta IS NOT NULL
	UNION
	SELECT DISTINCT Mes(DataTancament), Anyo(DataTancament) 
	FROM Incidencies
	WHERE DataTancament IS NOT NULL
	UNION
	SELECT DISTINCT Mes(DataInici), Anyo(DataInici) 
	FROM Actuacions
	WHERE DataInici IS NOT NULL
	UNION
	SELECT DISTINCT Mes(DataFi), Anyo(DataFi) 
	FROM Actuacions
	WHERE DataFi IS NOT NULL;


CREATE OR REPLACE PUBLIC SYNONYM vi_D_Temps FOR vi_D_Temps;


/******************************************************************************
	VISTA:		vi_F_Agents

	DESCRIPCI:		Vista que mostra les dades que s'emmagatzemaran al fet agents
 ******************************************************************************/    
CREATE OR REPLACE VIEW vi_F_Agents AS
	SELECT Mes, IdUbicacio, IdPerfil, NVL(MAX(NumAlta), 0) AS NumAlta, NVL(MAX(NumBaixa), 0) AS NumBaixa
	FROM
	(
		SELECT Mes(DataAlta) AS Mes, V.IdProvincia AS IdUbicacio, AE.IdPerfil, COUNT(*) AS NumAlta, 0 AS NumBaixa
		FROM Agents A, vi_Provincies V, AgentsProvincies AP, AgentsPerfils AE
		WHERE DataAlta IS NOT NULL AND DataBaixa IS NULL AND
		A.IdAgent = AP.IdAgent AND 
        A.IdAgent = AE.IdAgent AND 
		AP.IdProvincia = V.IdProvincia
		GROUP BY Mes(DataAlta), V.IdProvincia, AE.IdPerfil
		UNION ALL  
		SELECT Mes(DataBaixa) AS Mes, V.IdProvincia AS IdUbicacio, AE.IdPerfil, 0 AS NumAlta, COUNT(*) AS NumBaixa
		FROM Agents A, vi_Provincies V, AgentsProvincies AP, AgentsPerfils AE
		WHERE DataBaixa IS NOT NULL AND 
		A.IdAgent = AP.IdAgent AND 
        A.IdAgent = AE.IdAgent AND 
		AP.IdProvincia =  V.IdProvincia
		GROUP BY Mes(DataBaixa), V.IdProvincia, AE.IdPerfil) 
	GROUP BY Mes, IdUbicacio, IdPerfil; 


CREATE OR REPLACE PUBLIC SYNONYM vi_F_Agents FOR vi_F_Agents;


/******************************************************************************
	VISTA:		vi_F_Productes

	DESCRIPCI:		Vista que mostra les dades que s'emmagatzemaran al fet productes
 ******************************************************************************/    
CREATE OR REPLACE VIEW vi_F_Productes AS
	SELECT Mes, IdCentre, IdTipusProducte, NVL(MAX(NumAlta), 0) AS NumAlta, NVL(MAX(NumBaixa), 0) AS NumBaixa
	FROM
	(
		SELECT Mes(DataAlta) AS Mes, IdCentre, IdTipusProducte, COUNT(*) AS NumAlta, 0 AS NumBaixa
		FROM Productes
		WHERE DataAlta IS NOT NULL AND DataBaixa IS NULL
		GROUP BY Mes(DataAlta), IdCentre, IdTipusProducte
		UNION ALL  
		SELECT Mes(DataBaixa) AS Mes, IdCentre, IdTipusProducte, 0 AS NumAlta, COUNT(*) AS NumBaixa
		FROM Productes
		WHERE DataBaixa IS NOT NULL
		GROUP BY Mes(DataBaixa), IdCentre, IdTipusProducte) 
	GROUP BY Mes, IdCentre, IdTipusProducte; 


CREATE OR REPLACE PUBLIC SYNONYM vi_F_Productes FOR vi_F_Productes;


/******************************************************************************
	VISTA:		vi_F_Incidencies

	DESCRIPCI:		Vista que mostra les dades que s'emmagatzemaran al fet incidncies
 ******************************************************************************/    
CREATE OR REPLACE VIEW vi_F_Incidencies AS
	SELECT Mes, IdProducte, IdClient, IdTipusIncidencia, IdAgent, IdPrioritat, IdEstat, NVL(MAX(NumRegistrades), 0) AS NumRegistrades, NVL(MAX(NumTancades), 0) AS NumTancades
	FROM
	(
		SELECT Mes(DataAlta) AS Mes, IdProducte, IdClient, IdTipusIncidencia, IdAgent, IdPrioritat, IdEstat, COUNT(*) AS NumRegistrades, 0 AS NumTancades
		FROM Incidencies
		WHERE DataAlta IS NOT NULL
		GROUP BY Mes(DataAlta), IdProducte, IdClient, IdTipusIncidencia, IdAgent, IdPrioritat, IdEstat
		UNION ALL  
		SELECT Mes(DataTancament) AS Mes, IdProducte, IdClient, IdTipusIncidencia, IdAgent, IdPrioritat, IdEstat, 0 AS NumRegistrades, COUNT(*) AS NumTancades
		FROM Incidencies
		WHERE DataTancament IS NOT NULL
		GROUP BY Mes(DataTancament), IdProducte, IdClient, IdTipusIncidencia, IdAgent, IdPrioritat, IdEstat) 
	GROUP BY Mes, IdProducte, IdClient, IdTipusIncidencia, IdAgent, IdPrioritat, IdEstat; 


CREATE OR REPLACE PUBLIC SYNONYM vi_F_Incidencies FOR vi_F_Incidencies;


/******************************************************************************
	VISTA:		vi_F_Actuacions

	DESCRIPCI:		Vista que mostra les dades que s'emmagatzemaran al fet actuacions
 ******************************************************************************/    
CREATE OR REPLACE VIEW vi_F_Actuacions AS
	SELECT Mes, IdTipusActuacio, IdIncidencia, IdAgent, NVL(MAX(NumRegistrades), 0) AS NumRegistrades, NVL(MAX(NumFinalitzades), 0) AS NumFinalitzades, NVL(MAX(Costos), 0) as Costos
	FROM
	(
		SELECT Mes(DataInici) AS Mes, IdTipusActuacio, IdIncidencia, IdAgent, COUNT(*) AS NumRegistrades, 0 AS NumFinalitzades, 0 AS Costos
		FROM Actuacions
		WHERE DataInici IS NOT NULL
		GROUP BY Mes(DataInici), IdTipusActuacio, IdIncidencia, IdAgent
		UNION ALL  
		SELECT Mes(DataFi) AS Mes, IdTipusActuacio, IdIncidencia, IdAgent, 0 AS NumRegistrades, COUNT(*) AS NumFinalitzades, 0 AS Costos
		FROM Actuacions
		WHERE DataFi IS NOT NULL
		GROUP BY Mes(DataFi), IdTipusActuacio, IdIncidencia, IdAgent
		UNION ALL  
		SELECT Mes(DataInici) AS Mes, IdTipusActuacio, IdIncidencia, IdAgent, 0 AS NumRegistrades, 0 AS NumFinalitzades, SUM(Cost) AS Costos
		FROM Actuacions
		WHERE DataInici IS NOT NULL
		GROUP BY Mes(DataInici), IdTipusActuacio, IdIncidencia, IdAgent) 
	GROUP BY Mes, IdTipusActuacio, IdIncidencia, IdAgent; 


CREATE OR REPLACE PUBLIC SYNONYM vi_F_Actuacions FOR vi_F_Actuacions;


COMMIT;
