/*---------------------------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
--Base de Dades: Oracle 10g
Descripci: Procediments del subsistema de Control d'Accessos
----------------------------------------------------------------------*/


-- Schema BD_POSTVENDA
 ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

 COMMIT;


-------------------------------------------------------------------------------
-- Usuaris
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Alta_Usuari (
/******************************************************************************
	NOM:			Alta_Usuari

	REQUERIMENT N: 	1

	DESCRIPCI:		Dna d'alta un usuari al nostre sistema

	PARMETRES:

		p_Nom			Nom de l'usuari
		p_Cognoms		Cognoms de l'usuari
		p_Login		Login de l'usuari
		p_Password		Password de l'usuari
		p_DataAlta		Data d'Alta de l'usuari (Per defecte, la data del sistema)

******************************************************************************/
	p_Nom		IN	Usuaris.Nom%TYPE,
	p_Cognoms	IN	Usuaris.Cognoms%TYPE,
	p_Login	IN	Usuaris.Login%TYPE,
	p_Password	IN	Usuaris.Password%TYPE,
	p_DataAlta	IN	Usuaris.DataAlta%TYPE := Dia(SYSDATE)
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_Nom: '
		|| p_Nom
		|| ' p_Cognoms: '
		|| p_Cognoms
		|| ' p_Login: '
		|| p_Login
		|| ' p_Password: '
		|| p_Password
		|| ' p_DataAlta: '
		|| p_DataAlta;

	INSERT INTO Usuaris (Nom, Cognoms, Login, Password, DataAlta)
		VALUES (p_Nom, p_Cognoms, p_Login, p_Password, p_DataAlta);

	-- Log
	v_Fi := SYSDATE;
	v_Res := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix un altre usuari amb Login ' || p_Login;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Usuari (
/******************************************************************************
	NOM:			Modificar_Usuari

	REQUERIMENT N:	2

	DESCRIPCI:		Permet modificar les dades associades a un usuari del nostre sistema

	PARMETRES:

		p_IdUsuari		Identificador de l'usuari
		p_Nom			Nom de l'usuari
		p_Cognoms		Cognoms de l'usuari
		p_Login		Login de l'usuari
		p_Password		Password de l'usuari
		p_DataAlta		Data d'Alta de l'usuari (Per defecte, mant la data actual)

******************************************************************************/
	p_IdUsuari	IN	Usuaris.IdUsuari%TYPE,
	p_Nom		IN	Usuaris.Nom%TYPE,
	p_Cognoms	IN	Usuaris.Cognoms%TYPE,
	p_Login	IN	Usuaris.Login%TYPE,
	p_Password	IN	Usuaris.Password%TYPE,
	p_DataAlta	IN	Usuaris.DataAlta%TYPE :=  NULL
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdUsuari: '
		|| p_IdUsuari
		|| ' p_Nom: '
		|| p_Nom
		|| ' p_Cognoms: '
		|| p_Cognoms
		|| ' p_Login: '
		|| p_Login
		|| ' p_Password: '
		|| p_Password
		|| ' p_DataAlta: '
		|| p_DataAlta;

	UPDATE Usuaris
	SET Nom = p_Nom,
		Cognoms = p_Cognoms,
		Login = p_Login,
		Password = p_Password,
		DataAlta = NVL (Dia(p_DataAlta), DataAlta)
	WHERE IdUsuari = p_IdUsuari AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Usuari amb IdUsuari=' || p_IdUsuari || ' inexistent o donat de baixa';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Ja existeix un altre usuari amb Login ' || p_Login;
		ELSIF SQLCODE = -1407
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Baixa_Usuari (
/******************************************************************************
	NOM:			Baixa_Usuari

	REQUERIMENT N:	3

	DESCRIPCI:		Dna de baixa un usuari del nostre sistema.
				Si l'usuari no t cap perms associat s'eliminar,
				en cas contrari es marcar com donat de baixa
                
	PARMETRES:

		p_IdUsuari		Identificador de l'usuari

******************************************************************************/
	p_IdUsuari	IN	Usuaris.IdUsuari%TYPE,
	p_DataBaixa	IN	Usuaris.DataBaixa%TYPE:=Dia(SYSDATE)
   
   )
IS
	v_DataAlta		Usuaris.DataAlta%TYPE;
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := 
		' p_IdUsuari: '
		|| p_IdUsuari 
		|| ' p_DataBaixa: ' 
		|| p_DataBaixa ;

	-- Comprovem que l'usuari existeixi i estigui donat d'alta
	v_Res := 'Usuari amb IdUsuari=' || p_IdUsuari || ' inexistent o donat de baixa';

	SELECT DataAlta INTO v_DataAlta 
	FROM Usuaris
	WHERE IdUsuari = p_IdUsuari AND DataBaixa IS NULL;
   
	-- Comprovem que la data de baixa no sigui anterior a la d'alta
	if v_DataAlta > p_DataBaixa then
		v_Res := 'La data alta del usuari no pot ser anterior a la de baixa';
		raise_application_error (-20000, '');
	end if;   


	-- Eliminem l'usuari
	DELETE Usuaris
	WHERE IdUsuari = p_IdUsuari AND DataBaixa IS NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Usuari amb IdUsuari=' || p_IdUsuari || ' inexistent o donat de baixa';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -2292
		THEN
			-- T permisos associats. Es dna de baixa
			UPDATE Usuaris
			SET DataBaixa = Dia(SYSDATE)
			WHERE IdUsuari = p_IdUsuari AND DataBaixa IS NULL;

			v_Res := 'OK';
		ELSE
			IF v_Res IS NULL
			THEN
				-- Error no controlat per codi
				v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
			ELSE
				-- Error controlat per codi
				v_Res := 'Error:' || v_Res;
			END IF;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Recuperar_Usuari (
/******************************************************************************
	NOMBRE:		Recuperar_Usuari
   
	REQUERIMENT N:	4

	DESCRIPCI:		Permet recuperar un usuari del nostre sistema
                
	PARMETRES:

		p_IdUsuari		Identificador de l'usuari

******************************************************************************/
	p_IdUsuari	IN	Usuaris.IdUsuari%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdUsuari: ' || p_IdUsuari;

	UPDATE Usuaris
	SET DataBaixa = NULL
	WHERE IdUsuari = p_IdUsuari AND DataBaixa IS NOT NULL;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Usuari amb IdUsuari=' || p_IdUsuari || ' no est donat de baixa o inexistent';
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;




CREATE OR REPLACE PROCEDURE Obtenir_Usuari
/******************************************************************************
   
	NOMBRE:		Obtenir_Usuari
   
	REQUERIMENT N:	5

	DESCRIPCIN:	Permet obtenir una relaci dusuaris, en funci dalguns parmetres.
				Si especifiquem un identificador, ens retornar, si existeix, lusuari
				amb aquest identificador.
				Si no especifiquem lidentificador (null), retornar els usuaris el nom
				dels quals contingui el nom especificat per parmetre.
				Tamb tindrem lopci, per parmetre, de decidir si han de sortir o no
				els usuaris donats de baixa  
                
	PARMETRES:

		p_RS			Recordset amb el resultat
		p_IdUsuari		Identificador de l'usuari a cercar
		p_Usuari		Usuari a cercar
		p_InclouBaixes	Indica si es tindran en compte tamb els que estan donats de baixa.
					Per defecte no es tindran en compte (p_InclouBaixes=0)

******************************************************************************/

(   
	p_RS 			IN OUT	sys_refcursor,
	p_IdUsuari		IN		Usuaris.IdUsuari%TYPE:=Null,
	p_Usuari		IN		VARCHAR2:=Null,
	p_InclouBaixes	IN		NUMBER:=0
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		'p_RS (IN OUT), p_IdUsuari: ' || p_IdUsuari || 
		' p_Usuari:' || p_Usuari || 
		' p_InclouBaixes:' || p_InclouBaixes;

	IF p_IdUsuari IS NULL AND p_Usuari IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Usuaris WHERE
			CASE WHEN p_InclouBaixes=1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
	ELSIF p_IdUsuari IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Usuaris WHERE IdUsuari=p_IdUsuari AND
			CASE WHEN p_InclouBaixes=1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
	ELSIF p_Usuari IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Usuaris 
			WHERE UPPER(Nom || ' ' || Cognoms ) LIKE '%' || UPPER(p_Usuari) || '%' AND
			CASE WHEN p_InclouBaixes=1 THEN 1 ELSE CASE WHEN 
			DataBaixa IS NULL THEN 1 ELSE 0 END END = 1 ;
	END IF;

	v_Res:= 'OK';
    
	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
				-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
      	END IF;

      DBMS_OUTPUT.put_line (v_Res);
      v_Fi := SYSDATE;
      v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);    
END;
/




-------------------------------------------------------------------------------
-- Permisos
-------------------------------------------------------------------------------


CREATE OR REPLACE PROCEDURE Alta_Permis (
/******************************************************************************
	NOM:			Alta_Permis 

	REQUERIMENT N:	6

	DESCRIPCI:		Dna dalta un perms al nostre sistema

	PARMETROS:

		p_Permis	Nom del perms

******************************************************************************/
	p_Permis	IN	Permisos.Permis%TYPE)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_Permis: ' || p_Permis;

	INSERT INTO Permisos (Permis)
	VALUES (p_Permis);

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := 'OK';
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El Perms ' || p_Permis || ' ja existeix';
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Modificar_Permis (
/******************************************************************************
	NOM:			Modificar_Permis
   
	REQUERIMENT N:	7
   
	DESCRIPCI:		Permet modificar les dades associades a un perms del nostre sistema
                 
	PARMETRES:

		p_IdPermis	Identificador del perms
		p_Permis	Nom del perms

******************************************************************************/
	p_IdPermis	IN	Permisos.IdPermis%TYPE,
	p_Permis	IN	Permisos.Permis%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdPermis: ' ||
		p_IdPermis ||
		' p_Permis: ' ||
		p_Permis;

	UPDATE Permisos
	SET Permis = p_Permis
	WHERE IdPermis = p_IdPermis;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el perms amb IdPermis=' || p_IdPermis;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'El perms ' || p_Permis || ' ja existeix';             
        ELSIF SQLCODE = -1407
        THEN
            v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

      DBMS_OUTPUT.put_line (v_Res);
      v_Fi := SYSDATE;
      v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Eliminar_Permis (
/******************************************************************************
	NOM:			Eliminar_Permis 
                 
	REQUERIMENT N:	8

	DESCRIPCI:		Elimina un perms del nostre sistema

	PARMETRES:

		p_IdPermis	Identificador del perms

******************************************************************************/
	p_IdPermis	IN	Permisos.IdPermis%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdPermis: ' || p_IdPermis;

	DELETE Permisos WHERE IdPermis = p_IdPermis;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix el perms amb IdPermis=' || p_IdPermis;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE =  -2292 
		THEN
			-- No es pot eliminar perqu t usuaris associats
			v_Res := 'No es pot eliminar el perms ' || p_IdPermis || ' perqu t usuaris associats';
		END IF;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Obtenir_Permis
/******************************************************************************
	NOM:			Obtenir_Permis

	REQUERIMENT N:	9

	DESCRIPCI:		Permet obtenir una relaci de permisos, en funci dalguns parmetres. 
				Si especifiquem un identificador, ens retornar, si existeix, el perms 
				amb aquest identificador. Si no especifiquem lidentificador (null), 
				retornar els permisos el nom dels quals contingui el nom especificat per parmetre 

	PARMETRES:

		p_RS		Recordset amb el resultat
		p_IdPermis	IdPermis a cercar
		p_Permis	Permis a cercar

******************************************************************************/

(   
	p_RS		IN OUT	sys_refcursor,
	p_IdPermis	IN		Permisos.IdPermis%TYPE:=Null,
	p_Permis	IN		Permisos.Permis%TYPE:=Null
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := 'p_RS (IN OUT), p_IdPermis: ' || p_IdPermis || ' p_Permis:' || p_Permis ;

	IF p_IdPermis IS NULL AND p_Permis IS NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Permisos;
	ELSIF p_IdPermis IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Permisos Where IdPermis = p_IdPermis;
	ELSIF p_Permis IS NOT NULL THEN
		OPEN p_RS FOR
			SELECT * FROM Permisos WHERE UPPER(Permis) LIKE '%' || UPPER(p_Permis) || '%';
	END IF;

	v_Res:= 'OK';

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;




-------------------------------------------------------------------------------
-- Permisos Usuaris
-------------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE Donar_Permis_Usuari (
/******************************************************************************
	NOM:			Donar_Permis_Usuari
   
	REQUERIMENT N:	10

	DESCRIPCI:		Permet assignar un determinat perms a un determinat usuari

	PARMETRES:

		p_IdPermis	Identificador del perms
		p_IdUsuari	Identificador de l'usuari

******************************************************************************/
	p_IdPermis	IN	PermisosUsuaris.IdPermis%TYPE,
	p_IdUsuari	IN	PermisosUsuaris.IdUsuari%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada :=
		' p_IdPermis: ' || 
		p_IdPermis || ' 
		p_IdUsuari: ' || 
		p_IdUsuari;

	INSERT INTO PermisosUsuaris (IdUsuari, IdPermis)
	VALUES (p_IdUsuari, p_IdPermis);

	-- Log
	v_Fi := SYSDATE;
	v_Res := 'OK';
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1
		THEN
			v_Res := 'Usuari ' || p_IdUsuari || ' ja t el perms ' || p_IdPermis;
		ELSIF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		ELSIF SQLCODE = -2291
		THEN
			v_Res := 'No existeix usuari i/o perms';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;

CREATE OR REPLACE PROCEDURE Treure_Permis_Usuari (
/******************************************************************************
	NOM:			Treure_Permis_Usuari
   
	REQUERIMENT N:	11

	DESCRIPCI:		Permet treure un determinat perms a un determinat usuari

	PARMETRES:

		p_IdPermis	Identificador del perms
		p_IdUsuari	Identificador de l'usuari

******************************************************************************/
	p_IdPermis	IN	PermisosUsuaris.IdPermis%TYPE,
	p_IdUsuari	IN	PermisosUsuaris.IdUsuari%TYPE
)
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := ' p_IdPermis: ' || p_IdPermis || ' p_IdUsuari: ' || p_IdUsuari;

	DELETE PermisosUsuaris
	WHERE IdUsuari = p_IdUsuari AND IdPermis = p_IdPermis;

	IF SQL%NOTFOUND
	THEN
		v_Res := 'Error: No existeix assignaci perms='
			|| p_IdPermis
			|| ', usuari='
			|| p_IdUsuari;
	ELSE
		v_Res := 'OK';
	END IF;

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS
	THEN
		IF SQLCODE = -1400
		THEN
			v_Res := 'Falta introduir algun camp obligatori';
		END IF;

		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
END;


CREATE OR REPLACE PROCEDURE Obtenir_Permis_Usuari
/******************************************************************************
	NOM:			Obtenir_Permis_Usuari
   
	REQUERIMENTO N:	12
   
	DESCRIPCI:		Permet obtenir la relaci de permisos que tenen assignats els usuaris, 
				en funci dels identificadors passats com a parmetres.
				Si especifiquem els dos identificadors (usuari i perms), 
				obtindrem lassignaci corresponent si aquesta existeix.
				Si no especifiquem lidentificador de lusuari (null), obtindrem una 
				relaci dels usuaris que tinguin assignat el perms que correspon a lidentificador donat.
				Si no especifiquem lidentificador del perms (null), obtindrem una 
				relaci dels permisos que tingui assignat lusuari que correspon a lidentificador donat.
				Si no especifiquem cap dels dos identificadors (null), 
				obtindrem una relaci de totes les assignacions usuari  perms fetes

	PARMETROS:

		p_RS		Recordset amb el resultat
		p_IdPermis	Identificador del perms
		p_IdUsuari	Identificador de l'usuari

******************************************************************************/

(   

	p_RS			IN OUT	sys_refcursor,
	p_IdPermis		IN		PermisosUsuaris.IdPermis%TYPE:=NULL,
	p_IdUsuari		IN		PermisosUsuaris.IdUsuari%TYPE:=NULL
    ) 
IS
	v_Res			VARCHAR2 (150);
	v_Inici		LogExecucio.Inici%TYPE;
	v_Fi			LogExecucio.Fi%TYPE;
	v_ParamEntrada	LogExecucio.ParamEntrada%TYPE;
	v_ParamSortida	LogExecucio.ParamSortida%TYPE;
BEGIN
	-- Inici de l'execuci del procediment
	v_Inici := SYSDATE;
	v_ParamEntrada := '  p_RS (IN OUT) , p_IdPermis: ' || p_IdPermis || 'p_IdUsuari: ' || p_IdUsuari;

	OPEN p_RS FOR
		SELECT U.*, P.IdPermis, P.Permis FROM PermisosUsuaris UP, Usuaris U, Permisos P
		WHERE U.DataBaixa IS NULL AND UP.IdUsuari=U.IdUsuari and UP.IdPermis=P.IdPermis AND 
		CASE WHEN p_IdPermis IS NULL THEN 1 ELSE CASE WHEN 
			UP.IdPermis = p_IdPermis THEN 1 ELSE 0 END END = 1 AND   
		CASE WHEN p_IdUsuari Is Null THEN 1 ELSE CASE WHEN 
			UP.IdUsuari = p_IdUsuari THEN 1 ELSE 0 END END = 1 ;
    
	v_Res:= 'OK';

	-- Log
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);
EXCEPTION
	WHEN OTHERS   
	THEN
		-- No retorna cap registre
		OPEN p_RS FOR SELECT * FROM dual WHERE dummy IS NULL;
   
		IF v_Res IS NULL
		THEN
			-- Error no controlat per codi
			v_Res := 'Error:' || SQLCODE || SUBSTR (SQLERRM, 1, 100);
		ELSE
			-- Error controlat per codi
			v_Res := 'Error:' || v_Res;
		END IF;

	DBMS_OUTPUT.put_line (v_Res);
	v_Fi := SYSDATE;
	v_ParamSortida := v_Res;
	Grabar_LogExecucio (v_Inici, v_Fi, $$plsql_unit, v_ParamEntrada, v_ParamSortida);   
END;
/

COMMIT;
