/*---------------------------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Tests sobre el subsistema Agents
		S'ha d'haver fet la crrega inicial de dades
----------------------------------------------------------------------*/ 

-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

/******************************************************************************
	REQUERIMENT:	61

	Tests:
		Donar d'alta un perfil amb un nom existent
		Donar d'alta un perfil sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Perfil('Perfil_1');

EXEC Alta_Perfil(null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	62

	Tests:
		Modificar un perfil inexistent
		Modificar un perfil donat de baixa
		Modificar un perfil, posant-li un nom existent
		Modificar un perfil sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als quatre casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Perfil(50, 'Perfil_50');

EXEC Baixa_Perfil(3);

EXEC Modificar_Perfil(3, 'Perfil_100');

EXEC Modificar_Perfil(1, 'Perfil_2');

EXEC Modificar_Perfil(2, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	63

	Tests:
		Donar de baixa un perfil inexistent
		Donar de baixa un perfil donat de baixa prviament
		Donar de baixa un perfil, posant-li una data de baixa anterior a la d'alta

	RESULTAT ESPERAT:
		Les baixes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Baixa_Perfil(50);

EXEC Baixa_Perfil(3);

EXEC Baixa_Perfil(2, '10/01/2011');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	64

	Tests:
		Recuperar un perfil inexistent
		Recuperar un perfil que no est donat de baixa

	RESULTAT ESPERAT:
		Les recuperacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Recuperar_Perfil(50);

EXEC Recuperar_Perfil(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	65

	Tests:
		Obtenir un perfil mitjanant el seu Id
		Obtenir un perfil mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades del perfil correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS1 refcursor
BEGIN
    Obtenir_Perfil(:RS1, 5);
END;
PRINT RS1;

VARIABLE RS2 refcursor
BEGIN
    Obtenir_Perfil(:RS2, null, 'Perfil_1');
END;
PRINT RS2;

/******************************************************************************
	REQUERIMENT:	66

	Tests:
		Donar d'alta un agent sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Agent(null, 'Garca');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	67

	Tests:
		Modificar un agent inexistent
		Modificar un agent donat de baixa
		Modificar un agent sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Agent(50, 'Nom_1', 'Cognoms_1');

EXEC Baixa_Agent(3);

EXEC Modificar_Agent(3, 'Nom_100', 'Cognoms_100');

EXEC Modificar_Agent(2, null, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	68

	Tests:
		Donar de baixa un agent inexistent
		Donar de baixa un agent donat de baixa prviament
		Donar de baixa un agent , posant-li una data de baixa anterior a la d'alta

	RESULTAT ESPERAT:
		Les baixes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Baixa_Agent(50);

EXEC Baixa_Agent(3);

EXEC Baixa_Agent(2, '10/01/2011');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	69

	Tests:
		Recuperar un agent inexistent
		Recuperar un agent que no est donat de baixa

	RESULTAT ESPERAT:
		Les recuperacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Recuperar_Agent(50);

EXEC Recuperar_Agent(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	70, 71

	Tests:
		Obtenir un agent mitjanant el seu Id
		Obtenir un agent mitjanant el seu nom
		Obtenir un agent mitjanant la seva data d'alta

	RESULTAT ESPERAT:
		Ha de retornar les dades de l'agent correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS3 refcursor
BEGIN
    Obtenir_Agent(:RS3, 5);
END;
PRINT RS3;

VARIABLE RS4 refcursor
BEGIN
    Obtenir_Agent(:RS4, null, 'NomAgent_1');
END;
PRINT RS4;

VARIABLE RS5 refcursor
BEGIN
    Obtenir_Agent_Data(:RS5, 'A', '14/05/2011');
END;
PRINT RS5;

/******************************************************************************
	REQUERIMENT:	72

	Tests:
		Assignar un perfil inexistent
		Assignar un perfil a un agent inexistent
		Assignar un perfil a un agent, que ja el tenia assignat

	RESULTAT ESPERAT:
		Les assignacions no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Perfil_Agent(100, 1);

EXEC Alta_Perfil_Agent(1, 100);

EXEC Alta_Perfil_Agent(1, 1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	73

	Tests:
		Treiem una relaci perfil-agent inexistent

	RESULTAT ESPERAT:
		La operaci no s'ha de portar a terme
		S'ha de generar un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Eliminar_Perfil_Agent(100, 1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	74

	Tests:
		Obtenir els perfils d'un agent
		Obtenir els agents que tenen un determinat perfil
		Obtenir una relaci perfil-agent
		Obtenir totes les relacions perfil-agent

	RESULTAT ESPERAT:
		Ha de retornar les dades correctament
		S'ha de generar, als quatre casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS6 refcursor
BEGIN
    Obtenir_Perfil_Agent(:RS6, 1, null);
END;
PRINT RS6;

VARIABLE RS7 refcursor
BEGIN
    Obtenir_Perfil_Agent(:RS7, null, 1);
END;
PRINT RS7;

VARIABLE RS8 refcursor
BEGIN
    Obtenir_Perfil_Agent(:RS8, 1, 1);
END;
PRINT RS8;

VARIABLE RS9 refcursor
BEGIN
    Obtenir_Perfil_Agent(:RS9, null, null);
END;
PRINT RS9;

/******************************************************************************
	REQUERIMENT:	75

	Tests:
		Assignar una provncia inexistent
		Assignar una provncia a un agent inexistent
		Assignar una provncia a un agent, que ja la tenia assignada

	RESULTAT ESPERAT:
		Les assignacions no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Provincia_Agent(1000, 1);

EXEC Alta_Provincia_Agent(1, 100);

EXEC Alta_Provincia_Agent(1, 1);

EXEC Alta_Provincia_Agent(1, 1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	76

	Tests:
		Treiem una relaci provncia-agent inexistent

	RESULTAT ESPERAT:
		La operaci no s'ha de portar a terme
		S'ha de generar un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Eliminar_Provincia_Agent(1000, 1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	77

	Tests:
		Obtenir les provncies d'un agent
		Obtenir els agents que tenen una determinada provncia
		Obtenir una relaci provncia-agent
		Obtenir totes les relacions provncia-agent

	RESULTAT ESPERAT:
		Ha de retornar les dades correctament
		S'ha de generar, als quatre casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RSA refcursor
BEGIN
    Obtenir_Provincia_Agent(:RSA, 1, null);
END;
PRINT RSA;

VARIABLE RSB refcursor
BEGIN
    Obtenir_Provincia_Agent(:RSB, null, 1);
END;
PRINT RSB;

VARIABLE RSC refcursor
BEGIN
    Obtenir_Provincia_Agent(:RSC, 1, 1);
END;
PRINT RSC;

VARIABLE RSD refcursor
BEGIN
    Obtenir_Provincia_Agent(:RSD, null, null);
END;
PRINT RSD;

