/*---------------------------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Tests sobre el subsistema Auxiliars
		S'ha d'haver fet la crrega inicial de dades
----------------------------------------------------------------------*/ 

-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

/******************************************************************************
	REQUERIMENT:	13

	Tests:
		Donar d'alta un pas amb un nom existent
		Donar d'alta un pas sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Pais('Pais_1');

EXEC Alta_Pais(null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	14

	Tests:
		Modificar un pas inexistent
		Modificar un pas, posant-li un nom existent
		Modificar un pas sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Pais(50, 'Grecia');

EXEC Modificar_Pais(1, 'Pais_2');

EXEC Modificar_Pais(1, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	15

	Tests:
		Donar de baixa un pas inexistent
		Donar de baixa un pas amb provncies associades

	RESULTAT ESPERAT:
		Les baixes no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Eliminar_Pais(50);

EXEC Eliminar_Pais(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	16

	Tests:
		Obtenir un pas mitjanant el seu Id
		Obtenir un pas mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades del pas correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS1 refcursor
BEGIN
    Obtenir_Pais(:RS1, 5);
END;
PRINT RS1;

VARIABLE RS2 refcursor
BEGIN
    Obtenir_Pais(:RS2, null, 'Pais_3');
END;
PRINT RS2;

/******************************************************************************
	REQUERIMENT:	17

	Tests:
		Donar d'alta una provncia amb un nom existent
		Donar d'alta una provncia amb un pas inexistent
		Donar d'alta una provncia sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Provincia('Provincia_1:Pais_1', 1);

EXEC Alta_Provincia('Tarragona', 50);

EXEC Alta_Provincia('Tarragona', null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	18

	Tests:
		Modificar una provncia inexistent
		Modificar una provncia , posant-li un nom existent
		Modificar una provncia , posant-li un pas inexistent
		Modificar una provncia sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als tres primers casos, un registre amb l'error a la taula LogExecucio
		A l'ltim cas, ha de mantenir el valor anterior de pas
******************************************************************************/

EXEC Modificar_Provincia(100, 'Provincia_100:Pais_1');

EXEC Modificar_Provincia(2, 'Provincia_1:Pais_1');

EXEC Modificar_Provincia(1, 'Tarragona', 50);

EXEC Modificar_Provincia(1, 'Tarragona', null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	19

	Tests:
		Eliminar una provincia inexistent
		Eliminar una provncia amb localitats associades

	RESULTAT ESPERAT:
		Les eliminacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Eliminar_Provincia(100);

EXEC Eliminar_Provincia(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	20

	Tests:
		Obtenir una provncia mitjanant el seu Id
		Obtenir una provncia mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades de la provncia correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS3 refcursor
BEGIN
    Obtenir_Provincia(:RS3, 5);
END;
PRINT RS3;

VARIABLE RS4 refcursor
BEGIN
    Obtenir_Provincia(:RS4, null, 'Provincia_2:Pais_1');
END;
PRINT RS4;

/******************************************************************************
	REQUERIMENT:	21

	Tests:
		Donar d'alta una localitat amb un nom existent
		Donar d'alta una localitat amb una provncia inexistent
		Donar d'alta una localitat sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Localitat('Localitat_1:Provincia_1:Pais_1', 1);

EXEC Alta_Localitat('Reus', 50);

EXEC Alta_Localitat('Reus', null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	22

	Tests:
		Modificar una localitat inexistent
		Modificar una localitat , posant-li un nom existent
		Modificar una localitat , posant-li una provncia inexistent
		Modificar una localitat sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als tres primers casos, un registre amb l'error a la taula LogExecucio
		A l'ltim cas, ha de mantenir el valor anterior de provncia
******************************************************************************/

EXEC Modificar_Localitat(1000, 'Reus');

EXEC Modificar_Localitat(2, 'Localitat_1:Provincia_1:Pais_1');

EXEC Modificar_Localitat(1, 'Reus', 100);

EXEC Modificar_Localitat(1, 'Reus', null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	23

	Tests:
		Eliminar una localitat inexistent
		Eliminar una localitat amb cp associats

	RESULTAT ESPERAT:
		Les eliminacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Eliminar_Localitat(1000);

EXEC Eliminar_Localitat(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	24

	Tests:
		Obtenir una localitat mitjanant el seu Id
		Obtenir una localitat mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades de la localitat correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS5 refcursor
BEGIN
    Obtenir_Localitat(:RS5, 5);
END;
PRINT RS5;

VARIABLE RS6 refcursor
BEGIN
    Obtenir_Localitat(:RS6, null, 'Localitat_2:Provincia_1:Pais_1');
END;
PRINT RS6;

/******************************************************************************
	REQUERIMENT:	25

	Tests:
		Donar d'alta un CP amb un codi existent
		Donar d'alta un CP amb una localitat inexistent
		Donar d'alta un CP sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_CodiPostal(43201, 1);

EXEC Alta_CodiPostal(44556, 1000);

EXEC Alta_CodiPostal(44557, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	26

	Tests:
		Modificar un CP inexistent
		Modificar un CP , posant-li una localitat inexistent
		Modificar un CP sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_CodiPostal(1000, 5);

EXEC Modificar_CodiPostal(43201, 1000);

EXEC Modificar_CodiPostal(43201, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	27

	Tests:
		Eliminar un CP inexistent
		Eliminar un CP amb centres o clients associats

	RESULTAT ESPERAT:
		Les eliminacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Eliminar_CodiPostal(1000);

EXEC Eliminar_CodiPostal(43201);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	28

	Tests:
		Obtenir un CP mitjanant el seu codi

	RESULTAT ESPERAT:
		Ha de retornar les dades de la localitat correctament
		S'ha de generar un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS7 refcursor
BEGIN
    Obtenir_CodiPostal(:RS7, 43201);
END;
PRINT RS7;

/******************************************************************************
	REQUERIMENT:	29

	Tests:
		Obtenir les dades del log d'execuci

	RESULTAT ESPERAT:
		Ha de retornar les dades del log correctament
		S'ha de generar un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS8 refcursor
BEGIN
    Obtenir_LogExecucio(:RS8);
END;
PRINT RS8;

