/*---------------------------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Tests sobre el subsistema Clients
		S'ha d'haver fet la crrega inicial de dades
----------------------------------------------------------------------*/ 

-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

/******************************************************************************
	REQUERIMENT:	30

	Tests:
		Donar d'alta un client amb un dni existent
		Donar d'alta un client amb un CP inexistent
		Donar d'alta un client amb la data de naixement posterior a la d'alta
		Donar d'alta un client amb un sexe incorrecte
		Donar d'alta un client sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als cinc casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Client('Toni', 'Garca', '39890001', 43201, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/1975', 'H');

EXEC Alta_Client('Toni', 'Garca', '39899001', 13201, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/1975', 'H');

EXEC Alta_Client('Toni', 'Garca', '39899001', 43201, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/2011', 'H');

EXEC Alta_Client('Toni', 'Garca', '39899001', 43201, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/1975', 'M');

EXEC Alta_Client('Toni', 'Garca', '39899001', null, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/1975', 'H');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	31

	Tests:
		Modificar un client inexistent
		Modificar un client donat de baixa
		Modificar un client, posant-li un dni existent
		Modificar un client, posant-li un CP inexistent
		Modificar un client, posant-li la data de naixement posterior a la d'alta
		Modificar un client, posant-li un sexe incorrecte
		Modificar un pas sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als set casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Client(1000, 'Toni', 'Garca', '39890001', 43201, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/1975', 'H');

EXEC Baixa_Client(2);

EXEC Modificar_Client(2, 'Toni', 'Garca', '39890001', 43201, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/1975', 'H');

EXEC Modificar_Client(1, 'Toni', 'Garca', '39890005', 43201, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/1975', 'H');

EXEC Modificar_Client(1, 'Toni', 'Garca', '39899001', 13201, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/1975', 'H');

EXEC Modificar_Client(1, 'Toni', 'Garca', '39899001', 43201, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/2011', 'H');

EXEC Modificar_Client(1, 'Toni', 'Garca', '39899001', 43201, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/1975', 'M');

EXEC Modificar_Client(1, 'Toni', 'Garca', '39899001', null, 'Carrer 1', '977753661', '643577912', 'agarciaburg@uoc.edu', '15/10/1975', 'H');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	32

	Tests:
		Donar de baixa un client inexistent
		Donar de baixa un client donat de baixa prviament
		Donar de baica un client amb data de baixa anterior a la d'alta

	RESULTAT ESPERAT:
		Les baixes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Baixa_Client(1000);

EXEC Baixa_Client(2);

EXEC Baixa_Client(3, '10/05/2010');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	33

	Tests:
		Recuperar un client inexistent
		Recuperar un client que no est donat de baixa

	RESULTAT ESPERAT:
		Les recuperacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Recuperar_Client(1000);

EXEC Recuperar_Client(5);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	34, 35, 36

	Tests:
		Obtenir un client mitjanant el seu DNI
		Obtenir un client mitjanant el seu nom
		Obtenir un client mitjanant la data de baixa

	RESULTAT ESPERAT:
		Ha de retornar les dades del client correctament
		S'ha de generar, als tres casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS1 refcursor
BEGIN
    Obtenir_Client_DNI(:RS1, '39890001');
END;
PRINT RS1;

VARIABLE RS2 refcursor
BEGIN
    Obtenir_Client_Nom(:RS2, 'NomCli_5');
END;
PRINT RS2;

VARIABLE RS3 refcursor
BEGIN
    Obtenir_Client_Data(:RS3, 'B', '15/05/2011');
END;
PRINT RS3;

