/*---------------------------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Tests sobre el subsistema Control d'Accessos
		S'ha d'haver fet la crrega inicial de dades
----------------------------------------------------------------------*/ 

-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

/******************************************************************************
	REQUERIMENT:	1

	Tests:
		Donar d'alta un usuari amb un login existent
		Donar d'alta un usuari sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Usuari('Toni', 'Garca Burgueo', 'Usuari_1', 'agb');

EXEC Alta_Usuari('Toni', 'Garca Burgueo', 'Usuari_1', null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	2

	Tests:
		Modificar un usuari inexistent
		Modificar un usuari donat de baixa
		Modificar un usuari, posant-li un login existent
		Modificar un usuari sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als quatre casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Usuari(50, 'Toni', 'Garca Burgueo', 'Usuari_1', 'agb');

EXEC Baixa_Usuari(1);

EXEC Modificar_Usuari(1, 'Toni', 'Garca Burgueo', 'Usuari_1', 'agb');

EXEC Modificar_Usuari(2, 'Toni', 'Garca Burgueo', 'Usuari_3', 'agb');

EXEC Modificar_Usuari(2, null, 'Garca Burgueo', 'Usuari_1', 'agb');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	3

	Tests:
		Donar de baixa un usuari inexistent
		Donar de baixa un usuari donat de baixa prviament
		Donar de baixa un usuari, posant-li una data de baixa anterior a la d'alta

	RESULTAT ESPERAT:
		Les baixes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Baixa_Usuari(50);

EXEC Baixa_Usuari(1);

EXEC Baixa_Usuari(2, '10/01/2011');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	4

	Tests:
		Recuperar un usuari inexistent
		Recuperar un usuari que no est donat de baixa

	RESULTAT ESPERAT:
		Les recuperacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Recuperar_Usuari(50);

EXEC Recuperar_Usuari(3);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	5

	Tests:
		Obtenir un usuari mitjanant el seu Id
		Obtenir un usuari mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades de l'usuari correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS1 refcursor
BEGIN
    Obtenir_Usuari(:RS1, 5);
END;
PRINT RS1;

VARIABLE RS2 refcursor
BEGIN
    Obtenir_Usuari(:RS2, null, 'Nom_3');
END;
PRINT RS2;

/******************************************************************************
	REQUERIMENT:	6

	Tests:
		Donar d'alta un perms amb un nom existent
		Donar d'alta un perms sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Permis('Perms_1');

EXEC Alta_Permis(null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	7

	Tests:
		Modificar un perms inexistent
		Modificar un perms, posant-li un nom existent
		Modificar un perms sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Permis(100, 'Perms_100');

EXEC Modificar_Permis(1, 'Perms_2');

EXEC Modificar_Permis(1, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	8

	Tests:
		Eliminar un perms inexistent
		Eliminar un perms amb usuaris associats

	RESULTAT ESPERAT:
		Les eliminacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Eliminar_Permis(100);

EXEC Eliminar_Permis(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	9

	Tests:
		Obtenir un perms mitjanant el seu Id
		Obtenir un perms mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades del perms correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS3 refcursor
BEGIN
    Obtenir_Permis(:RS3, 5);
END;
PRINT RS3;

VARIABLE RS4 refcursor
BEGIN
    Obtenir_Permis(:RS4, null, 'Perms_3');
END;
PRINT RS4;

/******************************************************************************
	REQUERIMENT:	10

	Tests:
		Donar un perms inexistent
		Donar un perms a un usuari inexistent
		Donar un perms a un usuari, que ja el tenia donat

	RESULTAT ESPERAT:
		Les assignacions no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Donar_Permis_Usuari(100, 1);

EXEC Donar_Permis_Usuari(1, 100);

EXEC Donar_Permis_Usuari(1, 1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	11

	Tests:
		Treiem una relaci perms-usuari inexistent

	RESULTAT ESPERAT:
		La operaci no s'ha de portar a terme
		S'ha de generar un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Treure_Permis_Usuari(100, 1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	12

	Tests:
		Obtenir els permisos d'un usuari
		Obtenir els usuaris que tenen un determinat perms
		Obtenir una relaci perms-usuari
		Obtenir totes les relacions perms-usuari

	RESULTAT ESPERAT:
		Ha de retornar les dades correctament
		S'ha de generar, als quatre casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS5 refcursor
BEGIN
    Obtenir_Permis_Usuari(:RS5, 1, null);
END;
PRINT RS5;

VARIABLE RS6 refcursor
BEGIN
    Obtenir_Permis_Usuari(:RS6, null, 1);
END;
PRINT RS6;

VARIABLE RS7 refcursor
BEGIN
    Obtenir_Permis_Usuari(:RS7, 1, 1);
END;
PRINT RS7;

VARIABLE RS8 refcursor
BEGIN
    Obtenir_Permis_Usuari(:RS8, null, null);
END;
PRINT RS8;

