/*---------------------------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Tests sobre el subsistema Incidencies
		S'ha d'haver fet la crrega inicial de dades
----------------------------------------------------------------------*/ 

-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

/******************************************************************************
	REQUERIMENT:	78

	Tests:
		Donar d'alta un estat amb un nom existent
		Donar d'alta un estat sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Estat('Estat_1');

EXEC Alta_Estat(null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	79

	Tests:
		Modificar un estat inexistent
		Modificar un estat, posant-li un nom existent
		Modificar un estat sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Estat(10, 'Estat_10');

EXEC Modificar_Estat(1, 'Estat_2');

EXEC Modificar_Estat(1, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	80

	Tests:
		Eliminar un estat inexistent
		Eliminar un estat amb incidncies associades

	RESULTAT ESPERAT:
		Les eliminacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Eliminar_Estat(10);

EXEC Eliminar_Estat(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	81

	Tests:
		Obtenir un estat mitjanant el seu Id
		Obtenir un estat mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades de l'estat correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS1 refcursor
BEGIN
    Obtenir_Estat(:RS1, 2);
END;
PRINT RS1;

VARIABLE RS2 refcursor
BEGIN
    Obtenir_Estat(:RS2, null, 'Estat_3');
END;
PRINT RS2;

/******************************************************************************
	REQUERIMENT:	82

	Tests:
		Donar d'alta una prioritat amb un nom existent
		Donar d'alta una prioritat sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Prioritat('Prioritat_1');

EXEC Alta_Prioritat(null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	83

	Tests:
		Modificar una prioritat inexistent
		Modificar una prioritat, posant-li un nom existent
		Modificar una prioritat sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Prioritat(10, 'Prioritat_10');

EXEC Modificar_Prioritat(1, 'Prioritat_2');

EXEC Modificar_Prioritat(1, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	84

	Tests:
		Eliminar una prioritat inexistent
		Eliminar una prioritat amb incidncies associades

	RESULTAT ESPERAT:
		Les eliminacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Eliminar_Prioritat(10);

EXEC Eliminar_Prioritat(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	85

	Tests:
		Obtenir una prioritat mitjanant el seu Id
		Obtenir una prioritat mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades de la prioritat correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS3 refcursor
BEGIN
    Obtenir_Prioritat(:RS3, 2);
END;
PRINT RS3;

VARIABLE RS4 refcursor
BEGIN
    Obtenir_Prioritat(:RS4, null, 'Prioritat_3');
END;
PRINT RS4;

/******************************************************************************
	REQUERIMENT:	86

	Tests:
		Donar d'alta un tipus incidncia amb un nom existent
		Donar d'alta un tipus incidncia sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_TipusIncidencia('TipusIncidencia 1');

EXEC Alta_TipusIncidencia('TipusIncidencia 50', null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	87

	Tests:
		Modificar un tipus incidncia inexistent
		Modificar un tipus incidncia donat de baixa
		Modificar un tipus incidncia, posant-li un nom existent
		Modificar un tipus incidncia sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als quatre casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_TipusIncidencia(50, 'TipusIncidencia 50');

EXEC Baixa_TipusIncidencia(2);

EXEC Modificar_TipusIncidencia(2, 'TipusIncidencia 50');

EXEC Modificar_TipusIncidencia(3, 'TipusIncidencia 1');

EXEC Modificar_TipusIncidencia(1, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	88

	Tests:
		Donar de baixa un tipus incidncia inexistent
		Donar de baixa un tipus incidncia donat de baixa prviament
		Donar de baica un tipus incidncia amb data de baixa anterior a la d'alta

	RESULTAT ESPERAT:
		Les baixes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Baixa_TipusIncidencia(1000);

EXEC Baixa_TipusIncidencia(2);

EXEC Baixa_TipusIncidencia(1, '10/05/2010');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	89

	Tests:
		Recuperar un tipus incidncia inexistent
		Recuperar un tipus incidncia que no est donat de baixa

	RESULTAT ESPERAT:
		Les recuperacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Recuperar_TipusIncidencia(1000);

EXEC Recuperar_TipusIncidencia(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	90

	Tests:
		Obtenir un tipus incidncia mitjanant el seu Id
		Obtenir un tipus incidncia mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades del tipus incidncia correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS5 refcursor
BEGIN
    Obtenir_TipusIncidencia(:RS5, 1);
END;
PRINT RS5;

VARIABLE RS6 refcursor
BEGIN
    Obtenir_TipusIncidencia(:RS6, null, 'TipusIncidencia 2');
END;
PRINT RS6;

/******************************************************************************
	REQUERIMENT:	91

	Tests:
		Donar d'alta una incidncia amb un client inexistent
		Donar d'alta una incidncia amb un tipus d'incidncia inexistent
		Donar d'alta una incidncia amb un producte inexistent
		Donar d'alta una incidncia amb un agent inexistent
		Donar d'alta una incidncia amb un prioritat inexistent
		Donar d'alta una incidncia amb un estat inexistent
		Donar d'alta una incidncia sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als set casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Incidencia(1, 'Descripci 1', 1000, 1, 1, 1, 1);

EXEC Alta_Incidencia(50, 'Descripci 1', 1, 1, 1, 1, 1);

EXEC Alta_Incidencia(1, 'Descripci 1', 1, 1000, 1, 1, 1);

EXEC Alta_Incidencia(1, 'Descripci 1', 1, 1, 100, 1, 1);

EXEC Alta_Incidencia(1, 'Descripci 1', 1, 1, 1, 10, 1);

EXEC Alta_Incidencia(1, 'Descripci 1', 1, 1, 1, 1, 10);

EXEC Alta_Incidencia(1, null, 1, 1, 1, 1, 1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	92

	Tests:
		Modificar una incidncia inexistent
		Modificar una incidncia tancada
		Modificar una incidncia amb un client inexistent
		Modificar una incidncia amb un tipus d'incidncia inexistent
		Modificar una incidncia amb un producte inexistent
		Modificar una incidncia amb un agent inexistent
		Modificar una incidncia amb un prioritat inexistent
		Modificar una incidncia amb un estat inexistent
		Modificar una incidncia sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als vuit casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Incidencia(1000, 1, 'Descripci 1', 1, 1, 1, 1, 1, null, null, null);

UPDATE INCIDENCIES SET DataTancament='15/05/2011' WHERE IdIncidencia = 2;

COMMIT;

EXEC Modificar_Incidencia(2, 1, 'Descripci 1', 1, 1, 1, 1, 1, null, null, null);

EXEC Modificar_Incidencia(1, 1, 'Descripci 1', 1000, 1, 1, 1, 1, null, null, null);

EXEC Modificar_Incidencia(50, 1, 'Descripci 1', 1, 1, 1, 1, 1, null, null, null);

EXEC Modificar_Incidencia(1, 1, 'Descripci 1', 1, 1000, 1, 1, 1, null, null, null);

EXEC Modificar_Incidencia(1, 1, 'Descripci 1', 1, 1, 100, 1, 1, null, null, null);

EXEC Modificar_Incidencia(1, 1, 'Descripci 1', 1, 1, 1, 10, 1, null, null, null);

EXEC Modificar_Incidencia(1, 1, 'Descripci 1', 1, 1, 1, 1, 10, null, null, null);

EXEC Modificar_Incidencia(1, 1, null, 1, 1, 1, 1, 1, null, null, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	93, 94, 95, 96, 97, 98, 99

	Tests:
		Obtenir una incidncia mitjanant el seu tipus
		Obtenir una incidncia mitjanant la seva descripci
		Obtenir una incidncia mitjanant la data alta
		Obtenir una incidncia mitjanant la seva prioritat
		Obtenir una incidncia mitjanant el seu estat
		Obtenir una incidncia mitjanant el client
		Obtenir una incidncia mitjanant l'agent

	RESULTAT ESPERAT:
		Ha de retornar les dades de les incidncies correctament
		S'ha de generar, als set casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS7 refcursor
BEGIN
    Obtenir_Incidencia_TipusInc(:RS7, 1);
END;
PRINT RS7;

VARIABLE RS8 refcursor
BEGIN
    Obtenir_Incidencia_Desc(:RS8, 'Incidncia_2');
END;
PRINT RS8;

VARIABLE RS9 refcursor
BEGIN
    Obtenir_Incidencia_Data(:RS9, 'A', '05/01/2011');
END;
PRINT RS9;

VARIABLE RSA refcursor
BEGIN
    Obtenir_Incidencia_Prioritat(:RSA, 1);
END;
PRINT RSA;

VARIABLE RSB refcursor
BEGIN
    Obtenir_Incidencia_Estat(:RSB, 1);
END;
PRINT RSB;

VARIABLE RSC refcursor
BEGIN
    Obtenir_Incidencia_Client(:RSC, 1);
END;
PRINT RSC;

VARIABLE RSD refcursor
BEGIN
    Obtenir_Incidencia_Agent(:RSD, 1);
END;
PRINT RSD;

/******************************************************************************
	REQUERIMENT:	100

	Tests:
		Donar d'alta un tipus actuaci amb un nom existent
		Donar d'alta un tipus actuaci sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_TipusActuacio('TipusActuacio 1');

EXEC Alta_TipusActuacio('TipusActuacio 50', null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	101

	Tests:
		Modificar un tipus actuaci inexistent
		Modificar un tipus actuaci donat de baixa
		Modificar un tipus actuaci , posant-li un nom existent
		Modificar un tipus actuaci sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als quatre casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_TipusActuacio(50, 'TipusActuacio 50');

EXEC Baixa_TipusActuacio(2);

EXEC Modificar_TipusActuacio(2, 'TipusActuacio 50');

EXEC Modificar_TipusActuacio(3, 'TipusActuacio 1');

EXEC Modificar_TipusActuacio(1, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	102

	Tests:
		Donar de baixa un tipus actuaci inexistent
		Donar de baixa un tipus actuaci donat de baixa prviament
		Donar de baica un tipus actuaci amb data de baixa anterior a la d'alta

	RESULTAT ESPERAT:
		Les baixes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Baixa_TipusActuacio(1000);

EXEC Baixa_TipusActuacio(2);

EXEC Baixa_TipusActuacio(1, '10/05/2010');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	103

	Tests:
		Recuperar un tipus actuaci inexistent
		Recuperar un tipus actuaci que no est donat de baixa

	RESULTAT ESPERAT:
		Les recuperacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Recuperar_TipusActuacio(1000);

EXEC Recuperar_TipusActuacio(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	104

	Tests:
		Obtenir un tipus actuaci mitjanant el seu Id
		Obtenir un tipus actuaci mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades del tipus actuaci correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RSE refcursor
BEGIN
    Obtenir_TipusActuacio(:RSE, 1);
END;
PRINT RSE;

VARIABLE RSF refcursor
BEGIN
    Obtenir_TipusActuacio(:RSF, null, 'TipusActuacio 2');
END;
PRINT RSF;

/******************************************************************************
	REQUERIMENT:	105

	Tests:
		Donar d'alta una actuaci amb una incidncia inexistent
		Donar d'alta una actuaci amb un tipus d'actuaci inexistent
		Donar d'alta una actuaci amb un agent inexistent
		Donar d'alta una actuaci sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als quatre casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Actuacio(1, 1000, 1, 'Descripci 1');

EXEC Alta_Actuacio(50, 1, 1, 'Descripci 1');

EXEC Alta_Actuacio(1, 1, 50, 'Descripci 1');

EXEC Alta_Actuacio(1, 1, 1, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	106

	Tests:
		Modificar una actuaci inexistent
		Modificar una actuaci amb una incidncia inexistent
		Modificar una actuaci amb un tipus d'actuaci inexistent
		Modificar una actuaci amb un agent inexistent
		Modificar una actuaci sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als cinc casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Actuacio(1000, 1, 1000, 1, 'Descripci 1', null, null, null, null);

EXEC Modificar_Actuacio(1, 1, 1000, 1, 'Descripci 1', null, null, null, null);

EXEC Modificar_Actuacio(1, 50, 1, 1, 'Descripci 1', null, null, null, null);

EXEC Modificar_Actuacio(1, 1, 1, 50, 'Descripci 1', null, null, null, null);

EXEC Modificar_Actuacio(1, 1, 1, 1, null, null, null, null, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	107, 108, 109, 110, 111

	Tests:
		Obtenir una actuaci mitjanant el seu tipus
		Obtenir una actuaci mitjanant la seva descripci
		Obtenir una actuaci mitjanant la data inici
		Obtenir una actuaci mitjanant la seva incidncia
		Obtenir una actuaci mitjanant l'agent

	RESULTAT ESPERAT:
		Ha de retornar les dades de les actuacions correctament
		S'ha de generar, als set casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RSG refcursor
BEGIN
    Obtenir_Actuacio_TipusActuacio(:RSG, 1);
END;
PRINT RSG;

VARIABLE RSH refcursor
BEGIN
    Obtenir_Actuacio_Desc(:RSH, 'Actuaci_2');
END;
PRINT RSH;

VARIABLE RSI refcursor
BEGIN
    Obtenir_Actuacio_Data(:RSI, 'I', '02/01/2011');
END;
PRINT RSI;

VARIABLE RSJ refcursor
BEGIN
    Obtenir_Actuacio_Incidencia(:RSJ, 1);
END;
PRINT RSJ;

VARIABLE RSK refcursor
BEGIN
    Obtenir_Actuacio_Agent(:RSK, 1);
END;
PRINT RSK;


