/*---------------------------------------------------------------------
Data: 15/05/2011
Autor: Antoni Garca Burgueo
Base de Dades: Oracle 10g
Descripci: Tests sobre el subsistema Productes
		S'ha d'haver fet la crrega inicial de dades
----------------------------------------------------------------------*/ 

-- Schema BD_POSTVENDA
ALTER SESSION SET CURRENT_SCHEMA = BD_POSTVENDA;

/******************************************************************************
	REQUERIMENT:	37

	Tests:
		Donar d'alta un centre amb un nom existent
		Donar d'alta un centre amb un CP inexistent
		Donar d'alta un centre sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Centre('Centre_1', 'Carrer 1', 43201, '977771253', 'Josep Cases');

EXEC Alta_Centre('Centre_3', 'Carrer 1', 13201, '977771253', 'Josep Cases');

EXEC Alta_Centre('Centre_1', null, 43201, '977771253', 'Josep Cases');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	38

	Tests:
		Modificar un centre inexistent
		Modificar un centre, posant-li un nom existent
		Modificar un centre donat de baixa
		Modificar un centre, posant-li un CP inexistent
		Modificar un centre sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als cinc casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Centre(5, 'Centre_1', 'Carrer 1', 43201, '977771253', 'Josep Cases');

EXEC Modificar_Centre(1, 'Centre_2', 'Carrer 1', 43201, '977771253', 'Josep Cases');

EXEC Baixa_Client(2);

EXEC Modificar_Centre(2, 'Centre_9', 'Carrer 1', 43201, '977771253', 'Josep Cases');

EXEC Modificar_Centre(1, 'Centre_3', 'Carrer 1', 13201, '977771253', 'Josep Cases');

EXEC Modificar_Centre(1, 'Centre_1', null, 43201, '977771253', 'Josep Cases');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	39

	Tests:
		Donar de baixa un centre inexistent
		Donar de baixa un centre donat de baixa prviament
		Donar de baica un centre amb data de baixa anterior a la d'alta

	RESULTAT ESPERAT:
		Les baixes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Baixa_Centre(1000);

EXEC Baixa_Centre(2);

EXEC Baixa_Centre(1, '10/05/2010');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	40

	Tests:
		Recuperar un centre inexistent
		Recuperar un centre que no est donat de baixa

	RESULTAT ESPERAT:
		Les recuperacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Recuperar_Centre(1000);

EXEC Recuperar_Centre(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	41

	Tests:
		Obtenir un centre mitjanant el seu Id
		Obtenir un centre mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades del centre correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS1 refcursor
BEGIN
    Obtenir_Centre(:RS1, 1);
END;
PRINT RS1;

VARIABLE RS2 refcursor
BEGIN
    Obtenir_Centre(:RS2, null, 'Centre_2');
END;
PRINT RS2;

/******************************************************************************
	REQUERIMENT:	42

	Tests:
		Donar d'alta una garantia amb un nom existent
		Donar d'alta una garantia amb anys_insitu > anys_total
		Donar d'alta una garantia sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Garantia('Garantia_1', 3, 1);

EXEC Alta_Garantia('Garantia_5', 3, 4);

EXEC Alta_Garantia(null, 1, 1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	43

	Tests:
		Modificar una garantia inexistent
		Modificar una garantia, posant-li un nom existent
		Modificar una garantia sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Garantia(10, 'Garantia_10', 1, null);

EXEC Modificar_Garantia(1, 'Garantia_3', 1, null);

EXEC Modificar_Garantia(1, null, 1, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	44

	Tests:
		Eliminar una garantia inexistent
		Eliminar una garantia amb productes associats

	RESULTAT ESPERAT:
		Les eliminacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Eliminar_Garantia(100);

EXEC Eliminar_Garantia(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	45

	Tests:
		Obtenir una garantia mitjanant el seu Id
		Obtenir una garantia mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades de la garantia correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS3 refcursor
BEGIN
    Obtenir_Garantia(:RS3, 1);
END;
PRINT RS3;

VARIABLE RS4 refcursor
BEGIN
    Obtenir_Garantia(:RS4, null, 'Garantia_3');
END;
PRINT RS4;

/******************************************************************************
	REQUERIMENT:	46

	Tests:
		Donar d'alta un tipus de producte amb un nom existent
		Donar d'alta un tipus de producte amb un pare inexistent
		Donar d'alta un tipus de producte sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_TipusProducte('TipusProducte 1', null);

EXEC Alta_TipusProducte('TipusProducte 100', 200);

EXEC Alta_TipusProducte(null, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	47

	Tests:
		Modificar un tipus de producte inexistent
		Modificar un tipus de producte donat de baixa
		Modificar un tipus de producte, posant-li un nom existent
		Modificar un tipus de producte, posant-li un pare inexistent
		Modificar un tipus de producte sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als cinc casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_TipusProducte(50, 'TipusProducte 1', null);

EXEC Baixa_TipusProducte(2);

EXEC Modificar_TipusProducte(2, 'Tipus 10', null);

EXEC Modificar_TipusProducte(1, 'TipusProducte 3', null);

EXEC Modificar_TipusProducte(1, 'TipusProducte 1', 100);

EXEC Modificar_TipusProducte(1, null, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	48

	Tests:
		Donar de baixa un tipus de producte inexistent
		Donar de baixa un tipus de producte donat de baixa prviament
		Donar de baixa un tipus de producte amb data de baixa anterior a la d'alta

	RESULTAT ESPERAT:
		Les baixes no s'han de portar a terme
		S'ha de generar, als tres casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Baixa_TipusProducte(1000);

EXEC Baixa_TipusProducte(2);

EXEC Baixa_TipusProducte(1, '10/05/2010');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	49

	Tests:
		Recuperar un tipus de producte inexistent
		Recuperar un tipus de producte que no est donat de baixa

	RESULTAT ESPERAT:
		Les recuperacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Recuperar_TipusProducte(25);

EXEC Recuperar_TipusProducte(3);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	50

	Tests:
		Obtenir un tipus de producte mitjanant el seu Id
		Obtenir un tipus de producte mitjanant el seu nom

	RESULTAT ESPERAT:
		Ha de retornar les dades del tipus de producte correctament
		S'ha de generar, als dos casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS5 refcursor
BEGIN
    Obtenir_TipusProducte(:RS5, 3);
END;
PRINT RS5;

VARIABLE RS6 refcursor
BEGIN
    Obtenir_TipusProducte(:RS6, null, 'TipusProducte_4');
END;
PRINT RS6;

/******************************************************************************
	REQUERIMENT:	51

	Tests:
		Donar d'alta un producte amb un ISBN existent
		Donar d'alta un producte amb un centre inexistent
		Donar d'alta un producte amb un tipus de producte inexistent
		Donar d'alta un producte amb una garantia inexistent
		Donar d'alta un producte amb data venda > data alta
		Donar d'alta un producte sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les altes no s'han de portar a terme
		S'ha de generar, als sis casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Alta_Producte(3, 'TV Sony 19881', '0000000001', '432677771', '01/02/2011', 1, 1, null, null, null);

EXEC Alta_Producte(3, 'TV Sony 19881', '0001200001', '432677771', '01/02/2011', 5, 1, null, null, null);

EXEC Alta_Producte(60, 'TV Sony 19881', '0001200001', '432677771', '01/02/2011', 1, 1, null, null, null);

EXEC Alta_Producte(3, 'TV Sony 19881', '0001200001', '432677771', '01/02/2011', 1, 10, null, null, null);

EXEC Alta_Producte(3, 'TV Sony 19881', '0001200001', '432677771', '01/06/2011', 1, 1, null, null, null);

EXEC Alta_Producte(3, 'TV Sony 19881', '0001200001', null, '01/02/2011', 1, 1, null, null, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	52

	Tests:
		Modificar un producte inexistent
		Modificar un producte donat de baixa
		Modificar un producte , posant-li un ISBN existent
		Modificar un producte , posant-li un centre inexistent
		Modificar un producte , posant-li un tipus de producte inexistent
		Modificar un producte , posant-li una garantia inexistent
		Modificar un producte , posant-li data venda > data alta
		Modificar un producte sense introduir tots els camps obligatoris

	RESULTAT ESPERAT:
		Les modificacions no s'han de portar a terme
		S'ha de generar, als vuit casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Modificar_Producte(1000, 3, 'TV Sony 19881', '0001200001', '432677771', '01/02/2011', 1, 1, null, null, null);

EXEC Baixa_Producte(2);

EXEC Modificar_Producte(2, 3, 'TV Sony 19881', '0001200001', '432677771', '01/02/2011', 1, 1, null, null, null);

EXEC Modificar_Producte(3, 3, 'TV Sony 19881', '0000000001', '432677771', '01/02/2011', 1, 1, null, null, null);

EXEC Modificar_Producte(1, 3, 'TV Sony 19881', '0001200001', '432677771', '01/02/2011', 5, 1, null, null, null);

EXEC Modificar_Producte(1, 60, 'TV Sony 19881', '0001200001', '432677771', '01/02/2011', 1, 1, null, null, null);

EXEC Modificar_Producte(1, 3, 'TV Sony 19881', '0001200001', '432677771', '01/02/2011', 1, 10, null, null, null);

EXEC Modificar_Producte(1, 3, 'TV Sony 19881', '0001200001', '432677771', '01/06/2011', 1, 1, null, null, null);

EXEC Modificar_Producte(1, 3, 'TV Sony 19881', '0001200001', null, '01/02/2011', 1, 1, null, null, null);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	53

	Tests:
		Donar de baixa un producte inexistent
		Donar de baixa un producte donat de baixa prviament
		Donar de baica un producte amb data de baixa anterior a la d'alta

	RESULTAT ESPERAT:
		Les baixes no s'han de portar a terme
		S'ha de generar, als 3 casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Baixa_Producte(1000);

EXEC Baixa_Producte(2);

EXEC Baixa_Producte(1, '10/05/2010');

COMMIT;      

/******************************************************************************
	REQUERIMENT:	54

	Tests:
		Recuperar un producte inexistent
		Recuperar un producte que no est donat de baixa

	RESULTAT ESPERAT:
		Les recuperacions no s'han de portar a terme
		S'ha de generar, als dos casos, un registre amb l'error a la taula LogExecucio
******************************************************************************/

EXEC Recuperar_Producte(1000);

EXEC Recuperar_Producte(1);

COMMIT;      

/******************************************************************************
	REQUERIMENT:	55, 56, 57, 58, 59, 60

	Tests:
		Obtenir un producte mitjanant el seu tipus
		Obtenir un producte mitjanant la seva descripci
		Obtenir un producte mitjanant la data alta
		Obtenir un producte mitjanant el seu ISBN
		Obtenir un producte mitjanant el seu centre
		Obtenir un producte mitjanant la seva garantia

	RESULTAT ESPERAT:
		Ha de retornar les dades del producte correctament
		S'ha de generar, als sis casos, un registre correcte a la taula LogExecucio
******************************************************************************/

VARIABLE RS7 refcursor
BEGIN
    Obtenir_Producte_TipusProducte(:RS7, 1);
END;
PRINT RS7;

VARIABLE RS8 refcursor
BEGIN
    Obtenir_Producte_Desc(:RS8, 'Producte_2');
END;
PRINT RS8;

VARIABLE RS9 refcursor
BEGIN
    Obtenir_Producte_Data(:RS9, 'A', '14/05/2011');
END;
PRINT RS9;

VARIABLE RSA refcursor
BEGIN
    Obtenir_Producte_ISBN(:RSA, '0000000001');
END;
PRINT RSA;

VARIABLE RSB refcursor
BEGIN
    Obtenir_Producte_Centre(:RSB, 1);
END;
PRINT RSB;

VARIABLE RSC refcursor
BEGIN
    Obtenir_Producte_Garantia(:RSC, 1);
END;
PRINT RSC;

